/* -----------------------------------------------------------------
 * (c) Teradyne, Inc, 2007
 * All Rights Reserved
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------
 * CodeGenEngine.cs file acts as an interface through which
 * we can collect the generated code from the instrument plug-ins and
 * used to format the code string as per the code setting options
 * ------------------------------------------------------------------
 */

/*
 **************************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 *   =============================================================================================
 *   Palaniappa Babu 09/05/07    CodeGen Engine Implementation
 *   Palaniappa Babu 09/15/07    Addressed IDC Review comments
 *   Palaniappa Babu 09/20/07    Fixed tersw00111476
 **************************************************************************************************
 */

using System;
using System.Collections.Generic;
using System.Text;
using PARAMETERCOLLECTIONINTERFACELib;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    /// <summary>
    /// 
    /// </summary>
    public class CodeGenEngine
    {
        private const string NOTE_WITH_BLOCK = "\n\'Note: Uncommenting a With block in a running function will\n\'reset (stop) the VBA project.\n\t";
        private const string CAUTION_STMT = "'Generated code should be examined for accuracy and safety.";

       /// <summary>
        /// This method takes as an argument the NSPCP item object containing the definition 
        /// of a PSet/Signal and then invokes the code generation method in the appropriate 
        /// instrument plugin of the IParameterCollection interface. It then receives back the 
        /// generated code in a string and returns back the code string. 
       /// </summary>
       /// <param name="obj"></param>
       /// <param name="pColl"></param>
       /// <returns></returns>
        public String GenerateCode(NspcpItem obj, IParameterCollection pColl, ref List<String> errInfo )
        {
            String ret_strCode;
            ITCOBSTRs outErrorInfo;
            ITCOBSTRs outErrorNames;
            ret_strCode = null;

            outErrorInfo = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());
            outErrorNames = (ITCOBSTRs)(new TERCOMMONOBJLib.TCOBSTRs());

            try
            {
                if (obj != null && pColl != null)
                    pColl.GetVBTCode(obj.propName.ToString(), obj.pinName.ToString(), ref ret_strCode, ref outErrorNames, ref outErrorInfo);
                
                if (outErrorInfo.Count > 0)
                {
                    errInfo = CreateListfromBSTR((TERCOMMONOBJLib.ITCOBSTRs)outErrorInfo);
                }

                return ret_strCode;
            }
            catch (Exception e)
            {
                throw new Exception("Error receiving VBT Code from plugin " + e.Message);
            }
        }

        /// <summary>
        /// This method takes as an argument of strCode (ie.,Code generated for all the nspcp
        /// item - PSet/Signal) and based on the CodeSettings, the code is formatted for
        /// With block and new language.
        /// </summary>
        /// <param name="strCode">input data, which need to be compress</param>
        /// <param name="bNeedWithBlock">boolean used to find whether the Withblock need or not</param>
        /// <param name="bNewLang">boolean used to find whether the Newlanguage</param>
        /// <returns>return the PSets/Signals output data</returns>
        public String FormatVBTCode(String strCode, Boolean bNeedWithBlock, Boolean bNewLang)
        {
            if (String.IsNullOrEmpty(strCode))
                return null;
            String ret_finalCode = string.Empty;

            CVBTCodeCompressor vbtComp = new CVBTCodeCompressor();
            CVBTCodeGenWithOption VBTCodeGenWithOption = new CVBTCodeGenWithOption();
            //Checks whether NewLang
            if (bNewLang)
            {
                if (bNeedWithBlock)
                {
                    ret_finalCode = vbtComp.Compress(strCode);
                    ret_finalCode = VBTCodeGenWithOption.WithNewLanguageVBTCode(ret_finalCode);
                }
                else
                    ret_finalCode = VBTCodeGenWithOption.WithoutNewLanguageVBTCode(strCode);
            }
            else
            {
                if (bNeedWithBlock)
                {
                    ret_finalCode = vbtComp.Compress(strCode);
                    ret_finalCode = VBTCodeGenWithOption.AddWithBlockComment(ret_finalCode);
                    ret_finalCode = VBTCodeGenWithOption.GetCommentedOutput(ret_finalCode);
                }
                else
                    ret_finalCode = strCode;
            }
            
            return ret_finalCode;
        }

        /// <summary>
        /// This method converts an ITCOBSTR into a string.
        /// </summary>
        /// <returns></returns>
        private static List<String> CreateListfromBSTR(TERCOMMONOBJLib.ITCOBSTRs str)
        {
            List<String> strList;
            strList = new List<String>();
            if (str != null)
            {
                foreach (string s in str)
                {
                    strList.Add(s);
                }
            }
            else
            {
                throw new ArgumentNullException("Null Arguments passed");
            }
            return strList;
        }


    }
}
