// ------------------------------------------------------------------
// (c) Teradyne, Inc, 2006
//     All Rights Reserved
// Inclusion of a copyright notice does not imply that this software
// has been published.  This software is the trade secret information
// of Teradyne, Inc.  Use of this software is only in accordance with
// the terms of a license agreement from Teradyne, Inc.
// ------------------------------------------------------------------
// IUndoContext.cs - Interface to store the state information and the associated observer
// *************************************************************************************/
//   Revision history:
//   Author         Date        Description of change
//   =================================================================
//   shoban         9/4/2007    Created
// *************************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

namespace Teradyne.Igxl.Utilities
{
    /// <summary>
    /// Represents an undo context. Undo context may store a single undo information or
    /// composite a group of undo data.
    /// </summary>
    interface IUndoContext
    {
        /// <summary>
        /// Request to set the system to the current state. The implementation should notify
        /// the respective observer(s) to set the system to the specified state
        /// </summary>
        /// <param name="isUndoing">Indicates if the operation is for undo or redo</param>
        /// <returns>true when all states have been set. false if any of the set 
        /// request failed</returns>
        bool SetState(bool isUndoing);

        /// <summary>
        /// Returns the parent context in case nested undo context are used. 
        /// This can return null also
        /// </summary>
        IUndoContext Parent { get; }

        /// <summary>
        /// Returns the number of undo information maintained. This will be used by
        /// the CompositeUndoContext to return the number of undo operations composed. 
        /// 
        /// UndoContext will default to 1
        /// </summary>
        int Count { get; }
    }
}
