// ------------------------------------------------------------------
// (c) Teradyne, Inc, 2006
//     All Rights Reserved
// Inclusion of a copyright notice does not imply that this software
// has been published.  This software is the trade secret information
// of Teradyne, Inc.  Use of this software is only in accordance with
// the terms of a license agreement from Teradyne, Inc.
// ------------------------------------------------------------------
// UndoContext.cs - Stores the undo information along with the observer. Used by
//   the undo manager to resotre the state at some point of time.
// *************************************************************************************/
//   Revision history:
//   Author         Date        Description of change
//   =================================================================
//   shoban         9/6/2007    Created
//   shoban         9/7/2007    Added support for count
// *************************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

namespace Teradyne.Igxl.Utilities
{
    class UndoContext : IUndoContext
    {
        private IUndoable m_handler;
        private Object m_undoData;

        /// <summary>
        /// Represents an undo context which documents the object 
        /// to notify when a request to update the state is raised.
        /// </summary>
        /// <param name="handler">The observer who should be notified to update the state</param>
        /// <param name="undoData">The state to set</param>
        public UndoContext(IUndoable handler, Object undoData)
        {
            if (handler == null)
                throw new ArgumentNullException("handler", "Require a valid non-null IUndoable handler");

            m_handler = handler;
            m_undoData = undoData;
        }

        public bool SetState(bool isUndoing)
        {
            return m_handler.ResetState(m_undoData, isUndoing);
        }

        public IUndoContext Parent
        {
            get { return null; }
        }

        public int Count
        {
            get { return 1; }
        }
    }
}
