
/* -----------------------------------------------------------------
 * (c) Teradyne, Inc, 2007
 * All Rights Reserved
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------------------
 * frmPSetsSignalsSelector.cs is responsible for providing a user interface for the
 * user to select one or more PSet/Signal definitions that can be later populated 
 * ------------------------------------------------------------------------------
 */
/*
 ***************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 *   =================================================================
 *  Nadeem Khan     8/12        Created.
 *  Kiran           9/11        Modified for Compare window( Constructor has been modified)
 *  Shoban Jayaraj  9/20        Added warning message when changes were made and discarded
 *  Maria Raykin    10/01       tersw00112034. Added message handlers: lstSelected_Enter and lstAvailable_Enter
 *  Andy Kittross   10/06/07    tersw00112001 - fix problems with move left/right buttons, esp. multiple selection
 *  Maria Raykin    10/08/07    tersw00112499 - fix left to right selection problem
 *  P.Babu 	    11/23/07	tersw00114456 - Modified btnCancel_Click.
 ****************************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Teradyne.Igxl.PSetSignalsEditor;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    public partial class frmPSetsSignalsSelector : Form
    {
        private List<String> m_nspcpList;
        private int MAX_ELEMENTS = 10;
        private Boolean m_MovingLeft = false;
        private List<String> m_OriginalSelectedList;
        private globalPSetSignalValues m_GlobalPSetSignalValues;

        public frmPSetsSignalsSelector(List<String> nlist,ref globalPSetSignalValues glblPSetSigVals)
        {
            InitializeComponent();
            if (nlist == null) throw new ArgumentNullException("List passed to the PSet/Signals Selector is null");
            m_nspcpList = nlist;
            if (glblPSetSigVals == null) throw new ArgumentNullException("Invalid Input for globalPSetSignalValues");
            m_GlobalPSetSignalValues = glblPSetSigVals;
        }
        
        private void frmPSetsSignalsSelector_Load(object sender, EventArgs e)
        {
            lstAvailable.SelectionMode = SelectionMode.MultiExtended;
            lstSelected.SelectionMode = SelectionMode.MultiExtended;

            //Populate the list with the items received.
            foreach (string nspcpName in m_nspcpList)
            {
                lstAvailable.Items.Add(nspcpName);
            }

            //If only one Available item, select it automatically (tersw00112043)
            if (lstAvailable.Items.Count == 1)
            {
                lstAvailable.SetSelected(0, true);
            }

            //If there are items already selected in existing view then move them
            //to the selected pan.
            System.Collections.ArrayList useArray = new System.Collections.ArrayList();

            useArray = m_GlobalPSetSignalValues.globalNspcpValues;
            m_OriginalSelectedList = null;
            m_OriginalSelectedList = new List<string>();
            m_OriginalSelectedList.Clear();
            if (useArray.Count > 0)
            {
                foreach (String nspcpName in useArray)
                {
                    lstAvailable.Items.Remove(nspcpName);
                    lstSelected.Items.Add(nspcpName);
                    m_OriginalSelectedList.Add(nspcpName);
                }
            }
        }

        
        private void lstAvailable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstAvailable.SelectedItems.Count > MAX_ELEMENTS)
            {
                MessageBox.Show("Please select no more than 10 items at a time", GlobalUtils.MESSAGEBOX_CAPTION);
            }             
            UpdateMoveButtons();
        }

        private void UpdateMoveButtons()
        {
            btnMoveUp.Enabled = lstSelected.SelectedItems.Count == 1 && lstSelected.SelectedIndices[0] != 0;
            btnMoveDown.Enabled = lstSelected.SelectedItems.Count == 1 && lstSelected.SelectedIndices[0] != lstSelected.Items.Count - 1;
            
            //determine if to enable the move right button
            btnMoveRight.Enabled = false;
            if (lstAvailable.Items.Count > 0)//there are available items
                if (lstAvailable.SelectedItems.Count > 0) //user has selected at least one item
                   if (lstAvailable.SelectedItems.Count <= MAX_ELEMENTS)//user has not selected more than max items
                        if (lstSelected.Items.Count + lstAvailable.SelectedItems.Count <= MAX_ELEMENTS)//user has not selected more than the max allowed
                            btnMoveRight.Enabled = true;
            
            //determine if to enable the move left button
            btnMoveLeft.Enabled = false;
            if (lstSelected.Items.Count > 0)//there are selected items
                if (lstSelected.SelectedItems.Count > 0)//user has selected at least one item
                    btnMoveLeft.Enabled = true;
        }

        private void lstSelected_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateMoveButtons();
        }

        private void btnMoveRight_Click(object sender, EventArgs e)
        {
            if (lstAvailable.SelectedItems.Count > 0)
            {
                while (lstAvailable.SelectedItems.Count > 0)
                {
                    lstSelected.Items.Add(lstAvailable.SelectedItem);
                    lstAvailable.Items.Remove(lstAvailable.SelectedItem);
                }
            }
            lstAvailable.ClearSelected();
            lstSelected.ClearSelected();
            UpdateMoveButtons();
        }

        private void btnMoveLeft_Click(object sender, EventArgs e)
        {
            m_MovingLeft = true;
            if (lstSelected.SelectedItems.Count > 0)
            {
                while (lstSelected.SelectedItems.Count > 0)
                {
                    lstAvailable.Items.Add(lstSelected.SelectedItem);
                    lstSelected.Items.Remove(lstSelected.SelectedItem);
                }
            }
            lstAvailable.ClearSelected();
            lstSelected.ClearSelected();
            UpdateMoveButtons();
            m_MovingLeft = false;
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            m_GlobalPSetSignalValues.globalNspcpValues.Clear();

            for (int nspcps = 0; nspcps < lstSelected.Items.Count; nspcps++)
            {
                String nspcp = lstSelected.Items[nspcps].ToString();
                m_GlobalPSetSignalValues.globalNspcpValues.Add(nspcp);
            }
            Close();
        }

        private void btnMoveUp_Click(object sender, EventArgs e)
        {
            int idx = lstSelected.SelectedIndices[0];
            Object it1 = lstSelected.Items[idx];
            Object it2 = lstSelected.Items[idx - 1];
            lstSelected.Items.RemoveAt(idx);
            lstSelected.Items.RemoveAt(idx - 1);
            lstSelected.Items.Insert(idx - 1,it1);
            lstSelected.Items.Insert(idx, it2);
            lstSelected.Focus();
            lstSelected.ClearSelected();
            lstSelected.SelectedItem = lstSelected.Items[idx - 1];
            
            // Selected = true will not trigger an event change. So raise one manually
            lstSelected_SelectedIndexChanged(null, null);
        }

        private void btnMoveDown_Click(object sender, EventArgs e)
        {
            // swap places           
            int idx = lstSelected.SelectedIndices[0];
            Object it1 = lstSelected.Items[idx];
            Object it2 = lstSelected.Items[idx + 1];
            lstSelected.Items.RemoveAt(idx + 1);
            lstSelected.Items.RemoveAt(idx);
            lstSelected.Items.Insert(idx, it2);
            lstSelected.Items.Insert(idx + 1, it1);
            lstSelected.Focus();
            lstSelected.ClearSelected();
            lstSelected.SelectedItem = lstSelected.Items[idx + 1];

            // Selected = true will not trigger an event change. So raise one manually
            lstSelected_SelectedIndexChanged(null, null);
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (m_OriginalSelectedList.Count != lstSelected.Items.Count)
            {
                if (MessageBox.Show(Properties.Resources.PSetSignalSelectorDiscardData, this.Text, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    Close();
                }
            }
            else
            {
                foreach (String nspcps in lstSelected.Items)
                {
                    if (!m_OriginalSelectedList.Exists(nspcps.Equals))
                    {
                        // warn the user again
                        if (MessageBox.Show(Properties.Resources.PSetSignalSelectorDiscardData, this.Text, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                        {
                            Close();
                        }
                    }
                }
                Close();
            }
        }
       

        private void lstAvailable_DoubleClick(object sender, EventArgs e)
        {
            if (lstAvailable.SelectedItems.Count > 0)
            {
                if (lstSelected.Items.Count >= MAX_ELEMENTS)
                {
                    MessageBox.Show("Please select no more than 10 items at a time", GlobalUtils.MESSAGEBOX_CAPTION);
                    return;
                }
               lstSelected.Items.Add(lstAvailable.SelectedItem);
               lstAvailable.Items.Remove(lstAvailable.SelectedItem);
            }                
            UpdateMoveButtons();
        }

        private void lstSelected_DoubleClick(object sender, EventArgs e)
        {
            if (lstSelected.SelectedItems.Count > 0)
            {               
                
                lstAvailable.Items.Add(lstSelected.SelectedItem);
                lstSelected.Items.Remove(lstSelected.SelectedItem);
            }
            UpdateMoveButtons();
        }
 

        // If List Control has focus and no List items are selected, select the first one
        private void lstAvailable_Enter(object sender, EventArgs e)
        {
            if (lstAvailable.Items.Count != 0)
            {
                if (lstAvailable.SelectedItem == null)
                {
                    lstAvailable.SetSelected(0, true);
                }

            }
        }

        // If list Control has focus and no List items are selected, select the first one
        private void lstSelected_Enter(object sender, EventArgs e)
        {
            if (lstSelected.Items.Count != 0 && m_MovingLeft == false)
            {
                if (lstSelected.SelectedItem == null)
                {
                    lstSelected.SetSelected(0, true);
                }

            }
        }

    }
}