/* -----------------------------------------------------------------
 * (c) Teradyne, Inc, 2007
 * All Rights Reserved
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------
 * frmPSetSignalsEditor.cs file acts as an GUI interface through which
 * we can interact with various engines like Sheet engine, 
 * UserInterface Engine,and Instrument Engine to get the definitions 
 * and to set the definitions.
 * Note that despite the name, this is a User Control, not a Form.
 * ------------------------------------------------------------------
 */

/*
 **************************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 * *   =============================================================================================
 *   Prabu S        01/13/12    Fixed tersw00182344 - Added SystemParameter Information in Form_Load() to Access KeyboardShortcutKeys 
 *   Bruce Brolsma  05/23/11    Fixed tersw00175952 - Modified RefreshPSetSignalList() to track whether PSets or Signals remain displayed in grids; if not, clear the grids to remove the parameters column (col 0)
 *   Bruce Brolsma  04/19/11    Fixed tersw00175087 - Modified tsbCopyCode_Click() so errors display when dialog re-opened, and m_strCode is reset when there are errors
 *   Bruce Brolsma  04/13/11    Fixed tersw00174642 - Additional check in cboInstrument & cboMixedInstrument _SelectedIndexChanged() to prevent grid from disappearing inappropriately.
 *   Bruce Brolsma  04/12/11    Fixed tersw00174550 - Added cboInstrument_Enter() to populate instruments when tabbing.
 *   Rajasekaran    06/04/11    Fixed tersw00174547 - Modified HandleSiteActivation(),UpdateMenu(),HandleSourceChanged(),To avoid multiple time display of unsaved changes dialog when updatemenu() is called
 *   Rajasekaran    16/03/11    Fixed tersw00173759 - Modified HandleSetDriver() to enlarge the error pane if only errors occurs
 *   Venkatachalam  09/27/11    Fixed tersw00180364 - disabled the  Update Tester and Apply Signal
 *                              buttons for Concurrent Test.
 *   Venkatachalam  09/05/11    Updated for concurrent test modifications.
 *                              1)Modified the m_notifier_Refresh() event
 *   Kannan K       06/08/11    Added multi sheet support to WaveDef Sheet.
 *   Kumaravel      02/18/11    Fixed tersw00173342 - Removed exception thrown line, because it causes run time error message while launching PSet Signal Editor from Tool Manager.
 *   Prabakaran     08/11/10    Fixed tersw00146054 -Added support to launch the editor from the signals sheet and psets sheet.
 *   Rajanbabu M    09/03/10    Fixed tersw00168541 - Called the method RestoreTextColor() for Update Tester to restore color.   
 *   Kalaivani C D  08/17/10    Fixed tersw00151095 - as user resizes the tool window in TDE, the PSSE height will be resized as the display proxy resizes.
 *   Ananda Kumary  06/01/10    Fixed tersw00162405 - Set the cursor symbol to hour glass before populating and updating the parameter grid in CreateNewFromExisting().
 *   ManoharReddy   06/02/10    Fixed tersw00162403 - Modified in HandlePSetOrSignalsButtonClick() as sholud not modify/refresh the PSet/Signal when use doesnot modify/select from PSetSignalsSelector.
 *   Rajanbabu M    06/01/10    Fixed tersw00162390 - The newly created PSet/signal for driver should not be shown in display grid and definition selector for sheet definition in compare window.
 *                              In order to display the parameter grid properly in compare window called the method CanSwitchSource().  
 *   Uma Aravindhan 05/27/2010  Fixed tersw00162400 - In btnUpdateTester_Click at the end call CompareNspcps
 *   Gokuldass      05/28/10    tersw00162747 - fixed as to disable the Update Tester button when no definition is shown in main display grid 
 *   Ananda Kumary  02/23/10    Fixed tersw00156612 - Modified CompareNspcps() to compare the values( one pset/signal definition in Main wondow and 
 *                              more one definition in side-by-side window). 
 *   Gokuldass      02/06/10    tersw00152201 - fixed as to resolve "Apply to all Sites" for Load Setting Connect functionality by adding
 *                              a condition to HandleApplyLoad().
 *   John Devadas   12/10/09    Fixed  tersw00153897: Error while Updating Sheet after modify PSet if signal(s) is viewed in "Side-by-Side" window
 *   ManoharReddy   12/08/09    tersw00153898 - Fixed as modified the error message.
 *   Ananda Kumary  11/03/09    Fixed autotest 2521 failure.
 *   Subeetha       10/23/09    Fixed tersw00151138 - Resolved multiple interactive issue by overridding DataGridViewComboboxCell features.
 *   ManoharReddy   10/22/2009  tersw00151503 - Fixed as removed the zzz.NET Services Configuration
 *                              which will be already configured by the Datatool.
 *   Kranthi Bharath  10/20/09  Fixed the defect tersw00148907
 *   Kannan K       10/07/2009  Fixed tersw00150873 bug
 *   Kannan K       08/25/2009  Added the Dynamic LoadSettings button name feature.
 *   Viswanadh R    05/04/09    Fixed tersw00133334 - Modified Framework_OnSetSvcProvider()
 *   John Devadas   03/20/09    Fixed  tersw00139352: F12 - Refresh not working properly with Side-by-side with PSet & Signal window combination
 *   Hui Teng       11/06/08    Removed redundant calls to UpdateMenuItem to update menu items for Sheet Definition and Driver Definition
 *                              in function HandleSiteActivation and TDECommandsControl_Command. This function will be called by the radio
 *                              button selection change event handlers
 *   Manohar Reddy  11/06/08    Fixed  tersw00133724: Fixed as modified the AddMenuItem for the side-by-side shortcut in InitializeTDEToolMenu()
 *   Manohar Reddy  11/05/08    Fixed  tersw00133689: Removed the if condition in HandlePSetOrSignalsButtonClick() function.
 *   Simon Pogrebinsky 09/02/08    512sites Site Selection bug fixes
 *   Simon Pogrebinsky 08/06/08    512sites Project changes
 *   Simon Pogrebinsky 08/05/08    ToolStripCheckBox class is added
 *   Maria Raykin   08/13/08    Fixed  tersw00114804: display message boxes upon successful completion of "update/apply" tasks
 *   V.Mariappa Raja 05/09/08   Fixed 2519 AT failure (7.10_Core).
 *   Kiran Ch      05/06/08    Fixed the defect : tersw00120717 
 *   Suresh V A     04/10/08    Fixed defect tersw00120131.
 *   Latha S        04/01/08    Configure for .Net Services in Framework_OnSetSvcProvider().
 *   Abdul          04/01/08    Fixed:tersw00119491 - Column order is not retained.
 *   Abdul          03/05/08    Fixed tersw00118299
 *   Shoban Jayaraj 03/07/08    Fixed tersw00118428 - invalid choices in dropdown are cleared
 *   Kiran Kumar C  03/02/08    Fixed the defect : tersw00118279.
 *   Shoban Jayaraj 03/04/08    Fixed tersw00118282 - Clear instrument combo if pin list contains incompatible pins
 *   Suresh K M     02/26/08    Fixed defect TERSW00117641.
 *   Kiran Kumar C  02/22/08    Fixed tersw00117762.
 *   Kiran Kumar C  02/21/08    Fixed tersw00117703 - Exception on combo change of compare window
 *   Suresh V A     02/15/08    Fixed defects tersw00117341 and tersw00117339.
 *   Suresh K M     02/18/08    Fixed defect TERSW00117371.
 *   Suresh K M     02/15/08    Fixed defect TERSW00117388.
 *   Suresh K M     02/13/08    Fixed defects TERSW00117336 & TERSW00117333.
 *   Abdul          02/13/08    Fixed :tersw00117317,tersw00117330
 *   Shoban Jayaraj 01/25/08    Fixed tersw0014718 - Dropdown combo of last selected list when header is clicked
 *   Suresh V A     01/24/08    Fixed tersw00111854, tersw00116446. 
 *   Shoban Jayaraj 01/21/08    tersw00116317 - Removed ambiguous button in message box and added message box icon
 *   Shoban Jayaraj 01/18/08    tersw00116304 - Resume from add / duplicate NSPCP on error capability
 *   Shoban Jayaraj 01/17/08    tersw00116199 & tersw00116204 - capitilization and punctuation corrections
 *   Suresh V A     01/10/08    Fixed tersw00115943.
 *   Shoban Jayaraj 01/02/08    Fixed tersw00115591 - Cut / Copy with empty cell in edit mode throws exception
 *   Nadeem Khan    12/31/07    Fixed tersw00115651, tersw00115649
 *   Nadeem Khan    12/12/07    Fixed tersw00115117
 *   V.Mariappa Raja 11/29/07   Fixed tersw00114615.
 *   Suresh V A     11/28/07    Fixed tersw00114320.
 *   Nadeem Khan    11/27/07    Remove Split Button and Add separate buttons, Change Toggle Button Text.
 *   V.Mariappa Raja 11/27/07   Fixed tersw00113672.
 *   Abdul          11/23/07    Fixed: tersw00114407,tersw00114406,tersw00114384 
 *   Abdul          11/23/07    Fixed:tersw00114459,tersw00114475
 *   P.Babu 	    11/23/07	Fixed tersw00114456
 *   V.Mariappa Raja 11/19/07    Fixed tersw00114334.
 *   Kiran          11/22/07    Fixed :tersw00114453
 *   Kiran          11/22/07    Fixed :tersw00114455
 *   Abdul           11/20/07   Fixed :tersw00114280
 *   Kiran          11/20/07    Fixed tersw00114202. 
 * V.Mariappa Raja  11/19/07    Fixed tersw00114016.
 *   Kiran          11/16/07    Fixed tersw00114114
 *   Shoban Jayaraj 11/16/07    Fixed tersw00114190 - cut / copy / past does not work via the tool bar
 *   Andy Kittross  07-Nov-07   Optimizations (tersw00113622)
 *   Nadeem Khan    11/4/07     Added ability of the user to select only one pset/signal to apply/load.
 *   Nadeem Khan    11/4/07     Made retrieval of PSet/Signal list site sensitive
 *   M. Raykin      10/26/07    Fixed tersw00113237
 *   Andy Kittross  10/26/07    Fixed tersw00113232 - add message to explain why definitions may disappear upon refresh
 *   Andy Kittross  10/26/07    Fixed tersw00113190 - gray out 'Duplicate' button if more than one def'n shown
 *   M. Raykin      10/25/07    Fixed tersw00113135
 *   M. Raykin      10/24/07    Fixed tersw00113145
 *   M. Raykin      10/23/07    Fixed tersw00113129: added call to m_UIEngine.ConfigureCells() to enable/disable cells 
 *                              properly
 *   V. Mariappa Raja 10/22/07  Made Home and End keys to work while editing cell using double click.
 *   M. Raykin      10/22/07    Fixed tersw00113024 - made change to configureButtons() method:  synchronize 
 *                              default state of a SplitButton with radio button choice 
 *   V. Mariappa Raja 10/19/07  Fixed CutCopyPaste unit test failure.
 *   Andy Kittross  10/16/07    Refactor global settings to make panel behvior more consistent on left and right
 *   P.Babu         10/17/07    Fixed the Errorgrid multiselection and Refresh Definition changed.
 *   Shoban Jayaraj 10/17/07    Fix: Undo in voltage followed by redo lost old value.
 *   V. Mariappa Raja 10/17/07  Fixed tersw00111854.
 *   Shoban Jayaraj 10/17/07    Handled issue with SetDriver and Unso / Redo.
 *  R. Chorev       10/16/07    Fix issues with site selection and site dropdown
 *   M.Raykin       10/16/07    Fixed ShowMessageBox method so that it includes title and information icon;
 *                              Fixed TextBox "SelectAll" issue
 *   T. Friske      10/15/07    Fixed tersw00112548 - call Dispose on m_notifier to disconnect ConnectionPoint.
 *   Andy Kittross  10/15/07	tersw00112646 - remove All/Remove/Left/Right buttons on comapre window
 *   Shoban Jayaraj 10/17/07    SetDriver label is updated properly when changes are undone / redone
 *                              Reorganized history field
 *   Shoban Jayaraj 10/12/07    Fixed tersw00112724
 *   M. Raykin      10/04/07    Fixed Defects: tersw00112255, tersw00112355, tersw00112256
 *   Kiran          09/28/07    Fixed Defect : tersw00112057
 *   Kiran          09/28/07    Addressed Roy's comments for defect : tersw00111847
 *   V. Mariappa Raja 09/28/07  Fixed tersw00111829.
 *   Suresh V A     09/27/07    Fixed the Defect : tersw00111857
 *   Kiran          09/26/07    Fixed the defect : tersw00111864
 *   Kiran          09/25/07    Fixed tersw00111847
 *   Kiran          09/22/07    Fixed tersw00111764
 *   Kiran          09/22/07    Fixed tersw00111481
 *   Shoban Jayaraj 09/22/07    tersw00111720 - Dropdown combos in one click
 *   Shoban Jayaraj 09/21/07    Undo / redo fix - tersw00111680
 *   Shoban Jayaraj 09/21/07    Notify user when the pset editor is closed with changes
 *   V. Mariappa Raja 09/21/07  Fixed tersw00111679.
 *   P.Babu         09/21/07    Fixed tersw00111697 (Modified configureButtons()).
 *   P.Babu         09/20/07    Fixed tersw00111476
 *   P.Babu         09/18/07    Fixed tersw00111443
 *   Kiran          09/18/07    Changed for PSetSignalsSelector
 *   P.Babu         09/15/07    Handled tsbCopyCode click button to generate code.
 *   Kiran Kumar Ch 09/01/07    Addressed Internal review comments
 *   Kiran Kumar Ch 08/31/07    Added Event Handlers for the Next, prev, Add, remove compare items
 *   Nadeem Khan    07/2/07     Created.
 **************************************************************************************************
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;
using Teradyne.Igxl.Utilities;
using Teradyne.Igxl.PSetSignalsEditor.PSetSignalsEditorServices;
using DMGRPSETSSHEETLIB;
using DMGRSIGNALSSHEETLIB;
using CollectionEditHelperLib;
using TERCOMMONOBJLib;
using Teradyne.Igxl.PSetSignalsEditor.Exceptions;
using System.Runtime.InteropServices;
using Teradyne.Igxl.Proxies;
using Teradyne.Igxl.Utilities.PinInfo;
using CORELANGUAGELib;
using System.Diagnostics;
using Teradyne.Igxl.Utilities.CustomToolStripControls;
using Teradyne.Igxl.PSetSignalsEditor.BalloonTip;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    [ToolboxBitmap(typeof(EmbeddedResourceFinder), "Teradyne.Igxl.PSetSignalsEditor.Resources.icon.bmp")]
    public partial class frmPSetSignalsEditor : Tool, ITDETool, IUndoable
    {
        private enum UpdateMenuItemEnum : int { ENABLED, CHECKED,VISIBLE };        
        private enum ClipBoardAccessEnum : int { GETTEXT, SETTEXT, SETDATAOBJECT };
        private const int CLIPBOARD_ACCESS_COUNTER = 5;
        private const int CLIPBOARD_ACCESS_DELAY = 250;
        private const int ERROR_WINDOW_COLOR = 0xFFC0C0; //RGB(255,192,192)
        private const int SPI_SETKEYBOARDSHORTCUTS = 0x100B;  //Constant used to display for shortcutkeys.
        public GlobalSettings m_GlobalSettings = new GlobalSettings();
        public globalPSetSignalValues m_GlobalPSetSignalValues = new globalPSetSignalValues();
        //This method used to enabling  the shortcut key programmatically.  
        [DllImport("user32.dll")]
        public static extern void SystemParametersInfo(int uiAction, int uiParam, bool pvParam, int fWinIni); 

        String m_InstanceTag;
        String m_ToolName;
        String m_oldInstrumentValue;
        String m_compareOldInstrumentValue;
        TDEMemento m_WindowState;
        Boolean m_suppressEventHandler;
        CCollectionEditHelper psetHelper;
        CCollectionEditHelper signalHelper;
        String m_cellCache;               //Cached value of a cell, before it is edited.
        //The various engines used in the PSet/Signal Editor
        InstrumentEngine m_Main_InstrEngine;
        InstrumentEngine m_Compare_InstrEngine;
        UserInterfaceEngine m_UIEngine;
        UserInterfaceEngine m_CompUiEngine;
        ISheetEngine m_SheetEngine;
        PSetSheetEngine m_psetSheetEngine;
        SignalsSheetEngine m_signalsSheetEngine;
        CodeGenEngine m_CodeGenEngine;
        CoreEventHandler m_CoreHandler;
        Dictionary<String, NspcpItem> m_NSPCPList;   //Contains the list of all nspcps currently displayed.
        Dictionary<String, NspcpItem> m_CompNspcpList; //Contains the list of all nspcps currently displayed in Compare window.
        TDECOMMONSRVLib.ITDE m_TDE;     //Reference back to the TDE.
        TDECOMMONSRVLib.IPreference m_IPreferences;
        TDEEventNotifier m_notifier;
        Boolean m_DefChanged_NotAppliedToDriver;
        Boolean m_DefChanged_NotAppliedToSheet;
        Boolean m_bEnterTxtPinName;
        Boolean m_bEntertxtMixedPinName;
        Boolean m_CheckUnsavedDefinitionShowing;

        TDEControls.CTDECommands m_TDECommands;
        ActiveBar3Library.ActiveBar3 m_actBarPSE;
        const int CPDDCMDSTARTTOOLID = 50;
        int m_toolID = CPDDCMDSTARTTOOLID;
        Boolean m_bNewLang;
        Boolean m_bNeedWithBlock;
        Boolean m_NeedToCheckChanges;
        String m_strCode;
        private const string MNUTOOL = "mnuTool";
        private const string MNUEDIT = "mnuEdit";
        private const string PSETSIGNALSLANGUAGE = "PSetSignalsLanguage";
        private const string NEWPSETSIGNALS = "NewPSetSignals";
        private const string NEEDWITHBLOCK = "NeedWithBlock";
        private const string APPLYTOALLSITES = "ApplyToAllSites";
        private const string MIXEDCOMPAREGRID = "MixedCompareGrid";
        private const string NEWPSET = "NewPSet";
        private const string DUPLICATEPSET = "DuplicatePSet";
        private const string SHEETDEFINITION = "Sheetdefinition";
        private const string CURRENTDRIVERDEFINITION = "Currentdriverdefinition";
        private const string SETDRIVER = "SetDriver";
        private const string UPDATESHEET = "UpdateSheet";
        private const string APPLYPSET = "ApplyPSet";
        private const string LOADSIGNALSETTINGS = "LoadSignalSettings";
        private const string CUT = "Cut";
        private const string COPY = "Copy";
        private const string PASTE = "Paste";
        private const string UNDO = "Undo";
        private const string REDO = "Redo";
        private const string COPYCODE = "CopyCode";        
        private const string HELPTOPIC = "HelpTopic"; 
        private const string HELPABOUTTOOL = "HelpAboutTool";
        private const string HELPTOOL = "HelpTool";
        private const string HELPSPECIFICATIONS = "HelpSpecifications";
        private readonly Color READ_ONLY_COLOR = Control.DefaultBackColor;
        Boolean m_bUnitTest;
        String m_currWaveDefName;
        Boolean m_waveDefSelected;  //Determines whether the wavedef cell is selected or not.
        private const int PSET = 0;
        private const int SIGNAL = 1;
        private const string PSET_BUTTON_TEXT = "Add/Remove PSets...";
        private const string SIGNAL_BUTTON_TEXT = "Add/Remove Signals...";
        private const string NEW_PSET_TEXT = "New PSet";
        private const string NEW_SIGNAL_TEXT = "New Signal";
        private const int FIRST_ITEM = 0;
        private const string DEFAULT_TEXT_INST = "<Select>";
        private const string DEFAULT_TEXT_PIN = "<enter or browse>";
        private const string SOURCE_PSET = "PSet";
        private const string SOURCE_SIGNAL = "Signal";
        private const int COMPARABLE_COUNT = 2;
        private const int PARAM_NAME_COLUMN = 0;
        private const int PARAM_VALUE_COLUMN = 1;
        private const int DUPLICATE_COLUMN = 2;
        private const int MAIN_COLUMN = 1;
        private const int DUP_COLUMN_COUNT = 3;
        private const string TOOLVERSION = "1.0";
        private const int CUMSTOM_MENU_ENTRY = -100;
        private const string FIRST_PARAMETER_COLUMN_NAME = "Name";

        private readonly System.Drawing.Color COMP_COLOR = System.Drawing.Color.Yellow;
        
        private int currentSiteSelectionIdx = 0;
        private int prevSiteSelectionIdx = 0; //indicate the index of the selected site prior to changing it.
        private bool HandlingSiteActivation = false; //true if we are in the process of responding to a site selection change
        private bool HandlingTDESiteChangeEvent = false; //true if we are in the process of responding to a TDE site change
        private int prevTDEMasterSite; //indicates the previous TDE master site
        
        private bool _syncMasterSite = true;
        private bool _siteValidated = true;
        
        // For undo redo support
        UndoRedoManager m_undoManager;
        private const int UNDO_CAPACITY = 30;

        private bool m_bTimer = false;
        private DateTime endTime;

        private bool _isLoadSettingButtonHover = false;
        private BalloonTipInfo _rcInfo = null;
               

        // For to get the ColumnIndex and RowIndex.        
        System.Windows.Forms.DataGridView.HitTestInfo m_hitInfo = null;

        private KeyValuePair<int, Control>[] m_arrTabStopCtrls = null;     // Tab/Shift-Tab Control support
        private string m_strPinNames;
        private string m_lastSource;

        // used to track changes made in pin text box by the main as well as the Mixed Signal pane
        // This in turn is used by the instrument combo to reset the instrument list
        private bool m_bPinNameChanged;
        private int _colNum = -1;
        private long m_tmtoolID;

        private tlExecSiteMgr m_siteManager = null;

        /// <summary>
        /// This property will hold the tool id generated from tool manager, which will be later used 
        /// while locking the flow domains before executing flow domain dependent calls
        /// </summary>
        public long TMtoolID
        {
            get { return m_tmtoolID; }
            set { m_tmtoolID = value; }
        }

        /// <summary>
        /// This property is used to hold TDE reference which is required in InstrumentEngine to 
        /// set the flow domain before accessing flow domain dependent calls
        /// </summary>
        public TDECOMMONSRVLib.ITDE TDE
        {
            get { return m_TDE; }
            set { m_TDE = value; }
        }

        // public property
        public bool DefinitionChanged
        {
            get
            {
                return (m_DefChanged_NotAppliedToSheet||m_DefChanged_NotAppliedToDriver);
            }
            set
            {
                m_DefChanged_NotAppliedToSheet = m_DefChanged_NotAppliedToDriver=value;
            }
        }
        /// <summary>
        /// Returns the mouse hover status
        /// </summary>
        public bool IsLoadSettingButtonHover
        {
            get { return _isLoadSettingButtonHover; }
        }


        public frmPSetSignalsEditor()
        {
            try
            {
                //Initialize the display and attach services received from TDE.
                InitializeComponent();
                psetHelper = new CCollectionEditHelper();
                signalHelper = new CCollectionEditHelper();
                MyServices.Attach(Framework);
                //Create handlers for OnSetSvcProvider and OnApplicationQuit events.
                Framework.OnSetSvcProvider += new Framework.SetSvcProviderHandler(Framework_OnSetSvcProvider);
                Framework.OnApplicationQuit += new EventHandler(Framework_OnApplicationQuit);
                m_undoManager = new UndoRedoManager(UNDO_CAPACITY);
                m_bEnterTxtPinName = false;
                m_bEntertxtMixedPinName = false;

                m_bPinNameChanged = false;
                m_NeedToCheckChanges = true;

             }
            catch (Exception e)
            {
                Console.Write("Exception caught in frmPSetSignalsEditor {0}", e.StackTrace);
            }

        }

        /// <summary>
        /// Handle a refresh event from TDE.
        /// </summary>
        /// <param name="Types"></param>
        void m_notifier_Refresh(int Types)
        {
            switch ((TDECOMMONSRVLib.TDERefreshEvent)Types)
            {
                case TDECOMMONSRVLib.TDERefreshEvent.tdeDebugTrap:
                    HandleSiteActivation();
                    break;
                case TDECOMMONSRVLib.TDERefreshEvent.tdeJobExecutionEnd:
                    configureCopyCodeButton();
                    if (m_GlobalSettings.MainPanel.DATA_SOURCE == GlobalUtils.DATA_SOURCE.DRIVER) ClearMainParameterGrid();
                    if (m_GlobalSettings.ComparePanel.DATA_SOURCE == GlobalUtils.DATA_SOURCE.DRIVER) ClearCompareParameterGrid();
                    HandleSiteActivation();
                    break;
                case TDECOMMONSRVLib.TDERefreshEvent.tdeSiteChange:
                    if (_syncMasterSite)//we only want to respond to this event if we are listening to the master site
                    {
                        HandlingTDESiteChangeEvent = true; //set flag to indicate we are handling a site change
                        InitSiteDropDown(); //re-initialize the site drop down
                        configureCopyCodeButton();
                        HandleSiteActivation();
                        HandlingTDESiteChangeEvent = false; //set flag to indicate we are done handling a site change
                    }
                    break;
                case TDECOMMONSRVLib.TDERefreshEvent.tdeUserRefresh:
                    if (!m_CheckUnsavedDefinitionShowing) //If not already in the refresh process.
                    {
                    if (!CheckForUnsavedDefinitions())
                    {
                        if (IsSitesActive(GetSelectedSite()))
                        {
                            rbnCurrentDriverDefinition.Enabled = true;
                            UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.ENABLED, true);  // Update menu item
                            UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.CHECKED, true);  // Update menu item
                            UpdateMenuItem(SHEETDEFINITION, UpdateMenuItemEnum.CHECKED, false);  // Update menu item
                            rbnMixedCurrentDriverDefinition.Enabled = true;
                        }
                        else
                        {
                            rbnCurrentDriverDefinition.Enabled = false;
                            UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.CHECKED, false);  // Update menu item
                            UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.ENABLED, false);  // Update menu item                            
                            UpdateMenuItem(SHEETDEFINITION, UpdateMenuItemEnum.CHECKED, true);  // Update menu item
                            rbnMixedCurrentDriverDefinition.Enabled = false;
                        }
                        UpdateParameterGrid(false, ref m_UIEngine, ref m_GlobalSettings.MainPanel, ref m_GlobalPSetSignalValues.MainPanel,
                                                ref m_NSPCPList, txtPinName.Text, cboInstrument.Text,ref m_Main_InstrEngine);
                    }
                    //refreh compare grid
                    UpdateParameterGrid(false, ref m_CompUiEngine, ref m_GlobalSettings.ComparePanel, ref m_GlobalPSetSignalValues.ComparePanel,
                                            ref m_CompNspcpList, txtMixedPinName.Text, cboMixedInstrument.Text, ref m_Compare_InstrEngine);
                    }
                    break;
                case TDECOMMONSRVLib.TDERefreshEvent.tdeRunDebugStarted:
                    //Refresh Grid
                    break;
                case TDECOMMONSRVLib.TDERefreshEvent.tdeHardwareStateChange:
                    if (!(m_DefChanged_NotAppliedToDriver||m_DefChanged_NotAppliedToSheet))
                    {
                        if (IsSitesActive(GetSelectedSite()))
                        {
                            rbnCurrentDriverDefinition.Enabled = true;
                            UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.ENABLED, true);  // Update menu item
                            if (m_NSPCPList.Count == 0)
                            {
                                UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.CHECKED, true);  // Update menu item
                                UpdateMenuItem(SHEETDEFINITION, UpdateMenuItemEnum.CHECKED, false);  // Update menu item
                            }
                            rbnMixedCurrentDriverDefinition.Enabled = true;
                        }
                        else
                        {
                            rbnCurrentDriverDefinition.Enabled = false;
                            UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.CHECKED, false);  // Update menu item
                            UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.ENABLED, false);  // Update menu item
                            UpdateMenuItem(SHEETDEFINITION, UpdateMenuItemEnum.CHECKED, true);  // Update menu item
                            rbnMixedCurrentDriverDefinition.Enabled = false;
                        }
                        //refresh main grid
                        UpdateParameterGrid(false, ref m_UIEngine, ref m_GlobalSettings.MainPanel, ref m_GlobalPSetSignalValues.MainPanel,
                                                ref m_NSPCPList, txtPinName.Text, cboInstrument.Text, ref m_Main_InstrEngine);
                    }
                    //refreh compare grid
                    UpdateParameterGrid(false, ref m_CompUiEngine, ref m_GlobalSettings.ComparePanel, ref m_GlobalPSetSignalValues.ComparePanel,
                                            ref m_CompNspcpList, txtMixedPinName.Text, cboMixedInstrument.Text, ref m_Compare_InstrEngine);
                    break;
                   case TDECOMMONSRVLib.TDERefreshEvent.tdeFlowDomainChange:
                       //Clear the previous in Pin text box. 
                       txtPinName.Text = "<enter or browse>";
                       cboInstrument.Items.Clear();
                       //Clear the grid
                       ClearMainParameterGrid();
                       //Set the UpdateTester, UpdateSheet and ApplyPSet buttons are disabled.
                       btnUpdateTester.Enabled = false;
                       btnUpdateSheet.Enabled = false;
                       btnApplyPSet.Enabled = false;
                      break;

            }

            //check the clipboard status to see if paste should be enabled\disabled.
            CheckAndUpdatePasteMenuItem();
        }

      


        void Framework_OnApplicationQuit(object sender, EventArgs e)
        {
            m_arrTabStopCtrls = null;                       
            m_UIEngine = null;
            m_CompUiEngine = null;
            m_CodeGenEngine = null;
            m_CoreHandler = null;
            m_psetSheetEngine = null;
            m_signalsSheetEngine = null;
            m_SheetEngine = null;
            
            m_Main_InstrEngine.Dispose();
            m_Compare_InstrEngine.Dispose();
            m_Main_InstrEngine = null;
            m_Compare_InstrEngine = null;

            (psetHelper as FRAMEWORKINTERFACELib.IFrameWork).ApplicationQuit();
            (signalHelper as FRAMEWORKINTERFACELib.IFrameWork).ApplicationQuit();

            psetHelper = null;
            signalHelper = null;
            m_actBarPSE = null;
            object dummy = null;
            m_TDECommands.set_ActiveBar(ref dummy);
            m_TDECommands = null;
            (m_notifier as IDisposable).Dispose();
            m_notifier = null;
            m_TDE = null;
            m_IPreferences = null;
            m_bNeedWithBlock = false;
            m_bNewLang = false;
            m_WindowState = null;
            m_GlobalPSetSignalValues.MainPanel.PSetValues.Clear();
            m_GlobalPSetSignalValues.MainPanel.SignalValues.Clear();
            m_GlobalPSetSignalValues.ComparePanel.PSetValues.Clear();
            m_GlobalPSetSignalValues.ComparePanel.SignalValues.Clear();
        }

        void Framework_OnSetSvcProvider(object svcProvider)
        {
            try
            {
                if (svcProvider == null) return;
                
                //Create the PSet and Signals Collection Edit Helper.
                (psetHelper as FRAMEWORKINTERFACELib.IFrameWork).SvcProvider = svcProvider;
                (signalHelper as FRAMEWORKINTERFACELib.IFrameWork).SvcProvider = svcProvider;
                
                
                psetHelper.set_KeySet(GlobalUtils.REG_KEY_PSET, GlobalUtils.KEYSET_NAME);
                signalHelper.set_KeySet(GlobalUtils.REG_KEY_SIGNAL, GlobalUtils.KEYSET_NAME);

                //Obtain a reference to TDE.
                m_TDE = (TDECOMMONSRVLib.ITDE)svcProvider;

                //gets the Service Settings
                m_IPreferences = m_TDE.UserPreferenceDataService;
               
                m_notifier = new TDEEventNotifierImpl(m_TDE.CreateNewTDEEventNotifier(m_InstanceTag));
                m_notifier.RefreshTypes = Convert.ToByte(m_notifier.RefreshTypes) | Convert.ToByte(TDECOMMONSRVLib.TDERefreshEvent.tdeValidationPass) |
                    Convert.ToByte(TDECOMMONSRVLib.TDERefreshEvent.tdeValidationFail) | Convert.ToByte(TDECOMMONSRVLib.TDERefreshEvent.tdeUserRefresh) |
                    Convert.ToByte(TDECOMMONSRVLib.TDERefreshEvent.tdeSiteChange) | Convert.ToByte(TDECOMMONSRVLib.TDERefreshEvent.tdeDebugTrap) |
                    Convert.ToByte(TDECOMMONSRVLib.TDERefreshEvent.tdeJobExecutionEnd) | Convert.ToByte(TDECOMMONSRVLib.TDERefreshEvent.tdeHardwareStateChange) |
                    Convert.ToInt32(TDECOMMONSRVLib.TDERefreshEvent.tdeFlowDomainChange);
                   
                m_notifier.Refresh += new RefreshDelegate(m_notifier_Refresh);
                InitEditor();
            }
            catch (Exception e)
            {
                Console.Write(e.StackTrace);
            }
        }


        /// <summary>
        /// Initializes the editor 
        /// We also initialize the dropdowns here.
        /// </summary>
        private void InitEditor()
        {
            m_waveDefSelected = false;
            m_suppressEventHandler = false;
            m_NSPCPList = new Dictionary<String, NspcpItem>();
            m_CompNspcpList = new Dictionary<string, NspcpItem>();
            m_DefChanged_NotAppliedToDriver = false;    //No definition has been changed.
            m_DefChanged_NotAppliedToSheet = false;
            m_CheckUnsavedDefinitionShowing = false;
            m_bUnitTest = false;
            
            // Get the tool id from Tool manager, which needs to be passed while locking the 
            // flow domains. Tool manager in tur will use this id to handle locking multiple flow domains
            m_tmtoolID = MyServices.ToolManager.ToolListener.GetNextToolId();

            //Initialize Code String
            m_strCode = String.Empty;

            // initialize the pin names
            m_strPinNames = String.Empty;
            //Clear the parameter grid (incase there are any existing nspcp's being displayed)
            ClearMainParameterGrid();
            // subscribe to resize events of the display proxy holding PSSE.
            Proxy.Resize += new System.EventHandler(Proxy_Resize);

        }

        /// <summary>
        /// If sites are not active, disable driver selection radio button
        /// and activate sheet selection radio button only.
        /// Enable/Disable the copy code button accordingly.
        /// </summary>
        private void HandleSiteActivation()
        {
            if (HandlingSiteActivation) return; //if we are allready in the process of doing this ignore
            
            HandlingSiteActivation = true;
            int site = GetSelectedSite();
            
            if (IsSitesActive(site))
            {
                // Enable GUI elements for running program state
                rbnCurrentDriverDefinition.Enabled = true;
                rbnMixedCurrentDriverDefinition.Enabled = true;
                UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.ENABLED, true);  // Update menu item
                
                // clear the physical instrument cache dictionary, 
                // when the Site has been changed
                m_Main_InstrEngine.ClearPhysicalInstCache();
                m_Compare_InstrEngine.ClearPhysicalInstCache();

                if (dgvParametersEdit.Columns.Count < 2)
                {   // Grid is empty.  Force to driver mode.  
                    m_GlobalSettings.MainPanel.DATA_SOURCE = GlobalUtils.DATA_SOURCE.DRIVER;
                }
                if (dgvCompareParamEdit.Columns.Count < 2)
                {   // Grid is empty.  Force to driver mode.  
                    m_GlobalSettings.ComparePanel.DATA_SOURCE = GlobalUtils.DATA_SOURCE.DRIVER;
                }

                if (HandleSourceChanged(m_GlobalSettings.MainPanel.DATA_SOURCE, ref m_GlobalSettings.MainPanel, ref m_NSPCPList,
                            ref m_UIEngine, ref m_GlobalPSetSignalValues.MainPanel, txtPinName.Text, cboInstrument.Text,
                            ref rbnSheetDefinition, ref rbnCurrentDriverDefinition, ref m_Main_InstrEngine))
                {
                    if (HandleSourceChanged(m_GlobalSettings.ComparePanel.DATA_SOURCE, ref m_GlobalSettings.ComparePanel, ref m_CompNspcpList,
                            ref m_CompUiEngine, ref m_GlobalPSetSignalValues.ComparePanel, txtMixedPinName.Text, cboMixedInstrument.Text,
                            ref rbnMixedSheetDefinition, ref rbnMixedCurrentDriverDefinition, ref m_Compare_InstrEngine))
                    {
                        prevSiteSelectionIdx = tscboSite.SelectedIndex;
                    }
                    else
                    {
                        if (HandlingTDESiteChangeEvent)
                            tscboSite.Text = prevTDEMasterSite.ToString();
                        else
                            tscboSite.SelectedIndex = prevSiteSelectionIdx;
                    }
                }
                else
                {
                    if (HandlingTDESiteChangeEvent)
                        tscboSite.Text = prevTDEMasterSite.ToString();
                    else
                        tscboSite.SelectedIndex = prevSiteSelectionIdx;
                }
            }
            else
            {
                if (HandleSourceChanged(GlobalUtils.DATA_SOURCE.SHEET, ref m_GlobalSettings.MainPanel, ref m_NSPCPList,
                                ref m_UIEngine, ref m_GlobalPSetSignalValues.MainPanel, txtPinName.Text, cboInstrument.Text, 
                                ref rbnSheetDefinition, ref rbnCurrentDriverDefinition, ref m_Main_InstrEngine))
                {
                    if (HandleSourceChanged(GlobalUtils.DATA_SOURCE.SHEET, ref m_GlobalSettings.ComparePanel, ref m_CompNspcpList,
                                ref m_CompUiEngine, ref m_GlobalPSetSignalValues.ComparePanel, txtMixedPinName.Text, cboMixedInstrument.Text, 
                                ref rbnMixedSheetDefinition, ref rbnMixedCurrentDriverDefinition, ref m_Compare_InstrEngine))
                    {
                        prevSiteSelectionIdx = tscboSite.SelectedIndex;
                    }
                    else
                    {
                        if (HandlingTDESiteChangeEvent)
                            tscboSite.Text = prevTDEMasterSite.ToString();
                        else
                            tscboSite.SelectedIndex = prevSiteSelectionIdx;
                    }
                }
                else
                {
                    if (HandlingTDESiteChangeEvent)
                        tscboSite.Text = prevTDEMasterSite.ToString();
                    else
                        tscboSite.SelectedIndex = prevSiteSelectionIdx;
                }
                rbnCurrentDriverDefinition.Enabled = false;
                UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.CHECKED, false);  // Update menu item
                UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.ENABLED, false);  // Update menu item
                UpdateMenuItem(SHEETDEFINITION, UpdateMenuItemEnum.CHECKED, true);  // Update menu item
                rbnMixedCurrentDriverDefinition.Enabled = false;
            }
            configureButtons();
            HandlingSiteActivation = false;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Initial setup

            //TDE Menu Activation
            m_TDECommands = new TDEControls.CTDECommands();
            m_actBarPSE = new ActiveBar3Library.ActiveBar3();
            object objActBar = (object)m_actBarPSE;
            m_TDECommands.set_ActiveBar(ref objActBar);
            m_TDECommands.Command += new TDEControls.__CTDECommands_CommandEventHandler(TDECommandsControl_Command);
            InitializeTDEToolMenu();

            // Create subcomponents early, since these might be called when setting control properties such as selectedIndex
            m_Main_InstrEngine = new InstrumentEngine(this, ref psetHelper, ref signalHelper,ref m_GlobalSettings);
            m_Compare_InstrEngine = new InstrumentEngine(this, ref psetHelper, ref signalHelper, ref m_GlobalSettings);
            //Enable the underline access key by default
            SystemParametersInfo(SPI_SETKEYBOARDSHORTCUTS, 0, true, 0); 
            //Select sheet engine type
            m_psetSheetEngine = new PSetSheetEngine(ref psetHelper);
            m_signalsSheetEngine = new SignalsSheetEngine(ref signalHelper);

            m_UIEngine = new UserInterfaceEngine(ref dgvParametersEdit, ref dgvErrors, ref m_Main_InstrEngine);
            m_CompUiEngine = new UserInterfaceEngine(ref dgvCompareParamEdit, ref dgvErrors, ref m_Compare_InstrEngine);
            m_CodeGenEngine = new CodeGenEngine();
            m_CoreHandler = new CoreEventHandler();
            
            //Change from form to UserControl
            this.BorderStyle = BorderStyle.Fixed3D;  
          
            cboSource.SelectedIndex = (int)m_GlobalSettings.MainPanel.NSPCP_MODE;
            cboMixedSource.SelectedIndex = (int)m_GlobalSettings.MainPanel.NSPCP_MODE;
            // Set up the Edit Parameters grid
            dgvParametersEdit.ColumnCount = 1;
            dgvParametersEdit.BackgroundColor = Control.DefaultBackColor; // backcolor of interior area outside cells
            dgvParametersEdit.ColumnHeadersDefaultCellStyle.Font = new Font(dgvParametersEdit.Font, FontStyle.Bold);
            dgvParametersEdit.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.DisplayedCellsExceptHeaders;
            dgvParametersEdit.ColumnHeadersBorderStyle = DataGridViewHeaderBorderStyle.Single;
            dgvParametersEdit.CellBorderStyle = DataGridViewCellBorderStyle.Single;
            dgvParametersEdit.RowHeadersVisible = false;
            dgvParametersEdit.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells;
            dgvParametersEdit.AutoSizeColumnsMode = System.Windows.Forms.DataGridViewAutoSizeColumnsMode.AllCells;
            dgvParametersEdit.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right | System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Bottom)));
            dgvParametersEdit.Columns[0].Name = "Name";
            dgvParametersEdit.Columns[0].Width = 120;
            dgvParametersEdit.Columns[0].ReadOnly = true;
            dgvParametersEdit.Columns[0].DefaultCellStyle.BackColor = Control.DefaultBackColor;
            dgvParametersEdit.Columns[0].SortMode = DataGridViewColumnSortMode.NotSortable;
            dgvParametersEdit.Columns[0].Frozen = true;
            dgvParametersEdit.MultiSelect = true;
            dgvParametersEdit.SelectionMode = DataGridViewSelectionMode.CellSelect;
            // This has to be edit on f2 or key for Cut / Copy, Undo / Redo to work
            dgvParametersEdit.EditMode = System.Windows.Forms.DataGridViewEditMode.EditOnKeystrokeOrF2;
            // Suppress empty last row:
            dgvParametersEdit.AllowUserToAddRows = false;
            dgvParametersEdit.AllowUserToOrderColumns = true;
            dgvParametersEdit.ClipboardCopyMode = DataGridViewClipboardCopyMode.EnableWithoutHeaderText;

            if (dgvParametersEdit.CurrentCell != null)
                dgvParametersEdit.CurrentCell.Selected = false; // ensure no cell is initially selected


            // Set up the Compare parameters grid
            dgvCompareParamEdit.ColumnCount = 1;
            dgvCompareParamEdit.BackgroundColor = Control.DefaultBackColor; // backcolor of interior area outside cells
            dgvCompareParamEdit.ColumnHeadersDefaultCellStyle.Font = new Font(dgvCompareParamEdit.Font, FontStyle.Bold);
            dgvCompareParamEdit.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.DisplayedCellsExceptHeaders;
            dgvCompareParamEdit.ColumnHeadersBorderStyle = DataGridViewHeaderBorderStyle.Single;
            dgvCompareParamEdit.CellBorderStyle = DataGridViewCellBorderStyle.Single;
            dgvCompareParamEdit.RowHeadersVisible = false;
            dgvCompareParamEdit.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells;
            dgvCompareParamEdit.AutoSizeColumnsMode = System.Windows.Forms.DataGridViewAutoSizeColumnsMode.AllCells;// .AllCellsExceptHeader; // important when resizing                       
            dgvCompareParamEdit.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right | System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Bottom)));
            dgvCompareParamEdit.Columns[0].Name = "Name";
            dgvCompareParamEdit.Columns[0].Width = 120;
            dgvCompareParamEdit.Columns[0].ReadOnly = true;
            dgvCompareParamEdit.Columns[0].Frozen = true;
            dgvCompareParamEdit.Columns[0].DefaultCellStyle.BackColor = Control.DefaultBackColor;
            dgvCompareParamEdit.Columns[0].SortMode = DataGridViewColumnSortMode.NotSortable;

            dgvCompareParamEdit.MultiSelect = true;
            dgvCompareParamEdit.SelectionMode = DataGridViewSelectionMode.CellSelect;
            // Follow the same format as the first grid
            dgvCompareParamEdit.EditMode = System.Windows.Forms.DataGridViewEditMode.EditOnKeystrokeOrF2;
            // Suppress empty last row:
            dgvCompareParamEdit.AllowUserToAddRows = false;
            dgvParametersEdit.AllowUserToOrderColumns = true;
            if (dgvCompareParamEdit.CurrentCell != null)
                dgvCompareParamEdit.CurrentCell.Selected = false; // ensure no cell is initially selected

        
            ///Removed the Error Grid Initialization and Add the code for SplitterContainer Distance
            ///
            splitContainer1.SplitterDistance = splitContainer1.SplitterDistance + (this.Height - splitContainer1.SplitterDistance); 

            tsbCompareToggle.DisplayStyle = ToolStripItemDisplayStyle.Text;
            int BorderWidth = (this.Width - this.ClientSize.Width) / 2;
            this.Width = toolStrip1.Width + 4*BorderWidth;
            pnlLeft.Width = this.Width;


            pnlRight.Visible = false;
            splLeftRight.MinExtra = 0;
            splLeftRight.SplitPosition = pnlLeft.Width; // this must be set AFTER setting MinExtra
            splLeftRight.Visible = false;

            splitContainer1.Panel2MinSize = 35; // allows Close button to still be visible
            splitContainer1.Panel1MinSize = 284; // allows 2 rows of grid to still be visible           
            m_GlobalSettings.ComparePanel.NSPCP_MODE = m_GlobalSettings.MainPanel.NSPCP_MODE;

            rbnCurrentDriverDefinition.Enabled = false;
            rbnMixedCurrentDriverDefinition.Enabled = rbnCurrentDriverDefinition.Enabled;

            cboMixedSource.Text = cboSource.Text;
            m_GlobalSettings.ComparePanel.DATA_SOURCE = m_GlobalSettings.MainPanel.DATA_SOURCE;
            // line below has no effect
            cboSource.Select();

            // Using Tab/Shift-Tab controls to navigate controls in the Form
            SortedDictionary<int, Control> dictTabStopCtrls = new SortedDictionary<int, Control>();
            dictTabStopCtrls.Clear();
            EnumerateTabStopCtrls(ref dictTabStopCtrls, (Control)this);
            if (dictTabStopCtrls.Count > 0)
            {
                m_arrTabStopCtrls = new KeyValuePair<int, Control>[dictTabStopCtrls.Count];
                dictTabStopCtrls.CopyTo(m_arrTabStopCtrls, 0);                
            }            
            dictTabStopCtrls = null;

            UpdateMenuItemStatus(false);

        }


        /// <summary>
        /// Updates the menu item status.
        /// </summary>        
        private void UpdateMenuItemStatus(bool bStatus)
        {
            //Disable Cut and Copy when PSEE loads. Disable paste only if the clipboard is empty
            UpdateMenuItem(COPY, UpdateMenuItemEnum.ENABLED, bStatus);
            copyToolStripButton.Enabled = bStatus;
            UpdateMenuItem(CUT, UpdateMenuItemEnum.ENABLED, bStatus);
            cutToolStripButton.Enabled = bStatus;

            CheckAndUpdatePasteMenuItem();
        }

        /// <summary>
        /// Checks and update the paste menu item ( as Enanle /Disable ).
        /// </summary>
        private void CheckAndUpdatePasteMenuItem()
        {
            if (Clipboard.ContainsText())
            {
                UpdateMenuItem(PASTE, UpdateMenuItemEnum.ENABLED, true);
                pasteToolStripButton.Enabled = true;
            }
            else
            {
                UpdateMenuItem(PASTE, UpdateMenuItemEnum.ENABLED, false);
                pasteToolStripButton.Enabled = false;
            }
        }

        private void EnumerateTabStopCtrls(ref SortedDictionary<int, Control> dictTabStopCtrls, Control Ctrl)
        {
            if (Ctrl.TabIndex > 0 && !dictTabStopCtrls.ContainsKey(Ctrl.TabIndex))
                dictTabStopCtrls.Add(Ctrl.TabIndex, Ctrl);

            if (Ctrl.HasChildren && Ctrl.Controls != null && Ctrl.Controls.Count > 0)
            {
                foreach (Control ctrlChild in Ctrl.Controls)
                {
                    EnumerateTabStopCtrls(ref dictTabStopCtrls, ctrlChild);
                }
            }
        }

        // look for form wide key events
        protected override bool ProcessKeyPreview(ref Message m)
        {
            return ProcessKeyEventArgs(ref m);             
        }

        // Examines keystrokes and implements shortcuts to toolStrip items
        private void Form1_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            object senderTemp = null;
            EventArgs eTemp = null;
            bool bIsEventHandled = false;

            Cursor.Current = Cursors.Default;   // insurance against accidentily leaving a WaitCursor enabled

            // Control key contributes in all shortcuts
            if (e.Control == true)
            { 
                // Examine the key code
                switch(e.KeyCode)
                {                    
                    case Keys.X:    // Cut
                        cutToolStripButton_Click(senderTemp, eTemp);
                        bIsEventHandled = true;
                        break;
                    case Keys.C:    // Copy or CopyCode
                        // Copy Code
                        if (e.Shift == true)
                        {
                            if (tsbCopyCode.Enabled == true)
                            {
                                tsbCopyCode_Click(senderTemp, eTemp);
                                bIsEventHandled = true;
                            }
                        }
                        else    // Copy
                        {
                            copyToolStripButton_Click(senderTemp, eTemp);
                            bIsEventHandled = true;
                        }
                        break;
                    case Keys.V:    // Paste
                        pasteToolStripButton_Click(senderTemp, eTemp);
                        bIsEventHandled = true;
                        break;
                    case Keys.Z:    // Undo
                        tsbUndo_Click(senderTemp, eTemp);
                        bIsEventHandled = true;
                        break;
                    case Keys.Y:    // Redo
                        tsbRedo_Click(senderTemp, eTemp);
                        bIsEventHandled = true;
                        break;
                    case Keys.M:    // Mixed Compare
                        tsbCompareToggle_Click(senderTemp, eTemp);
                        bIsEventHandled = true;
                        break;
                }
            }            
            else if (e.KeyCode == Keys.Tab && m_arrTabStopCtrls.Length > 0) 
            { 
                bool bForward = true;
                if (e.Shift == true)
                    bForward = false;
                
                bool bFoundActiveCtrl = false;
                int iTabStopCtrlIndex = 0;

                for (; iTabStopCtrlIndex < m_arrTabStopCtrls.Length; iTabStopCtrlIndex++)
                {
                    if (m_arrTabStopCtrls[iTabStopCtrlIndex].Value.Focused)
                    {
                        bFoundActiveCtrl = true;
                        break;
                    }
                }

                if (!bFoundActiveCtrl)
                    return;
                
                int iIteratingIndex = iTabStopCtrlIndex;
                Control ctrlTabStop;
                bFoundActiveCtrl = false;
                bIsEventHandled = true;
                while (true)
                {
                    if (bForward)
                    {
                        if (iIteratingIndex == (m_arrTabStopCtrls.Length - 1))
                            iIteratingIndex = 0;
                        else
                            iIteratingIndex++;
                    }
                    else
                    {
                        if (iIteratingIndex == 0)
                            iIteratingIndex = m_arrTabStopCtrls.Length - 1;
                        else
                            iIteratingIndex--;
                    }

                    ctrlTabStop = m_arrTabStopCtrls[iIteratingIndex].Value;
                    if (ctrlTabStop.CanFocus)
                        ctrlTabStop.Focus();
                    else if(ctrlTabStop.CanSelect)                    
                        ctrlTabStop.Select();                    
                    else                    
                        this.ActiveControl = ctrlTabStop;

                    if (ctrlTabStop.Focused || (iIteratingIndex == iTabStopCtrlIndex))
                    {
                        ctrlTabStop = null;
                        break;
                    }
                }               
            }
            if (bIsEventHandled)
                e.Handled = true;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            (this as ITDETool).WindowState.Dispose();
        }


        private void tsbCompareToggle_Click(object sender, EventArgs e)
        {
            // Expands or contracts the window for comparing unlike PSets / Signals
            // toggle the window width
            if (tsbCompareToggle.DisplayStyle == ToolStripItemDisplayStyle.Text)
            {
                tsbCompareToggle.DisplayStyle = ToolStripItemDisplayStyle.ImageAndText;
                pnlRight.Visible = true;
                splLeftRight.Visible = true;
                splLeftRight.SplitPosition = pnlLeft.Width;

                this.Width = this.Width + 417; // expand form
            }
            else
            {
                tsbCompareToggle.DisplayStyle = ToolStripItemDisplayStyle.Text;
                splLeftRight.SplitPosition = pnlLeft.Width;
                pnlRight.Visible = false;

                splLeftRight.Visible = false;
                if (this.Width - 417 < 381)
                    this.Width = 381; // minimum width to show Edit groupbox
                else
                    this.Width = this.Width - 417; // contract form  

                pnlLeft.Width = this.Width;
                gbxEditPSet.Width = this.Width - 29; // Note: adjusting width by enabling/disabling Right anchor does *not* work

            }

            //update the hosting proxy's size
            Proxy.Size = this.Size;


            //This is used to resize the window, 
            //according to enable and disable of MixedComp Toggle button
            m_TDE.ToolSizeChanged(m_InstanceTag);
        }


        private void dgvParametersEdit_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            //Check if value has changed.
            String currValue = "";
            String nspcpName;
            NspcpItem obj;

            m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
            if (dgvParametersEdit[e.ColumnIndex, e.RowIndex].Value != null)
            {
            currValue = (String)dgvParametersEdit[e.ColumnIndex, e.RowIndex].Value;
            }

            
                if (!currValue.Equals(m_cellCache))
                {
                    String oldValue = m_cellCache; // Keep a copy as this will be overwritten when calling reconfigure grid

                    //If the value was changed, we change the text to blue...
                    dgvParametersEdit.CurrentCell.Style.ForeColor = GlobalUtils.CELL_CHANGE_COLOR;

                    //sets the BackColor of the current cell
                    dgvParametersEdit.CurrentCell.Style.BackColor = SystemColors.Control;

                    //Set the 'dirty bit' to true
                    m_DefChanged_NotAppliedToDriver = true;
                    m_DefChanged_NotAppliedToSheet = true;
                    //Enable/Disable the Apply PSet/Load Signal Buttons.
                    configureButtons();

                    //... and configure the nspcp column.
                    if (e.ColumnIndex > 0)
                    {
                        nspcpName = dgvParametersEdit.Columns[e.ColumnIndex].Name;
                        //Refresh the objects.
                        m_UIEngine.CollectGridInformation(ref m_NSPCPList);
                        //Get object
                        obj = m_NSPCPList[nspcpName];
                        //Configure the grid.
                        //HandleGridConfiguration(obj);
                        ReConfigureGrid(obj,ref m_Main_InstrEngine);
                        // Save the state change and update the UI
                        m_undoManager.SaveState(this, new GridUndoInfo(e.RowIndex, e.ColumnIndex, oldValue, currValue));
                        EnableDisableUndoRedoButtons();
                    }

                    //If the wave definition value has changed, cache it.
                    if (dgvParametersEdit[0, e.RowIndex].Value.Equals("Wave Def or Data"))
                    {
                        m_currWaveDefName = (String)dgvParametersEdit[e.ColumnIndex, e.RowIndex].Value;
                    }
                }
            }


        


        private void cboEditSource_SelectedIndexChanged(object sender, EventArgs e)
        {
            string strCurrentSource;
            strCurrentSource = cboSource.Text;
            if (m_lastSource != null)
            {
                if(m_lastSource.Equals(strCurrentSource))
                {
                    //user selected the same source , so ignore this
                    return;
                }
                else
                {
                    //make sure user is OK with losing changes, if any. 
                    if (CheckForUnsavedDefinitions())
                    {
                        cboSource.Text = m_lastSource;
                        return;
                    }
                       
                }
            }                
               

            // Changes the control labels depending on whether the selected edit Source is PSet or Signal
            switch (cboSource.SelectedIndex)
            {
                case PSET:
                    btnPSetsOrSignals.Text = PSET_BUTTON_TEXT;
                    lblParameters.Text = "P&Set parameters:";
                    toolTip1.SetToolTip(btnNewPSetOrSignal, NEW_PSET_TEXT);
                    HandleNSPCPTypeSelection(GlobalUtils.NspcpType.pset, ref m_GlobalSettings.MainPanel, ref m_Main_InstrEngine);
                    break;
                case SIGNAL:
                    btnPSetsOrSignals.Text = SIGNAL_BUTTON_TEXT;
                    lblParameters.Text = "Si&gnal parameters:";
                    toolTip1.SetToolTip(btnNewPSetOrSignal, NEW_SIGNAL_TEXT);
                    HandleNSPCPTypeSelection(GlobalUtils.NspcpType.signal, ref m_GlobalSettings.MainPanel, ref m_Main_InstrEngine);
                    break;
            }
            //Clear the parameter grid (in case there are any existing nspcp's being displayed)
            ClearMainParameterGrid();
            m_lastSource = cboSource.Text;

            //Disable Cut and Copy, but not the Paste. Only disable paste if clipboard is empty.
            UpdateMenuItemStatus(false);
        }

        private void cboCompareSource_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Changes the control labels depending on whether the selected compare Source is PSet or Signal
            switch (cboMixedSource.SelectedIndex)
            {
                case PSET:
                    btnMixedPSetsOrSignals.Text = PSET_BUTTON_TEXT;
                    lblMixedParameters.Text = "PSet pa&rameters:";
                    HandleNSPCPTypeSelection(GlobalUtils.NspcpType.pset, ref m_GlobalSettings.ComparePanel, ref  m_Compare_InstrEngine);
                    break;
                case SIGNAL:
                    btnMixedPSetsOrSignals.Text = SIGNAL_BUTTON_TEXT;
                    lblMixedParameters.Text = "Signa&l parameters:";
                    HandleNSPCPTypeSelection(GlobalUtils.NspcpType.signal, ref m_GlobalSettings.ComparePanel, ref m_Compare_InstrEngine);
                    break;
            }
            //Clear the parameter grid (in case there are any existing nspcp's being displayed)
            ClearCompareParameterGrid();
        }

        private void ClearCompareParameterGrid()
        {
            if (m_CompUiEngine.IsInitialized)
            {
                //Do not clear anything.
                while (dgvCompareParamEdit.Columns.Count > 1)
                {
                    // Clear any current PSet/Signal columns, but NOT Column 0; they'll be replaced by the new selection
                    dgvCompareParamEdit.Columns.Remove(dgvCompareParamEdit.Columns[dgvCompareParamEdit.Columns.Count - 1]);
                }

                dgvCompareParamEdit.Rows.Clear();

                //Clear the existing list (this operation will now occur when we reset the display)
                m_CompNspcpList.Clear();

                //Clear the list of selected nspcps
                if (m_GlobalSettings.ComparePanel.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                {
                    m_GlobalPSetSignalValues.ComparePanel.PSetValues.Clear();
                }
                else
                {
                    m_GlobalPSetSignalValues.ComparePanel.SignalValues.Clear();
                }

                //Set initialization to false.
                m_CompUiEngine.IsInitialized = false;
            }
        }

        private void enablePSetSignal(String pinName, String instrumentName,
                                      ref System.Windows.Forms.Button psetSignalsButtonControl)
        {
            if ((!pinName.Equals(DEFAULT_TEXT_PIN) || (!instrumentName.Equals(DEFAULT_TEXT_INST))))
            {
                psetSignalsButtonControl.Enabled = true;
            }
            else
            {
                psetSignalsButtonControl.Enabled = false;
            }
        }


        private String GetCompareId()
        {
            String itemList = "";
            foreach (NspcpItem obj in m_CompNspcpList.Values)
            {
                itemList = obj.propName;
            }
            return (String.Concat(cboMixedSource.Text.Trim(), itemList.ToUpper(), txtMixedPinName.Text.Trim(), cboMixedInstrument.Text.Trim()).ToUpper());
        }

        private void ClearMainParameterGrid()
        {
            if (m_UIEngine.IsInitialized)
            {
                //Do not clear anything.
                while (dgvParametersEdit.Columns.Count > 1)
                {
                    // Clear any current PSet/Signal columns, but NOT Column 0; they'll be replaced by the new selection
                    dgvParametersEdit.Columns.Remove(dgvParametersEdit.Columns[dgvParametersEdit.Columns.Count - 1]);
                }

                dgvParametersEdit.Rows.Clear();

                //Clear the existing list (this operation will now occur when we reset the display)
                m_NSPCPList.Clear();

                //Clear the list of selected nspcps (both psets and signals)
                m_GlobalPSetSignalValues.MainPanel.PSetValues.Clear();
                m_GlobalPSetSignalValues.MainPanel.SignalValues.Clear();

                //Clear the error grid as well
                m_UIEngine.ClearAll();

                //Disable the tsbCopyCode button on clearing the grid.
                configureCopyCodeButton();

                //Set initialization to false.
                m_UIEngine.IsInitialized = false;
                //Set Dirty flag to false
                m_DefChanged_NotAppliedToDriver = m_DefChanged_NotAppliedToSheet=false;
            }
        }



        private void frmPSetSignalsEditor_ResizeEnd(object sender, EventArgs e)
        {
            // No max size, but do set a min width.            
            if (this.Width < 389)
                this.Width = 389;

            // for design time testing only...
            //Console.WriteLine("Form width: " + this.Width.ToString());
        }    
    
        private void txtPinName_Enter(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();
             
            ShrunkErrorPane();

            m_bEnterTxtPinName = true;
        }

        private void txtPinName_MouseClick(object sender, MouseEventArgs e)
        {
            if (m_bEnterTxtPinName == true)
                txtPinName.SelectAll();
            m_bEnterTxtPinName = false;

        }

        private void btnSelectPinsGroups_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();

             
            ShrunkErrorPane();

            m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
            if (CheckForUnsavedDefinitions())
                return;
            else
            {
                m_DefChanged_NotAppliedToSheet = false;
                m_DefChanged_NotAppliedToDriver = false;
            }

            Cursor.Current = Cursors.WaitCursor;    // reset inside pin selector form
            m_Main_InstrEngine.Rebuild();
            if (HandleSelectPinsGroupsClick(cboInstrument.Text, ref txtPinName,ref m_Main_InstrEngine))
            {
                Cursor.Current = Cursors.WaitCursor;
                enablePSetSignal(txtPinName.Text, cboInstrument.Text, ref btnPSetsOrSignals);
                //Clear the parameter grid.
                ClearMainParameterGrid();
                
                // Set Instrument type in Instrument Combo
                cboInstrument.Items.Clear();  // Clear the Combobox list, if there exists any instruments for previous pin
                
                SetInstrumentsForPins(true);
                
                // Select the default instrument name when the pin is selected.
                if (cboInstrument.Items.Count > 0)
                {
                    cboInstrument.SelectedIndex = 0;
                }

                //Configure(ie., enables/disables) the apply pset,load signals and Update Tester buttons
                configureButtons();
            }
            Cursor.Current = Cursors.Default;
        }

        private void SetInstrumentsForPins(Boolean MainPanel)
        {
            try
            {
                String pinList, txtPinList;
                if (MainPanel)
                    txtPinList = txtPinName.Text;
                else
                    txtPinList = txtMixedPinName.Text;

                if ((txtPinList.Equals(DEFAULT_TEXT_PIN)) || (txtPinList.Equals("")))
                {
                    pinList = "";
                }
                else
                {
                    // remove spaces from pinName (in case if pinName is CSL of pins)
                    pinList = txtPinList.Replace(" ", "");
                    if (MainPanel)
                    {
                        if (!m_Main_InstrEngine.CheckPin(ref pinList))
                        {
                            MessageBox.Show("Invalid Pin Selected.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
                    else
                    {
                        if (!m_Compare_InstrEngine.CheckPin(ref pinList))
                        {
                            MessageBox.Show("Invalid Pin Selected.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
                }

                if (MainPanel)
                {
                    UpdateInstrumentCombo(cboInstrument, m_Main_InstrEngine.GetInstruments(pinList));
                }
                else
                {
                    UpdateInstrumentCombo(cboMixedInstrument, m_Compare_InstrEngine.GetInstruments(pinList));
                }                 
            }
            catch (Exception exp)
            {
                Console.Write("Exception caught in SetInstrumentsForPins : {0}", exp.StackTrace);
            }

        }
        
        /// <summary>
        /// Updates the combo box with new instrument list while maintaining the current selection if applicable
        /// </summary>
        /// <param name="cboInst">The combo box to update</param>
        /// <param name="instrList">A list of new instruments to populate</param>
        private void UpdateInstrumentCombo(ComboBox cboInst, String[] instrList)
        {
            String item = cboInst.SelectedIndex == -1 ? null : cboInst.SelectedItem.ToString();
            cboInst.Items.Clear();

            if (instrList != null && instrList.Length > 0)
            {
                cboInst.Items.AddRange(instrList);
                if (String.IsNullOrEmpty(item) || cboInst.FindString(item) == -1)
                    cboInst.SelectedIndex = 0;
                else
                    cboInst.SelectedIndex = cboInst.FindString(item);
            }
        }
        
        #region methods to enable/disable buttons



        /// <summary>
        /// This method enables/disables the apply pset,load signals and Update Tester buttons
        /// depending upon the current state of the editor.
        /// </summary>
        private void configureButtons()
        {
            bool bEnable, bEnableLoadSignals;
            bool isConcurrentFlow= false;
            if(MyServices.FlowDomain.ConcurrentContext != 0)
            {
                isConcurrentFlow= true;
            }

            //Handle settings for the 'Update Tester' button.
            if (m_UIEngine.IsInitialized && IsSitesActive(GetSelectedSite()) && (m_NSPCPList.Count > 0))
            {
                //Ensure whether the test is Serial or Concurrent Test
                if (isConcurrentFlow == false)
                {
                    bEnable = true;
                }
                else
                {
                    bEnable = false;
                }

                if (m_DefChanged_NotAppliedToDriver)
                {
                    btnUpdateTester.Text = GlobalUtils.UPDATE_TESTER_DIRTY;
                }
                else
                {
                    btnUpdateTester.Text = GlobalUtils.UPDATE_TESTER;
                }
                if (m_DefChanged_NotAppliedToSheet)
                {
                    btnUpdateSheet.Text = GlobalUtils.UPDATE_SHEET_DIRTY;
                }
                else
                {
                    btnUpdateSheet.Text = GlobalUtils.UPDATE_SHEET;
                }
            }
            else //Default selected button is the 'Update Sheet' button.
            {
                bEnable = false;                
            }
            
            btnUpdateTester.Enabled = bEnable;
            UpdateMenuItem(SETDRIVER, UpdateMenuItemEnum.ENABLED, bEnable);  // Update menu item

            bEnable = bEnableLoadSignals = false;

            //Handle settings for the Apply PSet / Load Signal Buttons
            if (m_UIEngine.IsInitialized && !m_DefChanged_NotAppliedToDriver &&
                (m_NSPCPList.Count > 0) && IsSitesActive(GetSelectedSite()))
            {
                if (m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                {
                    //Ensure whether the test is Serial or Concurrent Test
                    if (isConcurrentFlow == false)
                    {
                        bEnable = true;
                    }
                }
                else
                {
                    //Ensure whether the test is Serial or Concurrent Test
                    if (isConcurrentFlow == false)
                    {

                    bEnableLoadSignals = true;
            }
                }
            }

            if (m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset)
            {
                btnApplyPSet.Text = "Apply PSet";
                btnApplyPSet.Enabled = bEnable;
            }
            else
            {

                this._rcInfo = UpdateLoadSettingButtonText();

                btnApplyPSet.Text = _rcInfo.ButtonName;

                if (_rcInfo.IsToolTipAvailable && this.IsLoadSettingButtonHover)
                {
                    ShowResourceConflict();
                }
                else
                {
                    loadToolTip.Visible = false;
                }

                if (this._rcInfo.IsEnabled)
                {
                    btnApplyPSet.ForeColor = SystemColors.ControlText;
                }
                else
                {
                    btnApplyPSet.ForeColor = SystemColors.GrayText;
                }

                btnApplyPSet.Enabled = bEnableLoadSignals;
            }

            tsmiActionsApplyPSet.Enabled = bEnable;
            UpdateMenuItem(APPLYPSET, UpdateMenuItemEnum.ENABLED, bEnable);  // Update menu item
            tsmiActionsLoadSignals.Enabled = bEnableLoadSignals;
            UpdateMenuItem(LOADSIGNALSETTINGS, UpdateMenuItemEnum.ENABLED, bEnableLoadSignals);  // Update menu item            

            if (!m_UIEngine.IsInitialized)
            {
                btnApplyPSet.Enabled = btnUpdateTester.Enabled = btnUpdateSheet.Enabled = false;
                UpdateMenuItem(UPDATESHEET, UpdateMenuItemEnum.ENABLED, false);  // Update menu item
            }
            else
            {
                btnUpdateSheet.Enabled = true;
                UpdateMenuItem(UPDATESHEET, UpdateMenuItemEnum.ENABLED, true);  // Update menu item
                
            }

            // Enable 'Duplicate' button only if exactly one nspcp column is present (plus names column)
            bEnable = false;
            if (dgvParametersEdit.Columns.Count == 2)
                bEnable = true;
            btnDuplicatePSetOrSignal.Enabled = bEnable;
            UpdateMenuItem(DUPLICATEPSET, UpdateMenuItemEnum.ENABLED, bEnable);  // Update menu item
        }
        /// <summary>
        /// Shows the Resource conflict tool tip
        /// </summary>
        private void ShowResourceConflict()
        {
            if (this._rcInfo != null &&
                this._rcInfo.IsToolTipAvailable &&
                (!String.IsNullOrEmpty(_rcInfo.Title)) &&
                (!String.IsNullOrEmpty(_rcInfo.Body)))
            {
                lblTitle.Text = _rcInfo.Title;
                bodyText.Clear();
                bodyText.AppendText(_rcInfo.Body);
                loadToolTip.Visible = true;
                loadToolTip.Show(btnApplyPSet);
            }
            else
            {
                loadToolTip.Visible = false;
            }
        }
        /// <summary>
        /// Retuns the updated LoadSettingsButton face text
        /// </summary>
        /// <returns>returns the update button information</returns>
        private BalloonTipInfo UpdateLoadSettingButtonText()
        {
            
            if (!m_UIEngine.IsInitialized ||(m_NSPCPList == null || !(m_NSPCPList.Count > 0)))
            {
                return BalloonTipInfo.Empty;
            }
            
            try
            {
                //make call to first sucessfull item and return
                foreach (String name in m_NSPCPList.Keys)
                {
                    if (m_NSPCPList.ContainsKey(name))
                    {
                        string buttonName;
                        string title;
                        string toolTip;
                        int buttonEnabled;

                        NspcpItem obj = m_NSPCPList[name];
                        m_Main_InstrEngine.UpdateLoadSettingsButton(obj,out buttonName,out title,out toolTip,out buttonEnabled);

                        return new BalloonTipInfo(buttonName,(buttonEnabled == 0) ? false : true, title, toolTip);
                    }
                }
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Update LoadSetting Button Text Failed", exp.Message);
            }
            //return the default value
            return BalloonTipInfo.Empty;
        }

        #endregion

        private Boolean HandleSelectPinsGroupsClick(String instrument, ref System.Windows.Forms.TextBox txtPinNameControl, ref InstrumentEngine instr)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();

            ShrunkErrorPane();

            String instrName = " ";
            if (instrument.Equals(DEFAULT_TEXT_INST))
            {
                instrName = "";
            }
            else
            {
                instrName = instrument;
            }

            String pinList, txtPinList;
            txtPinList = txtPinNameControl.Text;
            if (txtPinList.Equals(DEFAULT_TEXT_PIN))
                txtPinList = "";
            // remove spaces from pinName (in case if pinName is CSL of pins)
            pinList = txtPinList.Replace(" ", "");
            // Bring up Pin / Group selector
            Form pgSelector=new frmPinsGroupsSelector(ref instr, instrName, pinList);
            DialogResult res = pgSelector.ShowDialog();
            if (res == DialogResult.Cancel)
                return false; // do nothing
            if (res == DialogResult.OK)
            {
                String newList = ((frmPinsGroupsSelector)pgSelector).PinGroupList;
                if (CheckIdenticalPinList(pinList, newList)) return false;

                txtPinNameControl.Text = newList;
                //Check if pinlist remains the same 
            }
            pgSelector.Dispose();
            return true;
        }


        /// <summary>
        /// Checks if the list of pins passed are identical or not.
        /// </summary>
        /// <param name="pinList">old list</param>
        /// <param name="newList">new list</param>
        /// <returns></returns>
        private bool CheckIdenticalPinList(string pinList, string newList)
        {
            //Tokenize both of the lists;
            if (newList == null || pinList == null) return false;

            pinList = pinList.ToLower();
            newList = newList.ToLower();
            Boolean match;
            
            String[] oldList = pinList.Split(',');
            String[] list = newList.Split(',');
            if (oldList.Length != list.Length) return false;
            foreach (String ele1 in oldList)
            {
                match = false;
                foreach (String ele2 in list)
                {
                    if (ele1.Equals(ele2)) match = true;
                }
                if (!match) return false;
            }
            return true;
        }


        private void btnMixedSelectPinsGroups_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            m_Compare_InstrEngine.UpdateHelper(m_GlobalSettings.ComparePanel.NSPCP_MODE);
            m_Compare_InstrEngine.Rebuild();

            if (HandleSelectPinsGroupsClick(cboMixedInstrument.Text, ref txtMixedPinName,ref m_Compare_InstrEngine))
            {   // User changed pinlist
                Cursor.Current = Cursors.WaitCursor;

                enablePSetSignal(txtMixedPinName.Text, cboMixedInstrument.Text, ref btnMixedPSetsOrSignals);
                //Clear the parameter grid.
                ClearCompareParameterGrid();

                // Set Instrument type in Instrument Combo
                cboMixedInstrument.Items.Clear();  // Clear the Combobox list, if there exists any instruments for previous pin

                SetInstrumentsForPins(false);

                // Select the default instrument name when the pin is selected.
                if (cboMixedInstrument.Items.Count > 0)
                {
                    cboMixedInstrument.SelectedIndex = 0;
                }
            }
            Cursor.Current = Cursors.Default;
        }

        
        private List<String> GetNSPCPList(String pinName, String instrName, GlobalUtils.DATA_SOURCE source,ref InstrumentEngine instr)
        {
            List<String> nspcpList = new List<String>();
            List<string> errors = new List<string>();

            // Load the available PSets or Signals into lstAvailable ..
            if (source == GlobalUtils.DATA_SOURCE.DRIVER)
            {
                Boolean gotOriginal = false;
                int selectedSite = GetSelectedSite();
                SiteWrapper.SetOverride(selectedSite);
                gotOriginal = true;

                try
                {
                    nspcpList = instr.GetList(pinName, instrName, ref errors);
                }
                catch (Exception exp)
                {
                    ThrowErrorMessage("Error retrieving PSet/Signal List", exp.Message);
                    return null;
                }
                finally
                {
                    //Restore sites.
                    if (gotOriginal)
                    {
                        SiteWrapper.RestoreFromOverride();
                    }
                }
            }
            else
            {
                try
                {
                    String[] sheetList = m_SheetEngine.GetList(pinName, instrName);
                    nspcpList.AddRange(sheetList);
                }
                catch (SheetNotPresentException exp)
                {
                    ThrowErrorMessage("Error retrieving PSet/Signal List", exp.Message);
                    return null;
                }
                catch (Exception exp)
                {
                    ThrowErrorMessage("Error retrieving PSet/Signal List", exp.Message);
                    return null;
                }
            }

            return nspcpList;
        }


        /// <summary>
        /// This method handles a Select PSet/Signals button click for either panel
        /// </summary>
        private void HandlePSetOrSignalsButtonClick(String pinName, String instrument,
                        ref PanelSettings settings, ref PanelNspcpValues values,
                        ref Dictionary<String, NspcpItem> items, ref UserInterfaceEngine UI, ref NspcpItemsGridView dgvParams)
        {
            if (!settings.READONLY)
                if (CheckForUnsavedDefinitions()) return;

            //Clear all existing errors.
            m_UIEngine.ClearAll();

            ShrunkErrorPane();

            if (pinName.Equals("") || instrument.Equals(""))
            {
                MessageBox.Show("Please select a valid pin and instrument name before adding a PSet or Signal.",this.Name,MessageBoxButtons.OK,MessageBoxIcon.Information);
                return;
            }

            // remove spaces from pinName (in case if pinName is CSL of pins)
            String pinList;
            pinList = pinName.Replace(" ", "");

            InstrumentEngine instr;
            if (settings.READONLY)
                instr = m_Compare_InstrEngine;
            else
                instr = m_Main_InstrEngine;
           
            instr.Rebuild();

            if (!instr.CheckPin(ref pinList))
            {
                MessageBox.Show("Selected pin name is invalid.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
            if (!instr.CheckInstrument(instrument, true, pinList))
            {
                MessageBox.Show("Selected instrument is invalid or does not map to the selected pin.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            if (settings.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                m_SheetEngine = new PSetSheetEngine(ref psetHelper);
            else
                m_SheetEngine = new SignalsSheetEngine(ref signalHelper);
            
            List<string> psetSignalList = GetNSPCPList(pinList, instrument, settings.DATA_SOURCE,ref instr);
            if (dgvParams.ColumnCount > 1)
            {
                //Grid Contains NSPCP Items, so catch the Nspcp names using DisplayIndex property
            m_GlobalPSetSignalValues.globalNspcpValues.Clear();
                SortedDictionary<int,string> panelDict = new SortedDictionary<int,string>();
                foreach (DataGridViewColumn column in dgvParams.Columns)
                {
                    if (column.Name != FIRST_PARAMETER_COLUMN_NAME) //excluding parameter name column 
                    {
                        panelDict.Add(column.DisplayIndex,column.Name);
                    }
                }
                foreach(string nspcpName in panelDict.Values)
                {
                    m_GlobalPSetSignalValues.globalNspcpValues.Add(nspcpName);
                }

            }
            else
            {
                m_GlobalPSetSignalValues.globalNspcpValues.Clear();
            }

            if (psetSignalList != null)
            {
                System.Collections.ArrayList arrList = (System.Collections.ArrayList)m_GlobalPSetSignalValues.globalNspcpValues.Clone();

                Form pgPSetsSignalsSelector = new frmPSetsSignalsSelector(psetSignalList, ref m_GlobalPSetSignalValues);
                DialogResult res = pgPSetsSignalsSelector.ShowDialog();

                if (res == DialogResult.Cancel)
                    return; // do nothing
                if (res == DialogResult.OK)
                {
                    bool bIsSelectionChanged = false;
                    if (arrList.Count != m_GlobalPSetSignalValues.globalNspcpValues.Count)
                    {
                        bIsSelectionChanged = true;
                    }
                    else
                    {
                        for (int i = 0; i < arrList.Count; i++)
                        {
                            if (!arrList[i].Equals(m_GlobalPSetSignalValues.globalNspcpValues[i]))
                            {
                                bIsSelectionChanged = true;
                                break;
                            }
                        }
                    }

                    if (bIsSelectionChanged == true)
                    {
                        if (settings.READONLY == true)
                            ClearCompareParameterGrid();
                        else
                            ClearMainParameterGrid();

                        dgvParams.AutoSizeColumnsMode = System.Windows.Forms.DataGridViewAutoSizeColumnsMode.AllCells;
                        if (settings.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                        {
                            values.PSetValues.Clear();
                            values.PSetValues.AddRange(m_GlobalPSetSignalValues.globalNspcpValues);
                        }
                        else
                        {
                            values.SignalValues.Clear();
                            values.SignalValues.AddRange(m_GlobalPSetSignalValues.globalNspcpValues);
                        }
                        //Get the list of NSPCPs that need to be displayed. 
                        System.Collections.ArrayList tmpArrayList = HandleRemoveColumns(ref items, ref UI, ref settings, ref values);
                        HandleSelectedNspcp(pinList, instrument, ref tmpArrayList, ref settings, ref values, ref instr);
                        CompareNspcps();    // Compare NSPCPs if the Compare panel is visible
                        configureButtons();

                        // undo updates
                        m_undoManager.Clear();
                        EnableDisableUndoRedoButtons();
                        configureCopyCodeButton();
                        // When there are any NSPCP items selected, update pin name
                        if (tmpArrayList.Count > 0)
                            m_strPinNames = pinList;
                    }

                    pgPSetsSignalsSelector.Dispose();
                }
            }
        }

        private void btnPSetsOrSignals_Click(object sender, EventArgs e)
        {
            m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);            
            HandlePSetOrSignalsButtonClick(txtPinName.Text, cboInstrument.Text,
                        ref m_GlobalSettings.MainPanel, ref m_GlobalPSetSignalValues.MainPanel,
                        ref m_NSPCPList, ref m_UIEngine, ref dgvParametersEdit);
        }

        /// <summary>
        /// This method refreshes the list of available nspcps for either panel
        /// Returns true if there was a change.
        /// </summary>
        private Boolean RefreshPSetSignalList(ref PanelSettings settings, ref PanelNspcpValues values,
                                String pin, String instrument, ref InstrumentEngine instr)
        {
            Boolean change = false;
            System.Collections.ArrayList DuplicateList;
            if (settings.NSPCP_MODE == GlobalUtils.NspcpType.pset)
            {
                DuplicateList = (System.Collections.ArrayList)values.PSetValues.Clone();
                m_SheetEngine = new PSetSheetEngine(ref psetHelper);                   
            }
            else
            {
                DuplicateList = (System.Collections.ArrayList)values.SignalValues.Clone();
                m_SheetEngine = new SignalsSheetEngine(ref signalHelper);
            }

            //Update the global values variables
            int PSetsRemaining = DuplicateList.Count;
            int SignalsRemaining = DuplicateList.Count;  // tersw00175952 both PSetsRemaining and SignalsRemaining are set to DuplicateList.Count because
                                                         // this method does not distinguish between the Main and Compare panels when doing its refresh of global values.
            for (int nItems = FIRST_ITEM; nItems < DuplicateList.Count; nItems++)
            {
                if (!ExistsNSPCP(pin, instrument, DuplicateList[nItems].ToString(), settings.DATA_SOURCE, true, ref instr))
                {
                    change = true;
                    //The PSet/Signal does not exist in that mode anymore
                    //1. Remove it from global vlues
                    if (settings.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                    {
                        values.PSetValues.Remove(DuplicateList[nItems].ToString());
                        PSetsRemaining--;                     
                    }
                    else
                    {
                        values.SignalValues.Remove(DuplicateList[nItems].ToString());
                        SignalsRemaining--;              
                    }
                }
            }
            // tersw00175952   Clear the parameter names column in the grids, but only if there are no PSets or Signals remaining.
            //                 Both grids are cleared because the DuplicateList used in this method does not distinguish which grid they're in.
            if ((PSetsRemaining < 1) ||
                (SignalsRemaining < 1))                
            {
                ClearMainParameterGrid();
                ClearCompareParameterGrid();
            }
            return change;
        }  // RefreshPSetSignalList        
        
        /// <summary>
        /// Handle a Select PSet/Signal button press for either panel.
        /// </summary>
        private void HandleSelectedNspcp(string PinName, string InstrName, ref System.Collections.ArrayList tmpArrayList,
                        ref PanelSettings settings, ref PanelNspcpValues values, ref InstrumentEngine instr)
        {
            Boolean gotOriginal = false;
            try
            {
                if (settings.DATA_SOURCE == GlobalUtils.DATA_SOURCE.DRIVER) //Select sites.
                {
                    int selectedSite = GetSelectedSite();
                    SiteWrapper.SetOverride(selectedSite);
                    gotOriginal = true;
                }

                //Loop through all the defintions that need to be displayed.
                for (int nItems = FIRST_ITEM; nItems < tmpArrayList.Count; nItems++)
                {
                    NspcpItem definitionObj = null;    //Initialize the variable.                
                    List<String> errors = new List<string>();

                    //Read NSPCP definition from Driver/Sheet (Depending upon data source selected).
                    if (settings.DATA_SOURCE == GlobalUtils.DATA_SOURCE.DRIVER)
                    {
                        definitionObj = ReadDefinitionFromDriver(PinName, InstrName, tmpArrayList, nItems, settings.NSPCP_MODE, ref errors, ref instr);
                    }
                    else
                    {
                        definitionObj = ReadDefinitionFromSheet(PinName, InstrName, tmpArrayList, nItems, settings.NSPCP_MODE);
                    }

                    if (definitionObj != null)  //Initialize and populate the display
                    {
                        if (settings.Equals(m_GlobalSettings.MainPanel))
                            InitializeAndPopulateMainDisplay(definitionObj);
                        else
                            InitializeAndPopulateCompareDisplay(definitionObj);
                    }
                    else //Error receiving definition for the object
                    {
			            string strErrorMessage = String.Empty;

                        if (settings.DATA_SOURCE == GlobalUtils.DATA_SOURCE.DRIVER)
                        {
                            if(errors.Count > 0)
                            {
                                string strSeparator = "";
                                foreach (string err in errors)
                                {
                                    strErrorMessage = strSeparator + err;
                                    strSeparator = ",";
                                }
                            }
                        }
                        else
                            strErrorMessage = "Definition could not be displayed.";

                        ThrowErrorMessage(tmpArrayList[nItems].ToString(), strErrorMessage);
                        //Remove the nspcp name from the global values. 
                        //Since we are unable to display the definition.
                        if (settings.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                        {
                            values.PSetValues.Remove(tmpArrayList[nItems].ToString());
                        }
                        else
                        {
                            values.SignalValues.Remove(tmpArrayList[nItems].ToString());
                        }
                    }
                }
            }
            catch (Exception exp)
            {
                ClearMainParameterGrid();
                ThrowErrorMessage("Exception while populating display", exp.Message);
            }
            finally
            {
                //Restore sites.
                if (gotOriginal)
                {
                    SiteWrapper.RestoreFromOverride();
                }
                MyServices.CoreLanguageServiceCOM.DidExecute = false;
            }
        }  // HandleSelectedNspcp

        /// <summary>
        /// Initialize and Populate the Compare display with the NSPCP definition in the object.
        /// </summary>
        /// <param name="definitionObj">NSPCP Object definition</param>
        private void InitializeAndPopulateCompareDisplay(NspcpItem definitionObj)
        {
            //Check if the display is initialized with parameter names
            if (!m_CompUiEngine.IsInitialized)
            {
                //If not, then initialize...
                m_CompUiEngine.InitDisplay(definitionObj.NameValuePairs.Keys);
                //and mark display as initialized.
                m_CompUiEngine.IsInitialized = true;
            }
            //Initialize the column which will hold the nspcp definition.
            m_Compare_InstrEngine.UpdateHelper(m_GlobalSettings.ComparePanel.NSPCP_MODE);

            m_CompUiEngine.AddColumn(definitionObj);

            //Configure the plugin.
            m_Compare_InstrEngine.Configure(ref definitionObj);

            //Populate the parameter grid with the definition just extracted.
            m_CompUiEngine.PopulateColumnWithoutDropdowns(definitionObj);


            //Add the object definition to the list of existing nspcp definitions being displayed.
            if (!m_CompNspcpList.ContainsKey(definitionObj.propName))
            {
                m_CompNspcpList.Add(definitionObj.propName, definitionObj);
            }
        }

        /// <summary>
        /// Initialize and Populate the display with the NSPCP definition in the object.
        /// </summary>
        /// <param name="definitionObj">NSPCP Object definition</param>
        private void InitializeAndPopulateMainDisplay(NspcpItem definitionObj)
        {
            try
            {
                m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
                //Check if the display is initialized with parameter names
                if (!m_UIEngine.IsInitialized)
                {
                    //If not, then initialize...
                    m_UIEngine.InitDisplay(definitionObj.NameValuePairs.Keys);
                    //and mark display as initialized.
                    m_UIEngine.IsInitialized = true;
                }
                //Initialize the column which will hold the nspcp definition.
                m_UIEngine.AddColumn(definitionObj);
                
                //Configure the plugin
                Boolean[] enables = m_Main_InstrEngine.Configure(ref definitionObj);

                //Populate the parameter grid with the definition just extracted.
                m_UIEngine.PopulateColumn(definitionObj);

                //After Population, enable disable the cells
                m_UIEngine.ConfigureCells(enables, definitionObj.propName);

                //Add the object definition to the list of existing nspcp definitions being displayed.
                if (!m_NSPCPList.ContainsKey(definitionObj.propName))
                {
                    m_NSPCPList.Add(definitionObj.propName, definitionObj);
                }

                //Check if Apply/Load button can be enabled or not.
                configureButtons();
                configureCopyCodeButton();
            }
            catch (Exception exp)
            {
                throw new Exception("Error Initializing Display " + exp.Message);
            }

        }

        /// <summary>
        /// Read the NSPCP Definition from the driver
        /// </summary>
        /// <param name="PinName"></param>
        /// <param name="InstrName"></param>
        /// <param name="tmpArrayList"></param>
        /// <param name="nItems"></param>
        /// <param name="errors"></param>
        /// <returns></returns>
        private NspcpItem ReadDefinitionFromDriver(string PinName, string InstrName, System.Collections.ArrayList tmpArrayList, int nItems, GlobalUtils.NspcpType NspcpType, ref List<String> errors, ref InstrumentEngine instr)
        {
            NspcpItem tmpObj = null;
           
            string pinList;
            pinList = PinName.Replace(" ", "");
               
            try
            {

                tmpObj = instr.GetDefiniton(pinList, InstrName, tmpArrayList[nItems].ToString(), NspcpType, ref errors);
            }
            catch (IncompatibleNSPCPException exp)
            {
                MessageBox.Show(exp.Message, GlobalUtils.MESSAGEBOX_CAPTION);
                return null;
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Error Reading from Driver", exp.Message);
                return null;
            }
            
            return tmpObj;
        }

        /// <summary>
        /// Read the NSPCP Definition from the Sheet.
        /// </summary>
        /// <param name="PinName"></param>
        /// <param name="InstrName"></param>
        /// <param name="tmpArrayList"></param>
        /// <param name="nItems"></param>
        /// <returns></returns>
        private NspcpItem ReadDefinitionFromSheet(string PinName, string InstrName, 
                        System.Collections.ArrayList tmpArrayList, int nItems, GlobalUtils.NspcpType type)
        {
            NspcpItem tmpObj;

            if (type.Equals(GlobalUtils.NspcpType.pset))
            {
                m_SheetEngine = new PSetSheetEngine(ref psetHelper);
            }
            else
            {
                m_SheetEngine = new SignalsSheetEngine(ref signalHelper);
            }

            try
            {
                tmpObj = m_SheetEngine.ReadSheet(PinName, InstrName, tmpArrayList[nItems].ToString());
            }
            catch (ArgumentNullException)
            {
                // Either pinanme or instrname or nspcpname are null
                ThrowErrorMessage("Sheet Read Error", "Either pinanme or instrname or pset/signal name are null");
                return null;
            }
            catch (KeyNotFoundException)
            {
                // PSet/Signal Sheet has an unrecognized parameter associated with it
                ThrowErrorMessage("Sheet Read Error", "PSet/Signal Sheet has an unrecognized parameter associated with it");
                return null;
            }
            catch (ArgumentException)
            {
                // Signal Sheet is not avilable
                ThrowErrorMessage("Sheet Read Error", "The sheet is not available");
                return null;
            }
            catch (System.Runtime.InteropServices.COMException)
            {
                // there are no signal sheet to read from, or no signals exist
                String eMsg;
                eMsg = "The " +
                    ((type.Equals(GlobalUtils.NspcpType.pset)) ? "PSet '" : "Signal '") +
                    tmpArrayList[nItems].ToString() + 
                    "' with PinSpec '" + PinName + "' can't be found on the sheet";
                ThrowErrorMessage("Sheet Read Error", eMsg);
                return null;
            }
            catch (Exception exp)
            {
                //Catch block for any other errors that are not handle
                ThrowErrorMessage("Sheet Read Error", exp.Message);
                return null;
            }
            return tmpObj;
        }

        private void ThrowErrorMessage(String errorName, String errorInfo)
        {
            
            ///Checks whether the Error Grid has already initialized or not
            if (dgvErrors.Rows.Count == 0 )
                EnlargeErrorPane();

            List<string> errList = new List<string>();
            errList.Add(errorInfo);
            m_UIEngine.ReportErrorInListBox(errorName, errList);
        }

        /// <summary>
        /// This method is used to Shrunk the ErrorGrid and
        /// Moves the SpliterContainer to the bottom of the form
        /// </summary>
        private void ShrunkErrorPane()
        {
            splitContainer1.SplitterDistance = splitContainer1.SplitterDistance + (this.Height - splitContainer1.SplitterDistance);
            dgvErrors.Height = 0;
        }
     
        /// <summary>
        /// Used to Enlarge the Error Grid when it has the error Information.
        /// </summary>
        private void EnlargeErrorPane()
        {
           
            //Decrease the Size of the SplitterContainer button..
            dgvErrors.Height = grpErrors.Height - 30;             

            //Splitter Distance
            splitContainer1.SplitterDistance = 472;

            //Initialize the Error Grid
            dgvErrors.ColumnCount = 2;
            dgvErrors.BackgroundColor = Control.DefaultBackColor; // backcolor of interior area outside cells
            dgvErrors.ColumnHeadersDefaultCellStyle.Font = new Font(dgvErrors.Font, FontStyle.Bold);
            dgvErrors.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.DisplayedCellsExceptHeaders;
            dgvErrors.ColumnHeadersBorderStyle = DataGridViewHeaderBorderStyle.Single;
            dgvErrors.CellBorderStyle = DataGridViewCellBorderStyle.Single;
            dgvErrors.RowHeadersVisible = false;
            dgvErrors.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells;
            dgvErrors.AutoSizeColumnsMode = System.Windows.Forms.DataGridViewAutoSizeColumnsMode.Fill; // important when resizing                       
            dgvErrors.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right | System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Bottom)));
            dgvErrors.Columns[0].Name = "Type";
            dgvErrors.Columns[0].Width = 300;
            dgvErrors.Columns[0].MinimumWidth = 150;
            dgvErrors.Columns[1].Name = "Error";
            dgvErrors.Columns[1].Width = 400;
            dgvErrors.Columns[0].ReadOnly = true;
            dgvErrors.Columns[1].ReadOnly = true;
            dgvErrors.Columns[0].SortMode = DataGridViewColumnSortMode.NotSortable;
            dgvErrors.Columns[1].SortMode = DataGridViewColumnSortMode.NotSortable;
            dgvErrors.MultiSelect = true;
            dgvErrors.SelectionMode = DataGridViewSelectionMode.CellSelect;
            // Enable single-click-to-edit for editable cells:
            dgvErrors.EditMode = System.Windows.Forms.DataGridViewEditMode.EditOnEnter;
            // Suppress empty last row:
            dgvErrors.AllowUserToAddRows = false;
            
        }

        /// <summary>
        /// This method checks which definitions have been added and which have been
        /// removed (in the pset/signal selector). Based on this information and the
        /// list of existing columns on the display, this method : 
        /// a) Removes all NSPCP definitions which the user does not want to see.
        /// b) Keeps the definitions which have not been removed by the user.
        /// c) Generates a list of all the additional definitions that need to be added in the 
        /// panel
        /// </summary>
        /// <returns>List of definitions that need to be added.</returns>
        private System.Collections.ArrayList HandleRemoveColumns(ref Dictionary<String, NspcpItem> items, ref UserInterfaceEngine UI,
                                    ref PanelSettings settings, ref PanelNspcpValues values)
        {
            //We use this technique to add only the recently added defintions.
            List<String> tmpList;
            tmpList = new List<String>();

            //We need to create a temporary list of names since we cannot
            //iterate using the dictionary keys and remove keys in the same loop.
            foreach (String nspcpName in items.Keys)
                tmpList.Add(nspcpName);
            //This is PSet/Signal sensitive
            System.Collections.ArrayList useArray;
            if (settings.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                useArray = values.PSetValues;
            else
                useArray = values.SignalValues;

            System.Collections.ArrayList tmpArrayList = (System.Collections.ArrayList)useArray.Clone();
            foreach (String name in tmpList)
            {
                if (useArray.Contains(name))
                    //Removing existing items from the new list so that 
                    //only the items to be added remain.
                    tmpArrayList.Remove(name);
                else
                {
                    //If the new list does not contain any existing nspcp,
                    //remove it from the display.
                    UI.DeleteColumn(items[name]);
                    if (items.ContainsKey(name))
                        items.Remove(name);

                }
            }
            return tmpArrayList;
        }

        /// <summary>
        /// Check if there are any unsaved definitions and notify user
        /// </summary>
        /// <returns>False - Continue without saving. True - Do not continue.</returns>
        public Boolean CheckForUnsavedDefinitions()
        {
            DialogResult res = DialogResult.Yes; //Initialize
            //Step 1 : Warn users to save the information currently displayed.
            if ((m_GlobalSettings.MainPanel.DATA_SOURCE== GlobalUtils.DATA_SOURCE.SHEET) && m_DefChanged_NotAppliedToSheet)
            {
                m_CheckUnsavedDefinitionShowing = true;
                res = MessageBox.Show(Properties.Resources.PSetSignalEditorDiscardData, GlobalUtils.MESSAGEBOX_CAPTION, MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
            }
            else if ((m_GlobalSettings.MainPanel.DATA_SOURCE== GlobalUtils.DATA_SOURCE.DRIVER) && m_DefChanged_NotAppliedToDriver)
            {
                m_CheckUnsavedDefinitionShowing = true;
                res = MessageBox.Show(Properties.Resources.PSetSignalEditorDiscardData, GlobalUtils.MESSAGEBOX_CAPTION, MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
            }
            else
            {
                return false;
            }
            m_CheckUnsavedDefinitionShowing = false;
            if (res == DialogResult.Yes)
            {
                //now that we are continuing without saving changes we can forget that changes were made
                m_DefChanged_NotAppliedToSheet = false;
                m_DefChanged_NotAppliedToDriver = false; 
                m_undoManager.Clear();
                EnableDisableUndoRedoButtons();
                return false;
            }
            else
            {
                return true;
            }
        }


        /// <summary>
        /// Check if the source (driver/sheet) can be switched or not for either panel.
        /// </summary>
        /// <param name="switchTo"></param>
        /// <param name="nspcpList"></param>
        /// <returns></returns>
        private Boolean CanSwitchSource(GlobalUtils.DATA_SOURCE switchTo, Dictionary<String, NspcpItem> nspcpList,
                         ref PanelSettings settings, ref PanelNspcpValues values, ref UserInterfaceEngine UI,
                         String pin, String instrument, ref InstrumentEngine instr)
        {
            Boolean okToProceedWithDiscard = false;
            
            if (switchTo == settings.DATA_SOURCE) return true;

            //Switch to pset/signal sheet engine depending upon global settings
            if (settings.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                m_SheetEngine = new PSetSheetEngine(ref psetHelper);
            else
                m_SheetEngine = new SignalsSheetEngine(ref signalHelper);

            // Check if the currently displayed NSPCPs are valid in the current mode (driver/sheet).
            foreach (KeyValuePair<String, NspcpItem> kvp in nspcpList)
            {
                if (!ExistsNSPCP(kvp.Value.pinName, kvp.Value.propLogicalInstrument, kvp.Value.propName, switchTo, false, ref instr))
                {
                    if (!okToProceedWithDiscard)
                    {
                        if (MessageBox.Show("One or more of the PSet/Signal definitions currently displayed do not exist in the selected mode. Do you want to discard these definitions and lose all changes?", GlobalUtils.MESSAGEBOX_CAPTION, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.No)
                            return false;
                        else
                            okToProceedWithDiscard = true;
                    }
                    if (okToProceedWithDiscard)
                    {
                        if (settings.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                        {
                            values.PSetValues.Remove(kvp.Value.propName);
                        }
                        else
                        {
                            values.SignalValues.Remove(kvp.Value.propName);
                        }
                    }
                }
            }
            //Compute the final list of nspcps to be displayed.
            System.Collections.ArrayList arrList = HandleRemoveColumns(ref nspcpList, ref UI, ref settings, ref values);
            //Add the nspcp definition to the parameter grid.
            HandleSelectedNspcp(pin, instrument, ref arrList, ref settings, ref values,ref instr);
            return true;
        }

        /// <summary>
        /// This method updates the values in the parameter grid.  Used for either panel.
        /// </summary>
        /// <param name="SourceChanged"></param>
        private void UpdateParameterGrid(Boolean SourceChanged, ref UserInterfaceEngine UI, ref PanelSettings settings,
                                ref PanelNspcpValues values, ref Dictionary<String, NspcpItem> nspcpList,
                                String pin, String instrument, ref InstrumentEngine instr)
        {
            if (!UI.IsInitialized) return; //if the UI Is not initialized, return.
            UI.ClearAll();
             
            ShrunkErrorPane();

            Dictionary<String, NspcpItem> backupList = null;
            System.Collections.ArrayList tmpArrayList = new System.Collections.ArrayList();
            if (!settings.READONLY)//Only update the dirty bits if we are updating the main panel
            {
                m_DefChanged_NotAppliedToSheet = false;
                m_DefChanged_NotAppliedToDriver = false;
            }
            if (settings.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                tmpArrayList = values.PSetValues;
            else
                tmpArrayList = values.SignalValues;

            //If the source has been changed by user, then perform these steps to compare definitions
            if (SourceChanged)
            {
                //Step 1 : Collect all the information from the previous mode
                UI.CollectGridInformation(ref nspcpList);
                backupList = DuplicateDictionary(nspcpList);
            }
            else //If source is same (Refresh Operation), do a routine check to see if the nspcp's still exist in that mode or not.
            {
                //Refresh the list of nspcp's
                if (RefreshPSetSignalList(ref settings, ref values, pin, instrument,ref instr))
                {
                    MessageBox.Show("Some definitions were removed because they don't exist in " + 
                        ((settings.DATA_SOURCE == GlobalUtils.DATA_SOURCE.SHEET) ? "Sheet" : "Driver")
                        + " mode.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    //Reconfigure display due to the refresh
                    tmpArrayList = HandleRemoveColumns(ref nspcpList, ref UI, ref settings, ref values);
                }
            }

            if (tmpArrayList.Count > 0)
            {
                HandleSelectedNspcp(pin, instrument, ref tmpArrayList,
                                ref settings, ref values,ref instr);
            }

            //In case some values were changed, we will restore the text color.
            UI.RestoreTextColor();

            if (!SourceChanged)
                return;

            foreach (String name in backupList.Keys)
            {
                try
                {
                    UI.CompareAndMark(nspcpList[name]);
                }
                catch (Exception exp)
                {
                    ThrowErrorMessage("Compare Error", exp.Message);
                }
            }
        }

        private Dictionary<String, NspcpItem> DuplicateDictionary(Dictionary<String, NspcpItem> dic)
        {
            Dictionary<String, NspcpItem> backupList;
            //Back up this data
            backupList = new Dictionary<string, NspcpItem>();
            foreach (KeyValuePair<String, NspcpItem> kvp in dic)
            {
                backupList.Add(kvp.Key, kvp.Value);
            }
            return backupList;
        }

        /// <summary>
        /// This method is used to reconfigure the cells in the grid after
        /// a value in one of the cells has been modified. This updates any values
        /// that might be affected by the modification and reloads new dropdowns.
        /// </summary>
        /// <param name="obj"></param>
        private void ReConfigureGrid(NspcpItem obj, ref InstrumentEngine instr)
        {
            try
            {
                if (m_UIEngine.IsInitialized)
                {
                    //Collect the existing information
                    m_UIEngine.CollectGridInformation(ref obj);
                    //Get the list of enables
                    Boolean[] enables = instr.Configure(ref obj);
                    //Repopulate the columns with the reconfigured cells and dropdowns.
                    m_UIEngine.PopulateColumn(obj);
                    //Enable/disable appropriate cells
                    m_UIEngine.ConfigureCells(enables, obj.propName);
                }
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Grid Configure Error", exp.Message);
            }
        }

        /// <summary>
        /// This method populates the parameter grid with the given definition
        /// and configures the display.
        /// </summary>
        /// <param name="obj"></param>
        private void HandleGridConfiguration(NspcpItem obj, ref UserInterfaceEngine UI, ref InstrumentEngine instr)
        {
            try
            {
                Boolean[] enables = instr.Configure(ref obj);
                UI.ConfigureCells(enables,obj.propName);
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Error getting configuration info ", exp.Message);
            }
        }

        private void splLeftRight_SplitterMoved(object sender, SplitterEventArgs e)
        {
            gbxEditPSet.Width = pnlLeft.Width - 29;

            // for design-time layout assistance only
            //Console.WriteLine("splLeftRight position: " + splLeftRight.SplitPosition.ToString());
            //Console.WriteLine("pnlLeft Width: " + pnlLeft.Width.ToString());
        }

        // Split Button methods
        #region SplitButton
        public class SplitButton : Button
        {
            private PushButtonState _state;
            private const int PushButtonWidth = 14;
            private int BorderSize = SystemInformation.Border3DSize.Width * 2;
            private bool skipNextOpen = false;
            private Rectangle dropDownRectangle = new Rectangle();
            private bool showSplit = true;

            public SplitButton()
            {
                this.AutoSize = true;
            }
            [DefaultValue(true)]
            public bool ShowSplit
            {
                set
                {
                    if (value != showSplit)
                    {
                        showSplit = value;
                        Invalidate();
                        if (this.Parent != null)
                        {
                            this.Parent.PerformLayout();
                        }
                    }
                }
            }

            private PushButtonState State
            {
                get
                {
                    return _state;
                }
                set
                {
                    if (!_state.Equals(value))
                    {
                        _state = value;
                        Invalidate();
                    }
                }
            }

            public override Size GetPreferredSize(Size proposedSize)
            {
                Size preferredSize = base.GetPreferredSize(proposedSize);
                if (showSplit && !string.IsNullOrEmpty(Text) && TextRenderer.MeasureText(Text, Font).Width + PushButtonWidth > preferredSize.Width)
                {
                    return preferredSize + new Size(PushButtonWidth + BorderSize * 2, 0);
                }
                return preferredSize;
            }

            protected override bool IsInputKey(Keys keyData)
            {
                if (keyData.Equals(Keys.Down) && showSplit)
                {
                    return true;
                }
                else
                {
                    return base.IsInputKey(keyData);
                }
            }

            protected override void OnGotFocus(EventArgs e)
            {
                if (!showSplit)
                {
                    base.OnGotFocus(e);
                    return;
                }

                if (!State.Equals(PushButtonState.Pressed) && !State.Equals(PushButtonState.Disabled))
                {
                    State = PushButtonState.Default;
                }
            }

            protected override void OnKeyDown(KeyEventArgs kevent)
            {
                if (showSplit)
                {
                    if (kevent.KeyCode.Equals(Keys.Down))
                    {
                        ShowContextMenuStrip();
                    }
                    else if (kevent.KeyCode.Equals(Keys.Space) && kevent.Modifiers == Keys.None)
                    {
                        State = PushButtonState.Pressed;
                    }
                }

                base.OnKeyDown(kevent);
            }

            protected override void OnKeyUp(KeyEventArgs kevent)
            {
                if (kevent.KeyCode.Equals(Keys.Space))
                {
                    if (Control.MouseButtons == MouseButtons.None)
                    {
                        State = PushButtonState.Normal;
                    }
                }
                base.OnKeyUp(kevent);
            }

            protected override void OnLostFocus(EventArgs e)
            {
                if (!showSplit)
                {
                    base.OnLostFocus(e);
                    return;
                }
                if (!State.Equals(PushButtonState.Pressed) && !State.Equals(PushButtonState.Disabled))
                {
                    State = PushButtonState.Normal;
                }
            }

            protected override void OnMouseDown(MouseEventArgs e)
            {
                if (!showSplit)
                {
                    base.OnMouseDown(e);
                    return;
                }

                if (dropDownRectangle.Contains(e.Location))
                {
                    ShowContextMenuStrip();
                }
                else
                {
                    State = PushButtonState.Pressed;
                }
            }

            protected override void OnMouseEnter(EventArgs e)
            {
                if (!showSplit)
                {
                    base.OnMouseEnter(e);
                    return;
                }

                if (!State.Equals(PushButtonState.Pressed) && !State.Equals(PushButtonState.Disabled))
                {
                    State = PushButtonState.Hot;
                }
            }

            protected override void OnMouseLeave(EventArgs e)
            {
                if (!showSplit)
                {
                    base.OnMouseLeave(e);
                    return;
                }

                if (!State.Equals(PushButtonState.Pressed) && !State.Equals(PushButtonState.Disabled))
                {
                    if (Focused)
                    {
                        State = PushButtonState.Default;
                    }
                    else
                    {
                        State = PushButtonState.Normal;
                    }
                }
            }

            protected override void OnMouseUp(MouseEventArgs mevent)
            {
                if (!showSplit)
                {
                    base.OnMouseUp(mevent);
                    return;
                }

                if (ContextMenuStrip == null || !ContextMenuStrip.Visible)
                {
                    SetButtonDrawState();
                    if (Bounds.Contains(Parent.PointToClient(Cursor.Position)) && !dropDownRectangle.Contains(mevent.Location))
                    {
                        OnClick(new EventArgs());
                    }
                }
            }

            protected override void OnPaint(PaintEventArgs pevent)
            {
                base.OnPaint(pevent);

                if (!showSplit)
                {
                    return;
                }

                Graphics g = pevent.Graphics;
                Rectangle bounds = this.ClientRectangle;

                // draw the button background as according to the current state.
                if (State != PushButtonState.Pressed && IsDefault && !Application.RenderWithVisualStyles)
                {
                    Rectangle backgroundBounds = bounds;
                    backgroundBounds.Inflate(-1, -1);
                    ButtonRenderer.DrawButton(g, backgroundBounds, State);

                    // button renderer doesnt draw the black frame when themes are off =(
                    g.DrawRectangle(SystemPens.WindowFrame, 0, 0, bounds.Width - 1, bounds.Height - 1);

                }
                else
                {
                    ButtonRenderer.DrawButton(g, bounds, State);
                }
                // calculate the current dropdown rectangle.
                dropDownRectangle = new Rectangle(bounds.Right - PushButtonWidth - 1, BorderSize, PushButtonWidth, bounds.Height - BorderSize * 2);

                int internalBorder = BorderSize;
                Rectangle focusRect =
                    new Rectangle(internalBorder,
                                  internalBorder,
                                  bounds.Width - dropDownRectangle.Width - internalBorder,
                                  bounds.Height - (internalBorder * 2));

                bool drawSplitLine = (State == PushButtonState.Hot || State == PushButtonState.Pressed || !Application.RenderWithVisualStyles);

                if (RightToLeft == RightToLeft.Yes)
                {
                    dropDownRectangle.X = bounds.Left + 1;
                    focusRect.X = dropDownRectangle.Right;
                    if (drawSplitLine)
                    {
                        // draw two lines at the edge of the dropdown button
                        g.DrawLine(SystemPens.ButtonShadow, bounds.Left + PushButtonWidth, BorderSize, bounds.Left + PushButtonWidth, bounds.Bottom - BorderSize);
                        g.DrawLine(SystemPens.ButtonFace, bounds.Left + PushButtonWidth + 1, BorderSize, bounds.Left + PushButtonWidth + 1, bounds.Bottom - BorderSize);
                    }
                }
                else
                {
                    if (drawSplitLine)
                    {
                        // draw two lines at the edge of the dropdown button
                        g.DrawLine(SystemPens.ButtonShadow, bounds.Right - PushButtonWidth, BorderSize, bounds.Right - PushButtonWidth, bounds.Bottom - BorderSize);
                        g.DrawLine(SystemPens.ButtonFace, bounds.Right - PushButtonWidth - 1, BorderSize, bounds.Right - PushButtonWidth - 1, bounds.Bottom - BorderSize);
                    }

                }

                // Draw an arrow in the correct location 
                PaintArrow(g, dropDownRectangle);

                // Figure out how to draw the text
                TextFormatFlags formatFlags = TextFormatFlags.HorizontalCenter | TextFormatFlags.VerticalCenter;

                // If we dont' use mnemonic, set formatFlag to NoPrefix as this will show ampersand.
                if (!UseMnemonic)
                {
                    formatFlags = formatFlags | TextFormatFlags.NoPrefix;
                }
                else if (!ShowKeyboardCues)
                {
                    formatFlags = formatFlags | TextFormatFlags.HidePrefix;
                }

                if (!string.IsNullOrEmpty(this.Text))
                {
                    TextRenderer.DrawText(g, Text, Font, focusRect, SystemColors.ControlText, formatFlags);
                }

                // draw the focus rectangle.

                if (State != PushButtonState.Pressed && Focused)
                {
                    ControlPaint.DrawFocusRectangle(g, focusRect);
                }
            }

            private void PaintArrow(Graphics g, Rectangle dropDownRect)
            {
                Point middle = new Point(Convert.ToInt32(dropDownRect.Left + dropDownRect.Width / 2), Convert.ToInt32(dropDownRect.Top + dropDownRect.Height / 2));

                //if the width is odd - favor pushing it over one pixel right.
                middle.X += (dropDownRect.Width % 2);

                Point[] arrow = new Point[] { new Point(middle.X - 2, middle.Y - 1), new Point(middle.X + 3, middle.Y - 1), new Point(middle.X, middle.Y + 2) };

                g.FillPolygon(SystemBrushes.ControlText, arrow);
            }

            private void ShowContextMenuStrip()
            {
                if (skipNextOpen)
                {
                    // we were called because we're closing the context menu strip
                    // when clicking the dropdown button.
                    skipNextOpen = false;
                    return;
                }
                State = PushButtonState.Pressed;

                if (ContextMenuStrip != null)
                {
                    ContextMenuStrip.Closing += new ToolStripDropDownClosingEventHandler(ContextMenuStrip_Closing);
                    ContextMenuStrip.Show(this, new Point(0, Height), ToolStripDropDownDirection.BelowRight);
                }
            }

            void ContextMenuStrip_Closing(object sender, ToolStripDropDownClosingEventArgs e)
            {
                ContextMenuStrip cms = sender as ContextMenuStrip;
                if (cms != null)
                {
                    cms.Closing -= new ToolStripDropDownClosingEventHandler(ContextMenuStrip_Closing);
                }

                SetButtonDrawState();

                if (e.CloseReason == ToolStripDropDownCloseReason.AppClicked)
                {
                    skipNextOpen = (dropDownRectangle.Contains(this.PointToClient(Cursor.Position)));
                }
            }


            private void SetButtonDrawState()
            {
                if (Bounds.Contains(Parent.PointToClient(Cursor.Position)))
                {
                    State = PushButtonState.Hot;
                }
                else if (Focused)
                {
                    State = PushButtonState.Default;
                }
                else
                {
                    State = PushButtonState.Normal;
                }
            }
        }
        #endregion

        #region ITDETool Members
        public string Caption
        {
            get { return m_ToolName; }
        }

        public object Commands
        {
            get
            {
                return m_TDECommands;
            }
        }

        public bool FixedSize
        {
            get { return true; }
        }

        public string InstanceTag
        {
            get
            {
                return m_InstanceTag;
            }
            set
            {
                m_InstanceTag = value;
            }
        }

        public void Loop(TDELoopEvent LoopEvent)
        {
        }

        /// <summary>
        /// This method can be called by external componenets to launch and initialize
        /// the PSet/Signals editor with values for Pin Name, Instrument Name, NSPCP
        /// name and NSPCP type (PSet/Signal).
        /// [Note: NSPCP = Named Setups of Pattern Controlled Parameters]
        /// </summary>
        /// <param name="Verb"></param>
        /// <param name="Resource"></param>
        public void PerformVerb(string Verb, object Resource)
        {
            String pinName;
            String instrName;
            String nspcpName;
            
            switch (Verb)
            {
                //This case occurs when the initialization verb is called
                case "InitInfo":
                    //Extract the information from the resource string.
                    String info = (String)Resource;
                    if (!String.IsNullOrEmpty(info))
                    {
                        String[] editorInfo = info.Split(new char[] { ',' });
                        if (editorInfo.GetUpperBound(0) != 3)
                        {
                            throw new Exception("PSet/Signals Editor InitInfo Verb expects 4 parameters");
                        }
                        pinName = editorInfo[0];
                        instrName = editorInfo[1];
                        nspcpName = editorInfo[2];
                        String nspcpType = editorInfo[3];
                        GlobalUtils.NspcpType enumNspcpType;

                        if (nspcpType.ToUpper().Equals("PSET"))
                            enumNspcpType = GlobalUtils.NspcpType.pset;
                        else if (nspcpType.ToUpper().Equals("SIGNAL"))
                            enumNspcpType = GlobalUtils.NspcpType.signal;
                        else
                            throw new Exception(Properties.Resources.InvalidResourceObject + " " + nspcpType);

                        // Update Collection Edit Helper depending upon what the currently selected nspcp type is.
                        m_Main_InstrEngine.UpdateHelper(enumNspcpType);
                        //.Rebuild a key set so that we can retrieve the pin and instrument for the current nspcp type
                        m_Main_InstrEngine.Rebuild();
                        // Update the dropdown items if needed and add the nspcp definition to the parameter grid.
                        DisplayNSPCP(enumNspcpType, pinName, instrName, nspcpName);
                    }
                    break;
                // This case occurs when the show verb is called
                case "Show":
                    if (Resource == null)
                        return;
                    // Extract the information from the resource object.
                    DMSignalssignals signalInfo = Resource as DMSignalssignals;
                    DMPsetspset psetInfo = Resource as DMPsetspset;
                    GlobalUtils.NspcpType enumNspcptype;
                    if (signalInfo != null)
                    {
                        nspcpName = signalInfo.Name;
                        pinName = signalInfo.Pins.Names.CSL;
                        DMSignalssignalspin signalPinInfo = signalInfo.Pins.Find(pinName);
                        instrName = signalPinInfo.Insttype;
                        enumNspcptype = GlobalUtils.NspcpType.signal;
                    }
                    else if (psetInfo != null)
                    {
                        nspcpName = psetInfo.Name;
                        pinName = psetInfo.Pins.Names.CSL;
                        DMPsetspsetpin psetPinInfo = psetInfo.Pins.Find(pinName);
                        instrName = psetPinInfo.Insttype;
                        enumNspcptype = GlobalUtils.NspcpType.pset;
                    }
                    else
                    {
                        throw new Exception(Properties.Resources.InvalidResourceObject);
                    }
                    
                    // Update Collection Edit Helper depending upon what the currently selected nspcp type is.
                    m_Main_InstrEngine.UpdateHelper(enumNspcptype);
                    //.Rebuild a key set so that we can retrieve the pin and instrument for the current nspcp type
                    m_Main_InstrEngine.Rebuild();
                    // Update the dropdown items if needed and add the nspcp definition to the parameter grid.
                    DisplayNSPCP(enumNspcptype, pinName, instrName, nspcpName);
                    break;
                default:
                    //Do nothing
                    break;
            }   // end switch
        }  // end PerformVerb

        /// <summary>
        /// This method is used to update the dropdown items and 
        /// display the Named Setups of Pattern Controlled Parameters.
        /// </summary>
        private void DisplayNSPCP(GlobalUtils.NspcpType enumNspcpType, String pinName, String instrName, String nspcpName)
        {
            // Update the dropdown items if needed. 
            if (txtPinName.Text != pinName)
            {
                if (CheckForUnsavedDefinitions()) return;
                txtPinName.Text = pinName;
                SetInstrumentsForPins(true);
            }
            if (cboInstrument.Text != instrName)
            {
                if (CheckForUnsavedDefinitions()) return;
                cboInstrument.SelectedIndex = cboInstrument.FindStringExact(instrName);
                if (cboInstrument.SelectedIndex == -1)
                {
                   cboInstrument.Items.Add(instrName);
                   cboInstrument.SelectedIndex = cboInstrument.FindStringExact(instrName);
                }
            }

            Boolean existNspcp = false;
            // Validate Pin, Instrument and NSPCP Name
            try
            {
                // Select the mode
               if (enumNspcpType == GlobalUtils.NspcpType.pset)
               {
                   if (cboSource.SelectedIndex != (int)GlobalUtils.NspcpType.pset)
                   {
                       if (CheckForUnsavedDefinitions()) return;
                       cboSource.SelectedIndex = (int)GlobalUtils.NspcpType.pset;
                       HandleNSPCPTypeSelection(GlobalUtils.NspcpType.pset, ref m_GlobalSettings.MainPanel, ref m_Main_InstrEngine);
                    }
                    if ((nspcpName != "") && ExistsNSPCP(pinName, instrName, nspcpName, m_GlobalSettings.MainPanel.DATA_SOURCE, true, ref m_Main_InstrEngine))
                    {
                        if (!m_GlobalPSetSignalValues.MainPanel.PSetValues.Contains(nspcpName))
                            m_GlobalPSetSignalValues.MainPanel.PSetValues.Add(nspcpName.ToLower());
                        existNspcp = true;
                    }
                }
                else if (enumNspcpType == GlobalUtils.NspcpType.signal)
                {
                     if (cboSource.SelectedIndex != (int)GlobalUtils.NspcpType.signal)
                     {
                         if (CheckForUnsavedDefinitions()) return;
                         cboSource.SelectedIndex = (int)GlobalUtils.NspcpType.signal;
                         HandleNSPCPTypeSelection(GlobalUtils.NspcpType.signal, ref m_GlobalSettings.MainPanel, ref m_Main_InstrEngine);
                     }
                     if ((nspcpName != "") && ExistsNSPCP(pinName, instrName, nspcpName, m_GlobalSettings.MainPanel.DATA_SOURCE, true, ref m_Main_InstrEngine))
                     {
                         if (!m_GlobalPSetSignalValues.MainPanel.SignalValues.Contains(nspcpName))
                            m_GlobalPSetSignalValues.MainPanel.SignalValues.Add(nspcpName.ToLower());
                         existNspcp = true;
                     }
                 }  // end else SIGNALS
                 if (existNspcp)
                 {
                    // Compute the final list of nspcps to be displayed.
                    System.Collections.ArrayList arrList = HandleRemoveColumns(ref m_NSPCPList, ref m_UIEngine,
                                                   ref m_GlobalSettings.MainPanel, ref m_GlobalPSetSignalValues.MainPanel);
                    // Add the nspcp definition to the parameter grid.
                    HandleSelectedNspcp(pinName, instrName, ref arrList, ref m_GlobalSettings.MainPanel, ref m_GlobalPSetSignalValues.MainPanel, ref m_Main_InstrEngine);
                 }
             }  // end try
             catch (Exception e)
             {
                 ThrowErrorMessage("Error initializing display", e.Message);
             }
        }

        /// <summary>
        /// This methods validates the pinName, instrument name and the nspcp name.
        /// The validation procedures called depend upon whether we are 
        /// </summary>
        /// <param name="pinName">Currently Selected Pin</param>
        /// <param name="instrName">Currently selected instrument</param>
        /// <param name="nspcpName">The PSet/Signal Name to check for</param>
        /// <param name="source">The current data source (Sheet/Driver)</param>
        /// <param name="showErrors">Show errors or not.</param>
        /// <returns>returns whether the pset/signal exists.</returns>
        public Boolean ExistsNSPCP(String pinName, String instrName, String nspcpName, GlobalUtils.DATA_SOURCE source, Boolean showErrors, ref InstrumentEngine instr)
        {
            // remove spaces from pinName (in case if pinName is CSL of pins)
            String pinList;
            pinList = pinName.Replace(" ", "");
            
            //Incase the source is the instrument drivers.
            if (source == GlobalUtils.DATA_SOURCE.DRIVER)
            {
                String err = "";
                Boolean gotOriginal = false;
                int selectedSite = GetSelectedSite();
                SiteWrapper.SetOverride(selectedSite);
                gotOriginal = true;

                try
                {
                    //Call the validation method in the instrument engine.
                    if (!instr.ValidateNSPCP(pinList, instrName, nspcpName, ref err))
                    {
                        if (showErrors) MessageBox.Show(err);
                        return false;
                    }
                }
                catch (Exception exp)
                {
                    ThrowErrorMessage("Error validating PSet/Signal: ", exp.Message);
                }
                finally
                {
                    //Restore sites.
                    if (gotOriginal)
                    {
                        SiteWrapper.RestoreFromOverride();
                    }
                }
            }
            else //In case the source is the sheet.
            {
                Boolean found = false;

                //We check pin name and instrument name separately.
                //we do not do this separately in the previous case since 
                //ValidateNSPCP method of instrument engine already does this for us.
                if (!instr.CheckPin(ref pinList))
                {
                    MessageBox.Show("Selected pin name is invalid.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }

                if (!instr.CheckInstrument(instrName, true, pinList))
                {
                    MessageBox.Show("Selected instrument is invalid or does not map to the selected pin.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return false;
                }
                //Get the list of nspcp names
                try
                {
                    String[] list = m_SheetEngine.GetList(pinList, instrName);

                    //Look for nspcp name in the sheet.
                    foreach (String name in list)
                    {
                        if (nspcpName.ToLower().Equals(name.ToLower())) found = true;
                    }
                    if (!found)
                    {
                        if (showErrors) ThrowErrorMessage("Error retrieving sheet definition", "The PSet/Signal (" + nspcpName + ") could not be found on the active sheet");
                        return false;
                    }
                }
                catch (Exception exp)
                {
                    ThrowErrorMessage("Error reading list of PSets/Signals", exp.Message);
                    return false;
                }
            }

            return true;
        }

        public void PreferenceChanged(string PrefName, object value)
        {

        }

        public bool QueryShutdown()
        {
            if (CheckForUnsavedDefinitions())
                return false;

            return true;
        }

        public void ShowState(TDEShowState mode)
        {

        }

        public void StoreState()
        {

        }

        /// <summary>
        /// Updates the menu.
        /// </summary>
        /// <param name="BeforeFocus">if set to <c>true</c> [before focus].</param>
        public void UpdateMenu(bool BeforeFocus)
        {
            object temp;
            if (BeforeFocus)
            {
                m_bNeedWithBlock = false;
                m_bNewLang = false;
                //enable or disable the PSet/Signals launch button
                enablePSetSignal(txtPinName.Text, cboInstrument.Text, ref btnPSetsOrSignals);

                //Initialize the Sheet engine to PSet Engine by default (Can be changed later)
                //            m_SheetEngine = (ISheetEngine)m_psetSheetEngine;

                if (m_SheetEngine == null)
                {
                    HandleNSPCPTypeSelection(GlobalUtils.NspcpType.pset, ref m_GlobalSettings.MainPanel, ref m_Main_InstrEngine);
                }
                
                //Initialize Site DropDown
                InitSiteDropDown();
                //set to false because when updatemenu triggers no need to check the unsaved changes
                m_NeedToCheckChanges = false; 
                //Check if the select site is active or not.
                HandleSiteActivation();

                temp = m_IPreferences.GetToolPreference("PSSE", "NeedWithBlock");
                if (temp != null)
                {
                    m_bNeedWithBlock = Convert.ToBoolean(temp);
                    if (m_bNeedWithBlock)
                        UpdateMenuItem(NEEDWITHBLOCK, UpdateMenuItemEnum.CHECKED, true);
                }
                temp = m_IPreferences.GetToolPreference("PSSE", "NewPSetSignals");
                if (temp != null)
                {
                    m_bNewLang = Convert.ToBoolean(temp);
                    if (m_bNewLang)
                    {
                        UpdateMenuItem(NEWPSETSIGNALS, UpdateMenuItemEnum.CHECKED, true);
                        UpdateMenuItem(PSETSIGNALSLANGUAGE, UpdateMenuItemEnum.CHECKED, false);
                    }
                    else
                    {
                        UpdateMenuItem(PSETSIGNALSLANGUAGE, UpdateMenuItemEnum.CHECKED, true);
                        UpdateMenuItem(NEWPSETSIGNALS, UpdateMenuItemEnum.CHECKED, false);
                    }
                }

                UpdateMenuItem(HELPABOUTTOOL, UpdateMenuItemEnum.ENABLED, true);
                UpdateMenuItem(HELPSPECIFICATIONS, UpdateMenuItemEnum.VISIBLE, false);
                UpdateMenuItem(HELPTOOL, UpdateMenuItemEnum.VISIBLE, false);
                UpdateMenuItem(SHEETDEFINITION, UpdateMenuItemEnum.CHECKED, rbnSheetDefinition.Checked);
                UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.CHECKED, rbnCurrentDriverDefinition.Checked);

                //Disable the CopyCode TSButton
                configureCopyCodeButton();
            }
            else
            {
                m_IPreferences.SetToolPreference("PSSE", "NeedWithBlock", m_bNeedWithBlock);
                m_IPreferences.SetToolPreference("PSSE", "NewPSetSignals", m_bNewLang);
            }

        }

        public TDEMemento WindowState
        {
            get
            {
                return null;
            }
            set
            {
                m_WindowState = value;
            }
        }

        public new string Name
        {
            get
            {
                return m_ToolName;
            }
            set
            {
                m_ToolName = value;
            }
        }

        #endregion

        #region Instrument Dropdown event handlers
        private void cboInstrument_MouseClick(object sender, MouseEventArgs e)
        {
            if (!cboInstrument.Text.Equals(DEFAULT_TEXT_INST))
            {
                m_oldInstrumentValue = cboInstrument.Text;
            }
        }

        //tersw00174550
        private void cboInstrument_Enter(object sender, EventArgs e)
        {
            cboInstrument_DropDown(sender, e);
        }

        private void cboInstrument_DropDown(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
            m_Main_InstrEngine.Rebuild();
            
            // tersw00112429
            if (!cboInstrument.Text.Equals(DEFAULT_TEXT_INST))
            {
                m_oldInstrumentValue = cboInstrument.Text;
            }
            SetInstrumentsForPins(true);
            Cursor.Current = Cursors.Default;
        }

        private void cboInstrument_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
            enablePSetSignal(txtPinName.Text, cboInstrument.Text, ref btnPSetsOrSignals);
            //Inform user that the grid will be cleared and ask for saving permission.
            if (m_UIEngine.IsInitialized)
            {
                // tersw00112429 - clear grid if selection changed
                if (cboInstrument.Text != m_oldInstrumentValue)
                {
                    if (!CheckForUnsavedDefinitions())
                    {
                        // tersw00174642 - in addition, clear it only if no old instrument value present
                        if (m_oldInstrumentValue != null)
                        {
                            ClearMainParameterGrid();
                        }
                    }
                    else
                    {
                        cboInstrument.Text = m_oldInstrumentValue;
                    }
                }
            }
        }

        #endregion

        private void cboMixedInstrument_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_Compare_InstrEngine.UpdateHelper(m_GlobalSettings.ComparePanel.NSPCP_MODE);
            enablePSetSignal(txtMixedPinName.Text, cboMixedInstrument.Text, ref btnMixedPSetsOrSignals);

            // tersw00117703 - clear compare grid if Selection changed
            if (cboMixedInstrument.Text != m_compareOldInstrumentValue)
            {
                // tersw00174642 - in addition, clear it only if no old instrument value present
                if (m_oldInstrumentValue != null)
                {
                    ClearCompareParameterGrid();
                }
            }
        }



        private void txtPinName_Leave(object sender, EventArgs e)
        {

            string strCurrentPinNames;

            strCurrentPinNames = txtPinName.Text.Replace(" ", "");
            // The user had a valid previous selection of NSPCP items
            if (m_strPinNames != String.Empty)
            {
                // different pin name selected
                if (strCurrentPinNames != m_strPinNames)
                {
                    if (CheckForUnsavedDefinitions())
                    {
                        // user is not ready to loose changes
                        txtPinName.Text = m_strPinNames;

                    }

                    else  //user is ready to loose changes
                    {
                        ClearMainParameterGrid();
                        m_strPinNames = string.Empty;  // grid is empty so clear the cache
                    }

                }

            }
            if (txtPinName.Text.Equals(""))
            {
                txtPinName.Text = DEFAULT_TEXT_PIN;
            }

            if (m_bPinNameChanged)
            {
                SetInstrumentsForPins(true);
                m_bPinNameChanged = false;
            }
        }

        /// <summary>
        /// This event is fired when the user enter a grid cell.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgvParametersEdit_CellEnter(object sender, DataGridViewCellEventArgs e)
        {
            //If it is an error cell we remove the error
            m_UIEngine.ClearCell(e.RowIndex, e.ColumnIndex);
            //We store the existing value
            if (dgvParametersEdit[e.ColumnIndex, e.RowIndex].Value != null)
            {
            m_cellCache = (String)dgvParametersEdit[e.ColumnIndex, e.RowIndex].Value;
            }
            else
            {
                m_cellCache = "";
            }

            EnableDisableUndoRedoButtons();

            //Disable Cut and Copy, but not the Paste. Only disable paste if clipboard is empty.According to the DataGrid
            UpdateMenuItemStatus( true );
        }

       
        /// <summary>
        ///  This method is used to update the Status of Cut, Copy menu items, 
        /// when the  focus leave from  dgvParamEdit grid control.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgvParametersEdit_CellLeave(object sender, DataGridViewCellEventArgs e)
        {
            EnableDisableUndoRedoButtons();

            //Disable Cut and Copy, but not the Paste. Only disable paste if clipboard is empty.According to the DataGrid
            UpdateMenuItemStatus(false);
        }

        /// <summary>
        /// Get the list of active sites.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tscboSite_DropDown(object sender, EventArgs e)
        {
            //uncheck Master CheckBox if Site Combobox selection is changed
            _syncMasterSite = false;
            //set default color
            tscboSite.BackColor = GetDefaultWindowColor();
            InitSiteDropDown();
        }

        private void InitSiteDropDown()
        {
            try
            {
                int CurrentIndex = tscboSite.SelectedIndex;
                if (CurrentIndex == -1) //no selection - select 1st item in the list
                    CurrentIndex = 0;

                //Clear the current list of items.
                tscboSite.Items.Clear();

                //Get the list of sites from TDE and add it to the site list toolbar.
                System.Int16[] sitelist = (System.Int16[])m_TDE.SiteList;
                foreach (System.Int16 site in sitelist)
                {
                    tscboSite.Items.Add(Convert.ToString(site));
                }

                HandlingSiteActivation = true;

                masterCheckBox.Checked = _syncMasterSite;
                if (_syncMasterSite)
                    tscboSite.SelectedIndex = m_TDE.Site;
                else
                tscboSite.SelectedIndex = CurrentIndex;

                HandlingSiteActivation = false;
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Error while retrieving site list", exp.StackTrace);
            }
        }

        /// <summary>
        /// This method returns the currently selected site.
        /// </summary>
        /// <returns>Currently selected site.</returns>
        private int GetSelectedSite()
        {
            try
            {
                if (_syncMasterSite)
                {
                    return m_TDE.Site;   //Get the master site number from TDE.
                }
                else
                {
                    return Convert.ToInt16(tscboSite.Text);  //Get the site number from the dropdown.
                }
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Error while retrieving selected site ", exp.StackTrace);
                return GlobalUtils.ERR_VALUE;
            }
        }

        /// <summary>
        /// Returns whether there are any sites active.
        /// </summary>
        /// <returns>Returns true if there are any active sites</returns>
        private Boolean IsSitesActive()
        {
            try
            {
                return (MyServices.SiteMgr.ActiveSites.Count > 0);
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Error while checking site activity", exp.StackTrace);
                return false;
            }
        }

        /// <summary>
        /// Specifies whether a given site is active or not.
        /// </summary>
        /// <param name="SiteNumber">Site Number</param>
        /// <returns>whether a given site is active or not</returns>
        private Boolean IsSitesActive(int SiteNumber)
        {
            try
            {
                return SiteWrapper[SiteNumber].Active;

            }
            catch (Exception e)
            {
                ThrowErrorMessage("Error while checking Site Activity", e.StackTrace);
                return false;
            }
        }

        /// <summary>
        /// Returns whether a Job is valid or not.
        /// </summary>
        /// <returns></returns>
        private Boolean IsJobValid()
        {
            try
            {
                return (Boolean)MyServices.CoreLanguageServiceCOM.Exec.JobIsValid;

            }
            catch (Exception exp)
            {
                MessageBox.Show("Error while checking Job Validity" + exp.StackTrace, this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return false;
            }
        }

        /// <summary>
        /// Handle the action when the user clicks 'Update Tester'
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmiActionsSetDriver_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll(); 
            ShrunkErrorPane();
            HandleSetDriver();
        }

        /// <summary>
        /// Handle the action when the user clicks 'Update Sheet'
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmiActionsUpdateSheet_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();
            ShrunkErrorPane();
            HandleUpdateSheet();
            MyServices.CoreLanguageServiceCOM.DidExecute = false;
        }

        private void HandleUpdateSheet()
        {
            //Collect the information from the grid
            m_UIEngine.CollectGridInformation(ref m_NSPCPList);
            
            foreach (NspcpItem nspcpItem in m_NSPCPList.Values)
            {
                try
                {   
                   // Switch to pset/signal sheet engine depending upon nspcpItem Type
                   if (nspcpItem.propType == GlobalUtils.NspcpType.pset)
                       m_SheetEngine = new PSetSheetEngine(ref psetHelper);
                   else
                       m_SheetEngine = new SignalsSheetEngine(ref signalHelper);

                    //Update the sheet
                    m_SheetEngine.UpdateSheet(nspcpItem);
                    //Set the dirty bit to false
                    m_DefChanged_NotAppliedToSheet = false;
                    //Restore Text Color
                    m_UIEngine.RestoreTextColor();
                }
                catch (ArgumentNullException)
                {
                    //Any or all of the passed parameters are null
                    ThrowErrorMessage("Updating Sheet Error", "Any or all of the passed parameters are null");
                }
                catch (KeyNotFoundException)
                {
                    // PSet/Signal Sheet has an unrecognized parameter associated with it
                    ThrowErrorMessage("Updating Sheet Error", "PSet/Signal Sheet has an unrecognized parameter associated with it");
                }
                catch (ArgumentException)
                {
                    // Any of the Passed information is not valid.
                    ThrowErrorMessage("Updating Sheet Error", "Any of the Passed information is not valid.");
                }
                catch (System.Runtime.InteropServices.COMException)
                {
                    // Do nothing. There is no signal sheet in existence
                    ThrowErrorMessage("Updating Sheet Error", "No sheet exists on the workbook.");
                }
                catch (Exception exp)
                {
                    //Catch block for any other errors that are not handled
                    ThrowErrorMessage("Updating Sheet Error", exp.Message);
                }
            }

            if (!AnyErrors())
            {
                MessageBox.Show("\"Update Sheet\" operation has been successful", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            m_undoManager.Clear(); // Clear buffer as the changes has been commited
            EnableDisableUndoRedoButtons();
        }

        /// <summary>
        /// This method handles the click of the 'Update Tester' Button.
        /// </summary>
        private void HandleSetDriver()
        {
            Boolean gotOriginal = false;

            try
            {
                //Collect the latest paramter values from the parameter grid
                m_UIEngine.CollectGridInformation(ref m_NSPCPList);

                if (!applyAllToolStripButton.Checked)
                {
                    int selectedSite = GetSelectedSite();
                    SiteWrapper.SetOverride(selectedSite);
                    gotOriginal = true;
                }

                //Update the driver.
                m_Main_InstrEngine.SetDefinition(ref m_NSPCPList);

                if (AnyErrors())
                {
                //Report errors (if any) to the UI.
                    ///Checks whether the Error Grid has already initialized or not
                    if (dgvErrors.Rows.Count == 0)
                        m_UIEngine.initializeContainer(ref splitContainer1, grpErrors.Height);//set the current status of splitcontainer and error pane height

                m_UIEngine.ReportErrors(ref m_NSPCPList);
                    m_UIEngine.ScrollToFirstErrorInGrid();
                }
                else
                {
                    //Reset the 'dirty bit'
                    //Set the dirty bit to false
                    m_DefChanged_NotAppliedToDriver = false;
                    //Restore Text Color
                    m_UIEngine.RestoreTextColor();
                    //Reconfigure the buttons on the display
                    configureButtons();

                    //refresh everything (only for driver defintion mode)                   
                    if (m_GlobalSettings.MainPanel.DATA_SOURCE == GlobalUtils.DATA_SOURCE.DRIVER)
                    {
                        m_TDE.HardwareStateChanged();
                    }

                    // clear the undo buffer data now that the data has been set
                    m_undoManager.Clear();
                    EnableDisableUndoRedoButtons();
                }
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Error Updating Driver", exp.Message);
            }
            finally
            {
                //Restore the original list of selected sites (if the variable was initialized).
                if (gotOriginal)
                {
                    SiteWrapper.RestoreFromOverride();
                }
                MyServices.CoreLanguageServiceCOM.DidExecute = false;
            }

            if (!AnyErrors())
            {
                MessageBox.Show("\"Update Tester\" operation has been successful", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        /// <summary>
        /// Checks whether there are any errors in the displayed definitions.
        /// </summary>
        /// <param name="objList"></param>
        /// <returns></returns>
        public Boolean AnyErrors()
        {
            foreach (NspcpItem obj in m_NSPCPList.Values)
            {
                if (obj.propErrInfo.Count > 0)
                {
                    return true;
                }
            }
            return false;
        }

        private void rbnSheetDefinition_CheckedChanged(object sender, EventArgs e)
        {
            if (rbnSheetDefinition.Checked)
            {
                m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
                m_Main_InstrEngine.Rebuild();
                if (!m_suppressEventHandler)
                {
                HandleSourceChanged(GlobalUtils.DATA_SOURCE.SHEET, ref m_GlobalSettings.MainPanel, ref m_NSPCPList,
                                ref m_UIEngine, ref m_GlobalPSetSignalValues.MainPanel, txtPinName.Text, cboInstrument.Text, 
                                ref rbnSheetDefinition, ref rbnCurrentDriverDefinition, ref m_Main_InstrEngine);
            }
                else
                {
                    m_suppressEventHandler = false;
                }
            }
            UpdateMenuItem(SHEETDEFINITION, UpdateMenuItemEnum.CHECKED, rbnSheetDefinition.Checked);  // Update menu item
        }

        private void rbnCurrentDriverDefinition_CheckedChanged(object sender, EventArgs e)
        {
            if (rbnCurrentDriverDefinition.Checked)
            {
                m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
                m_Main_InstrEngine.Rebuild();
                if (!m_suppressEventHandler)
                {
                HandleSourceChanged(GlobalUtils.DATA_SOURCE.DRIVER, ref m_GlobalSettings.MainPanel, ref m_NSPCPList,
                                ref m_UIEngine, ref m_GlobalPSetSignalValues.MainPanel, txtPinName.Text, cboInstrument.Text, 
                                ref rbnSheetDefinition, ref rbnCurrentDriverDefinition, ref m_Main_InstrEngine);
            }
                else
                {
                    m_suppressEventHandler = false;
                }
            }
            UpdateMenuItem(CURRENTDRIVERDEFINITION, UpdateMenuItemEnum.CHECKED, rbnCurrentDriverDefinition.Checked);  // Update menu item
        }

        /// <summary>
        /// Handle the change of nspcp type.
        /// Note: Caller should clear the grid or any other cleanup associated with changing NSPCP type
        /// </summary>
        /// <param name="type"></param>
        public void HandleNSPCPTypeSelection(GlobalUtils.NspcpType type, ref PanelSettings settings, ref InstrumentEngine instr)
        {

            //1) Set global variable to this type.
            settings.NSPCP_MODE = type;

            //2) Update the SheetEngine to be used.
            if (type == GlobalUtils.NspcpType.pset)
            {
                m_SheetEngine = (ISheetEngine)m_psetSheetEngine;
            }
            else
            {
                m_SheetEngine = (ISheetEngine)m_signalsSheetEngine;
            }

            //3) Update the collection edit helper being used in the instrument engine.
            if (instr != null)
            {
                instr.UpdateHelper(type);
            }

            //4) - Enable/Disable appropriate buttons (apply pset, load signal)
            configureButtons();
        }

        /// <summary>
        /// Handle the change of data source for either panel
        /// Note: this routine may cause a cancel of the source change, and will set the radio buttons accordingly
        /// </summary>
        /// <param name="source">Specifies whether source will be sheet or driver.</param>
        private bool HandleSourceChanged(GlobalUtils.DATA_SOURCE source, ref PanelSettings settings,
                ref Dictionary<String, NspcpItem> nspcpList, ref UserInterfaceEngine UI,
                ref PanelNspcpValues values, String pin, String instrument,
                ref System.Windows.Forms.RadioButton rbnSheet, ref System.Windows.Forms.RadioButton rbnDriver,
                ref InstrumentEngine instr)
        {
            Boolean ProceedWithSwitch = false;

            //Check if source can be changed
            if (UI.IsInitialized) //is the panel visible?
            {
                //if false no need to update or check unsaved changes 
                if (m_NeedToCheckChanges)
                {
                // Need to update the PSet/signal information in parameter grid for both Main and Side-by-Side window.
                // Is it an editable panel?.Check for unsaved changes of Main window.
                if (settings.READONLY || !CheckForUnsavedDefinitions())
                {
                    if (CanSwitchSource(source, nspcpList, ref settings, ref values, ref UI, pin, instrument, ref m_Main_InstrEngine))  //Check if source can be switched
                    {
                        ProceedWithSwitch = true;

                    }
                }
                }
                else m_NeedToCheckChanges = true;
            }

            if (!UI.IsInitialized || ProceedWithSwitch)
            {
                //Step 1 - Update the global variable 
                settings.DATA_SOURCE = source;

                //Step 2 - Check the appropriate radio button.
                if (source == GlobalUtils.DATA_SOURCE.DRIVER)
                    rbnDriver.Checked = true;
                else
                    rbnSheet.Checked = true;

                //Update the parameter grid with the definition from the new source            
                if (UI.IsInitialized)
                {
                    UI.ClearAll();
                     
                    ShrunkErrorPane();

                    UpdateParameterGrid(true, ref UI, ref settings, ref values, ref nspcpList, pin, instrument, ref instr);
                }

                //Step 4 - Enable/Disable appropriate buttons (apply pset, load signal)
                configureButtons();
                
                ProceedWithSwitch = true;
            }
            else
            {
                m_suppressEventHandler = true;
                //Restore the radio button changes
                if (settings.DATA_SOURCE == GlobalUtils.DATA_SOURCE.DRIVER)
                {
                    rbnDriver.Checked = true;
                }
                else
                {
                    rbnSheet.Checked = true;
                }
                //The above will trigger the event handler for the radio button switch event again
                //we can suppress that by setting this flag.
            }
            return ProceedWithSwitch;
        }

        /// <summary>
        /// This method is used to add a new PSet/Signal to the existing collection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNewPSetOrSignal_Click(object sender, EventArgs e)
        {
            Boolean bExists = false;
            //Clear all existing errors.
            m_UIEngine.ClearAll();
            ShrunkErrorPane();

            m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);

            //Validate Pin and Instrument Names
            GlobalUtils.DATA_SOURCE source = m_GlobalSettings.MainPanel.DATA_SOURCE;

            if (txtPinName.Text.Equals("") || cboInstrument.Text.Equals(""))
            {
                MessageBox.Show("Please select a valid pin and instrument name before adding a PSet.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            String pinList, txtPinList;
            txtPinList = txtPinName.Text;
            // remove spaces from pinName (in case if pinName is CSL of pins)
            pinList = txtPinList.Replace(" ", "");

            m_Main_InstrEngine.Rebuild();
            
            if (!m_Main_InstrEngine.CheckPin(ref pinList))
            {
                MessageBox.Show("Selected pin name is invalid.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
            if (!m_Main_InstrEngine.CheckInstrument(cboInstrument.Text, true, pinList))
            {
                MessageBox.Show("Selected instrument is invalid or does not map to the selected pin.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            //Show the Dialog box to prompt user to enter a new pset/signal name.
            Form frmAdd = new frmAddNewNspcp(m_GlobalSettings.MainPanel.NSPCP_MODE);
            DialogResult res = frmAdd.ShowDialog();

            if (res == DialogResult.Cancel)
                return; // do nothing
            if (res == DialogResult.OK)
            {
                if (rbnSheetDefinition.Checked)
                {
                    try
                    {
                        if (m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                            MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET);
                        else
                            MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET);
                    }
                    catch
                    {
                        source = GlobalUtils.DATA_SOURCE.DRIVER;
                    }
                }

                try
                {
                    //Get the name from the dialog. Note: Basic validations are done
                    String nspcpName = (frmAdd as frmAddNewNspcp).NSPCP_NAME;

                    //Check if PSet/Signal already exists - either in sheet or driver collection
                    if (ExistsNSPCP(pinList, cboInstrument.Text, nspcpName, source, false, ref  m_Main_InstrEngine))
                    {
                        MessageBox.Show(((m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset) ? "PSet" : "Signal") + 
                        " with name \"" + nspcpName + "\" already exists on the selected data source. Please choose another name.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                    else
                    {
                        // first check if the NCPCP with the same name exists on the grid
                        // (can be it's not added to Sheet/Driver yet)
                        m_UIEngine.CollectGridInformation(ref m_NSPCPList);
                        foreach (NspcpItem nspcpItem in m_NSPCPList.Values)
                        {
                            if (nspcpName == nspcpItem.propName)
                            {
                                MessageBox.Show(((m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset) ? "PSet" : "Signal") + 
                                    " with name \"" + nspcpName + "\" already exists on the grid. Please choose another name.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                                bExists = true;
                            }
                        }
                        if (!bExists)
                        {
                            //Create a new nspcpItem object with this information
                            NspcpItem newObj = new NspcpItem(m_GlobalSettings.MainPanel.NSPCP_MODE, nspcpName.ToLower(), pinList, cboInstrument.Text, "", 0, new Dictionary<string, string>());

                            //Add this object to the global list of values.
                            if (m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                                m_GlobalPSetSignalValues.MainPanel.PSetValues.Add(nspcpName.ToLower());
                            else
                                m_GlobalPSetSignalValues.MainPanel.SignalValues.Add(nspcpName.ToLower());

                            //Configure the object to get the parameter names.
                            Boolean[] enables = m_Main_InstrEngine.Configure(ref newObj);

                            if (!m_UIEngine.IsInitialized)
                            {
                                m_UIEngine.InitDisplay(newObj.NameValuePairs.Keys);
                                m_UIEngine.IsInitialized = true;
                            }

                            //Add the column with the pset/signal value in the grid.
                            m_UIEngine.AddColumn(newObj);

                            //Populate the column with the values.
                            m_UIEngine.PopulateColumn(newObj);

                            //After Population, enable disable the cells
                            m_UIEngine.ConfigureCells(enables, newObj.propName);

                            //Add the object definition to the list of existing 
                            //nspcp definitions being displayed.
                            if (!m_NSPCPList.ContainsKey(newObj.propName))
                            {
                                m_NSPCPList.Add(newObj.propName, newObj);
                            }

                            //We set the dirty bit to true since we did not save this pset/signal yet.
                            m_DefChanged_NotAppliedToSheet = true;
                            m_DefChanged_NotAppliedToDriver = true;
                            configureButtons();
                        }
                    }
                    for (int i = 1; i < dgvParametersEdit.Columns.Count; i++)
                    {
                        dgvParametersEdit.Columns[i].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells ;
                    }
                    MyServices.CoreLanguageServiceCOM.DidExecute = false;
                }
                catch (Exception exp)
                {
                    ThrowErrorMessage("Error selecting " + 
                        ((m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset) ? "PSet" : "Signal") 
                        , exp.Message);
                }

            }
        }

        //Check if the selected site is active or not and take appropriate actions.
        private void tscboSite_SelectedIndexChanged(object sender, EventArgs e)
        {
            ApplySite();
        }

        /// <summary>
        /// Adding a new PSet/Signal based on existing definition.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDuplicatePSetOrSignal_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();

            ShrunkErrorPane();

            m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
            //Assert : Grid is initialized
            if (!m_UIEngine.IsInitialized)
            {
                MessageBox.Show("Cannot Duplicate : Display not initialized.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
            //Assert : Only one definition is being displayed.
            if (m_NSPCPList.Count > 1)
            {
                MessageBox.Show("Cannot Duplicate : Duplication cannot occur when more than one definition is being displayed.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
            //Assert : Definition is validated (not changed)
            if (m_DefChanged_NotAppliedToDriver && (m_GlobalSettings.MainPanel.DATA_SOURCE == GlobalUtils.DATA_SOURCE.DRIVER))
            {
                MessageBox.Show("Cannot Duplicate : Definition has been changed, either 'Refresh' or 'Update Tester'/'Update Sheet' the current definition and try again.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
            //There are not validation errors
            if (AnyErrors())
            {
                MessageBox.Show("Cannot Duplicate : There are errors in the currently displayed definitions.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            m_Main_InstrEngine.Rebuild();

            Form frmAdd = new frmAddNewNspcp(m_GlobalSettings.MainPanel.NSPCP_MODE);
            DialogResult res = frmAdd.ShowDialog();
            if (res == DialogResult.Cancel)
                return; // do nothing
            if (res == DialogResult.OK)
            {
                String nspcpName = (frmAdd as frmAddNewNspcp).NSPCP_NAME; // Basic validations are already done

                if (ExistsNSPCP(txtPinName.Text, cboInstrument.Text, nspcpName, m_GlobalSettings.MainPanel.DATA_SOURCE, false, ref  m_Main_InstrEngine))
                {
                    MessageBox.Show(((m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset) ? "PSet" : "Signal") +
                        " with name \"" + nspcpName + "\" already exists on the selected data source. Please choose another name.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    //Update the nspcp definitions from the grid
                    m_UIEngine.CollectGridInformation(ref m_NSPCPList);

                    CreateNewFromExisting(nspcpName.ToLower(), m_GlobalSettings.MainPanel.DATA_SOURCE);

                    // update undo
                    m_undoManager.Clear();
                    EnableDisableUndoRedoButtons();
                }
            }
        }

        /// <summary>
        /// This method is used to create a new pset/signal
        /// and add it to the instrument driver.
        /// </summary>
        /// <param name="nspcpName">name of the new pset/signal</param>
        private void CreateNewFromExisting(String nspcpName, GlobalUtils.DATA_SOURCE source)
        {
            NspcpItem newObj;
            //Get the name of the currently displayed nspcp
            foreach (NspcpItem obj in m_NSPCPList.Values)
            {
                //Create a new object with the new name.
                newObj = obj.Duplicate(nspcpName);

                if (source == GlobalUtils.DATA_SOURCE.DRIVER)
                {
                    DialogResult res = MessageBox.Show("This PSet/Signal will be added to the instrument driver, proceed?", GlobalUtils.MESSAGEBOX_CAPTION, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (res == DialogResult.Yes)
                    {
                        //Set this object in the grid.
                        m_Main_InstrEngine.SetDefinition(ref newObj);
                    }
                    else
                    {
                        return;
                    }
                }
                else
                {
                    DialogResult res = MessageBox.Show("This PSet/Signal will be added to the appropriate sheet, proceed?", GlobalUtils.MESSAGEBOX_CAPTION, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (res == DialogResult.Yes)
                    {
                        try
                        {
                            m_SheetEngine.UpdateSheet(newObj);
                        }
                        catch (ArgumentNullException)
                        {
                            //Any or all of the passed parameters are null
                            ThrowErrorMessage("Updating Sheet Error", "Any or all of the passed parameters are null");
                        }
                        catch (KeyNotFoundException)
                        {
                            // PSet/Signal Sheet has an unrecognized parameter associated with it
                            ThrowErrorMessage("Updating Sheet Error", "PSet/Signal Sheet has an unrecognized parameter associated with it");
                        }
                        catch (ArgumentException)
                        {
                            // Any of the Passed information is not valid.
                            ThrowErrorMessage("Updating Sheet Error", "Any of the Passed information is not valid.");
                        }
                        catch (System.Runtime.InteropServices.COMException)
                        {
                            // Do nothing. There is no signal sheet in existence
                            ThrowErrorMessage("Updating Sheet Error", "No sheet exists on the workbook.");
                        }
                        catch (Exception exp)
                        {
                            //Catch block for any other errors that are not handled
                            ThrowErrorMessage("Updating Sheet Error", exp.Message);
                        }
                    }
                    else
                    {
                        return;
                    }
                }
            }
            //Get the current cursor and save it before we set it to wait cursor and reset it at the end.            
            Cursor PrevCursor = Cursor.Current;
            Cursor.Current = Cursors.WaitCursor;
            if (m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset)
            {
                m_GlobalPSetSignalValues.MainPanel.PSetValues.Add(nspcpName.ToLower());
            }
            else
            {
                m_GlobalPSetSignalValues.MainPanel.SignalValues.Add(nspcpName.ToLower());
            }

            m_TDE.HardwareStateChanged();
            System.Collections.ArrayList arrList = HandleRemoveColumns(ref m_NSPCPList, ref m_UIEngine, 
                                                ref m_GlobalSettings.MainPanel, ref m_GlobalPSetSignalValues.MainPanel);
            HandleSelectedNspcp(txtPinName.Text, cboInstrument.Text, ref arrList, ref m_GlobalSettings.MainPanel, ref m_GlobalPSetSignalValues.MainPanel, ref m_Main_InstrEngine);
            if (dgvParametersEdit.Columns.Count == DUP_COLUMN_COUNT)
            {
                dgvParametersEdit.Columns[DUPLICATE_COLUMN].DefaultCellStyle.BackColor = dgvParametersEdit.Columns[MAIN_COLUMN].DefaultCellStyle.BackColor;
            }
            // put back the saved cursor, once updation is over.
            Cursor.Current = PrevCursor;
        }

        /// <summary>
        /// This action applies a pset to the instrument hardware.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmiActionsApplyPSet_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();

            ShrunkErrorPane();

            HandleApplyLoad();
        }

        /// <summary>
        /// This action loads a signal.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmiActionsLoadSignals_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();

            ShrunkErrorPane();

            HandleApplyLoad();
        }

        private void HandleApplyLoad()
        {
            //Assert : The display has been initialized and there are some psets displayed.
            if (!m_UIEngine.IsInitialized)
            {
                MessageBox.Show("The display is not initialized.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

            //Assert : The display has been initialized and there are some psets displayed.
            if (!(m_NSPCPList.Count > 0))
            {
                MessageBox.Show("There are no PSets/Signals being shown currently.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

            //Assert : There are no errors on the grid and that the pset(s) has 
            //been validated.
            if (m_DefChanged_NotAppliedToDriver)
            {
                MessageBox.Show("Cannot Apply. The tester has not been updated.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            //Assert : There are no errors currently being displayed.
            if (AnyErrors())
            {
                MessageBox.Show("Cannot apply/load PSet/signal, there are still errors in the definition.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

            Boolean gotOriginal = false;

            try
            {
                String nspcpName="";

                if (m_NSPCPList.Count > 1)
                {
                    frmApply SelectForm = new frmApply(ref m_NSPCPList, m_GlobalSettings);
                    DialogResult res = SelectForm.ShowDialog();

                    if (res == DialogResult.OK)
                    {
                        //Get the name of the NSPCP
                        nspcpName = SelectForm.NSPCP_NAME;

                    }
                }
                else  //Only one NSPCP is displayed on the parameter grid.
                {
                    foreach (String name in m_NSPCPList.Keys)
                    {
                        nspcpName = name;
                    }
                }

                if (nspcpName != "")
                {
                    if (m_NSPCPList.ContainsKey(nspcpName))
                    {
                        //Override appropriate site only when "Apply to all Site" button not selected.
                        if (!applyAllToolStripButton.Checked)
                        {
                            //Select the site to apply on.
                            int selectedSite = GetSelectedSite();
                            SiteWrapper.SetOverride(selectedSite);
                            gotOriginal = true;
                        }
                        NspcpItem obj = m_NSPCPList[nspcpName];
                        List<String> errors = new List<String>(); ;
                        if (!m_Main_InstrEngine.ApplyOrLoad(obj.propLogicalInstrument, obj.pinName, obj.propName, ref errors))
                        {
                            foreach(String error in errors) {
                                ThrowErrorMessage("Apply/Load Error", error);
                            }
                        }

                        m_TDE.HardwareStateChanged();
                    }
                    else
                    {
                        throw new Exception("Cannot find the definition name '" + nspcpName + "' in the list of available definitions");
                    }
                }
                
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Load Settings/Apply PSet Failed", exp.Message);
            }
            finally
            {
                //Restore sites.
                if (gotOriginal)
                {
                    SiteWrapper.RestoreFromOverride();
                }
            }

            if (!AnyErrors())
            {
                MessageBox.Show("\"Apply PSet/Load Signal Settings\" operation has been successful", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        #region Code Generation feature
        //CopyCode Click Event
        private void tsbCopyCode_Click(object sender, EventArgs e)
        {
            String strLogicalInstrument;
            const string TDE_TOOL_NAME = "PSet/Signals Editor";
            const string ERR_TYPE = "Copy Code";
            List<String> errInfo = null;

            try
            {
                //Clear all existing errors.
                m_UIEngine.ClearAll();

                ShrunkErrorPane();

                //Collect the information from the grid
                m_UIEngine.CollectGridInformation(ref m_NSPCPList);

                if (m_NSPCPList.Count > 0)
                {
                    foreach (NspcpItem psItem in m_NSPCPList.Values)
                    {
                        if (psItem != null)
                        {
                            strLogicalInstrument = psItem.propLogicalInstrument;
                            PARAMETERCOLLECTIONINTERFACELib.IParameterCollection pColl = m_Main_InstrEngine.GetInstrumentInterface(strLogicalInstrument);
                            if (pColl != null)
                            {
                                //calls the Code Engine's Generate Code method
                                m_strCode += m_CodeGenEngine.GenerateCode(psItem, pColl, ref errInfo);
                                m_strCode += "\n";

                                if (errInfo != null && errInfo.Count > 0)
                                {
                                    // tersw00175087   Ensure that errors can display when dialog is closed and then re-opened. 
                                    //                 Then reset the code string so that its content is not retained 
                                    //                 and included in the next successful Copy Code operation
                                    if (splitContainer1 == null || dgvErrors.Rows.Count == 0)
                                    {
                                        m_UIEngine.initializeContainer(ref splitContainer1, grpErrors.Height);
                                    }
                                    m_strCode = string.Empty;
                                    m_UIEngine.ReportErrorInListBox(ERR_TYPE, errInfo);
                                    return;
                                }
                            }
                        }
                    }
                    if (m_strCode != String.Empty)
                    {
                        //Format the code based on the Language selection and with block options
                        m_strCode = m_CodeGenEngine.FormatVBTCode(m_strCode, m_bNeedWithBlock, m_bNewLang);
                        //Log the Formatted code to the CodeViewer tool through TDE
                        m_TDE.CSAppendCode(m_strCode, TDE_TOOL_NAME);
                        m_strCode = String.Empty;
                    }
                }
            }
            catch (Exception ex)
            {
                Console.Write("frmPSetSignalsEditor:tsbCopyCode_Click Failed", ex.Message);
            }
        }

        //Enables or disable the tsbCopyCode button based on the nspcp selection
        private void configureCopyCodeButton()
        {
            bool bEnable = false;
            int selectedSite = 0;
            if (m_UIEngine.IsInitialized && m_NSPCPList.Count > 0 && IsSitesActive(selectedSite))
                bEnable = true;
            
            tsbCopyCode.Enabled = bEnable;
            UpdateMenuItem(COPYCODE, UpdateMenuItemEnum.ENABLED, bEnable);  // Update menu item            
        }

        #region TDEToolMenu Code

        // Adds an activebar tool in the specified band and sets its properties
        private ActiveBar3Library.Tool AddMenuItem(string strBand, string strTool, string strCaption, string strTooltip, ActiveBar3Library.ToolTypes tooltypeABItem, 
                                                    String strShortcut)
        {
            ActiveBar3Library.Tool aTool;
            aTool = AddABItem(ref m_actBarPSE, ref strBand, ref strTool, false, strCaption, tooltypeABItem, ActiveBar3Library.ToolStyles.ddSStandard, strTooltip, null, strShortcut);
            return aTool;            
        }

        //Builds the TDEToolMenu with TDECommands and ActiveBar.
        private void InitializeTDEToolMenu()
        {
            ActiveBar3Library.Tool aTool;
            const String MNUCODESETTINGS = "mnuCodeSettings";
            const String MNUSEP = "CSSep";
            const String TOOL_CODESETTINGS = "CodeSettings";

            try
            {
                // Add Tool specific menu items
                //Code Settings - PSet Language, New PSet, separator and With Block
                aTool = AddMenuItem(MNUCODESETTINGS, PSETSIGNALSLANGUAGE, "&PSet/Signals Language", "Generate PSet/Signal Language", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool.Checked = true;
                aTool = AddMenuItem(MNUCODESETTINGS, NEWPSETSIGNALS, "&New PSet/Signals", "Creates New PSet/Signal", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool = AddMenuItem(MNUCODESETTINGS, MNUSEP, "", "", ActiveBar3Library.ToolTypes.ddTTSeparator,null);
                aTool = AddMenuItem(MNUCODESETTINGS, NEEDWITHBLOCK, "&With Block", "Generate PSet/Signal Language", ActiveBar3Library.ToolTypes.ddTTButton,null);

                aTool = AddMenuItem(MNUTOOL, TOOL_CODESETTINGS, "Code &Settings", "Code Settings", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool.SubBand = MNUCODESETTINGS;

                aTool = AddMenuItem(MNUTOOL, APPLYTOALLSITES, "&Apply to All Sites", "Apply to all sites", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool = AddMenuItem(MNUTOOL, MIXEDCOMPAREGRID, "Side-b&y-Side        Ctrl+M", "Expand / collapse window for comparing two or more PSets or Signals", ActiveBar3Library.ToolTypes.ddTTButton,null);

                aTool = AddMenuItem(MNUTOOL, MNUSEP, "", "", ActiveBar3Library.ToolTypes.ddTTSeparator,null);

                aTool = AddMenuItem(MNUTOOL, NEWPSET, "&New PSet Or Signal", "Create a new PSet or Signal", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool = AddMenuItem(MNUTOOL, DUPLICATEPSET, "&Duplicate PSet Or Signal", "Duplicate PSet or Signal", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool.Enabled = false;

                aTool = AddMenuItem(MNUTOOL, MNUSEP, "", "", ActiveBar3Library.ToolTypes.ddTTSeparator,null);

                aTool = AddMenuItem(MNUTOOL, SHEETDEFINITION, "S&heet Definition", "Sheet definition", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool.Checked = true;
                aTool = AddMenuItem(MNUTOOL, CURRENTDRIVERDEFINITION, "&Current Driver Definition", "Current driver definition", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool.Enabled = false;
                
                aTool = AddMenuItem(MNUTOOL, MNUSEP, "", "", ActiveBar3Library.ToolTypes.ddTTSeparator,null);

                aTool = AddMenuItem(MNUTOOL, SETDRIVER, "Update &Tester", "Set Driver", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool.Enabled = false;
                aTool = AddMenuItem(MNUTOOL, UPDATESHEET, "&Update Sheet", "Update Sheet", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool.Enabled = false;
                aTool = AddMenuItem(MNUTOOL, APPLYPSET, "Apply &PSet", "Apply PSet", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool.Enabled = false;
                aTool = AddMenuItem(MNUTOOL, LOADSIGNALSETTINGS, "&Load Signal Settings", "Load Signal Settings", ActiveBar3Library.ToolTypes.ddTTButton,null);
                aTool.Enabled = false;

                // Add Edit menu items                
                aTool = AddMenuItem(MNUEDIT, CUT,  "Cu&t", "Cut", ActiveBar3Library.ToolTypes.ddTTButton,"Control+X");
                //Defect Fix: tersw00117339
                aTool.Enabled = false;
                aTool = AddMenuItem(MNUEDIT, COPY, "&Copy", "Copy", ActiveBar3Library.ToolTypes.ddTTButton,"Control+C");
                //Defect Fix: tersw00117339
                aTool.Enabled = false;
                aTool = AddMenuItem(MNUEDIT, PASTE,"&Paste", "Paste", ActiveBar3Library.ToolTypes.ddTTButton,"Control+V");
                //Defect Fix: tersw00117339
                if( Clipboard.ContainsText())
                    aTool.Enabled = true;
                else
                    aTool.Enabled = false;

                
                aTool = AddMenuItem(MNUEDIT, MNUSEP, "", "", ActiveBar3Library.ToolTypes.ddTTSeparator,null);

                aTool = AddMenuItem(MNUEDIT, UNDO, "&Undo", "Undo", ActiveBar3Library.ToolTypes.ddTTButton,"Control+Z");
                aTool.Enabled = false;
                aTool = AddMenuItem(MNUEDIT, REDO, "&Redo", "Redo", ActiveBar3Library.ToolTypes.ddTTButton,"Control+Y");
                aTool.Enabled = false;

                aTool = AddMenuItem(MNUEDIT, MNUSEP, "", "", ActiveBar3Library.ToolTypes.ddTTSeparator,null);

                aTool = AddMenuItem(MNUEDIT, COPYCODE,"C&opy Code", "Copy Code", ActiveBar3Library.ToolTypes.ddTTButton,"Control+Shift+C");
                aTool.Enabled = false;
            }
            catch (Exception e)
            {
                Console.Write("Exception caught in frmPSetSignalsEditor:InitializeTDEToolMenu {0}", e.StackTrace);
            }
        }

        // Adds an activebar tool in the specified band and sets its properties
        private ActiveBar3Library.Tool AddABItem(ref ActiveBar3Library.ActiveBar3 ret_ActiveBar, ref string strBand, ref string strTool, bool bAddToToolbar, 
                    string strCaption, ActiveBar3Library.ToolTypes tooltypeABItem, ActiveBar3Library.ToolStyles toolstyleABItem, string tooltipABItem, 
                    stdole.IPictureDisp iconABItem, String strShortcut)
        {
            ActiveBar3Library.Tool functionReturnValue = null;
            ActiveBar3Library.Band band;
            ActiveBar3Library.Tool tool;

            // Get or create the Band
            band = AddABBand(ref ret_ActiveBar, ref strBand, true);

            // Add Tool
            tool = AddABTool(ref band, ref strTool);

            SetABToolProperties(ref tool, strCaption, tooltypeABItem, toolstyleABItem, tooltipABItem, iconABItem, strShortcut);

            // Add to toolbar, if needed
            if (bAddToToolbar)
            {
                strBand = "Toolbar";
                band = AddABBand(ref ret_ActiveBar, ref strBand, false);
                band.Tools.Insert(-1, tool);
            }

            functionReturnValue = tool;
            return functionReturnValue;
        }

        //Adds an actibar band
        private ActiveBar3Library.Band AddABBand(ref ActiveBar3Library.ActiveBar3 ret_activebarAddBand, ref string strBand, bool bPopupMenu)
        {
            ActiveBar3Library.Band band = null;
            //Get or create the Band - No facility is available to check the presence of the particular band
            try
            {
                object objBand = (object)strBand;
                band = ret_activebarAddBand.Bands.Item(ref objBand);
            }
            catch (Exception ex)
            {
                band = ret_activebarAddBand.Bands.Add(strBand);
                if (bPopupMenu) band.Type = ActiveBar3Library.BandTypes.ddBTPopup;
            }

            return band;
        }

        //Adds an actibar tool
        private ActiveBar3Library.Tool AddABTool(ref ActiveBar3Library.Band ret_bandAddTool, ref string strTool)
        {
            ActiveBar3Library.Tool tool;

            tool = ret_bandAddTool.Tools.Add(m_toolID, strTool);
            m_toolID = (m_toolID + 1);
            return tool;
        }

        // Sets activebar tool's properties
        private void SetABToolProperties(ref ActiveBar3Library.Tool tool, string strCaption, ActiveBar3Library.ToolTypes tooltypeABItem,
                                    ActiveBar3Library.ToolStyles toolstyleABItem, string tooltipABItem, stdole.IPictureDisp iconABItem, String strShortcut)
        {
            // Initialize the tool
            tool.Caption = strCaption;
            tool.ControlType = tooltypeABItem;
            tool.Style = toolstyleABItem;
            tool.TooltipText = tooltipABItem;

            //Below line of code is an work-around for defect TERSW00117641. Kindly refer the analysis document in CQ for more details.
            //Problem is with the Activebar component, as short-cut key events are not getting posted to PSSE, rather it was sent to TDE Frame.
            tool.TagVariant = CUMSTOM_MENU_ENTRY;            
            if ((iconABItem != null))
            {
                tool.SetPicture(ActiveBar3Library.ImageTypes.ddITNormal, iconABItem, null);
            }

            if (!String.IsNullOrEmpty(strShortcut))
            {
                ActiveBar3Library.ShortCut[] shortcuts = new ActiveBar3Library.ShortCut[1];
                shortcuts[0] = new ActiveBar3Library.ShortCut();
                shortcuts[0].Value = strShortcut;
                tool.ShortCuts = shortcuts;
            }

        }

        //Command Handler for TDECommands_Command event
        private void TDECommandsControl_Command(ref string key)
        {
            object senderTemp = null;
            EventArgs eTemp = null;

            if (key == SETDRIVER || key == UPDATESHEET || key == APPLYPSET || key == LOADSIGNALSETTINGS)
            {
                m_UIEngine.ClearAll();
                 
                ShrunkErrorPane();

                m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
            }

            switch (key)
            {
                case PSETSIGNALSLANGUAGE:
                    m_bNewLang = false;
                    UpdateMenuItem(PSETSIGNALSLANGUAGE, UpdateMenuItemEnum.CHECKED, true);
                    UpdateMenuItem(NEWPSETSIGNALS, UpdateMenuItemEnum.CHECKED, false);
                    break;
                case NEWPSETSIGNALS:
                    m_bNewLang = true;
                    UpdateMenuItem(NEWPSETSIGNALS, UpdateMenuItemEnum.CHECKED, true);
                    UpdateMenuItem(PSETSIGNALSLANGUAGE, UpdateMenuItemEnum.CHECKED, false);
                    break;
                case NEEDWITHBLOCK:
                    m_bNeedWithBlock = !m_bNeedWithBlock;
                    UpdateMenuItem(NEEDWITHBLOCK, UpdateMenuItemEnum.CHECKED, m_bNeedWithBlock);
                    break;
                case APPLYTOALLSITES:
                    applyAllToolStripButton.Checked = !applyAllToolStripButton.Checked;
                    UpdateMenuItem(APPLYTOALLSITES, UpdateMenuItemEnum.CHECKED, applyAllToolStripButton.Checked);
                    break;
                case MIXEDCOMPAREGRID:
                    tsbCompareToggle_Click(senderTemp, eTemp);
                    break;
                case NEWPSET:
                    btnNewPSetOrSignal_Click(senderTemp, eTemp);
                    break;
                case DUPLICATEPSET:
                    btnDuplicatePSetOrSignal_Click(senderTemp, eTemp);
                    break;
                case SHEETDEFINITION:                    
                    rbnSheetDefinition.Checked = true;
                    break;
                case CURRENTDRIVERDEFINITION:
                    rbnCurrentDriverDefinition.Checked = true;
                    break;
                case SETDRIVER:                    
                    HandleSetDriver();
                    CompareNspcps();
                    break;
                case UPDATESHEET:                    
                    HandleUpdateSheet();                    
                    break;
                case APPLYPSET:                
                case LOADSIGNALSETTINGS:                    
                    HandleApplyLoad();
                    break;
                case CUT:
                    cutToolStripButton_Click(senderTemp, eTemp);
                    break;
                case COPY:
                    copyToolStripButton_Click(senderTemp, eTemp);
                    break;
                case PASTE:
                    pasteToolStripButton_Click(senderTemp, eTemp);
                    break;
                case UNDO:
                    tsbUndo_Click(senderTemp, eTemp);
                    break;
                case REDO:
                    tsbRedo_Click(senderTemp, eTemp);
                    break;
                case COPYCODE:
                    if (tsbCopyCode.Enabled == true)
                        tsbCopyCode_Click(senderTemp, eTemp);
                    break;
                case HELPTOPIC:
                    m_TDE.ShowHelp(this.Handle.ToInt32(), "TDE.chm", TDECOMMONSRVLib.TDEHelpCommand.tdeHelpContext, 13000);
                    break;
                case HELPSPECIFICATIONS:
                    break;
                case HELPTOOL:
                    break;
                case HELPABOUTTOOL:
                    MessageBox.Show(string.Format(Properties.Resources.AboutPSSEToolString, TOOLVERSION, Environment.NewLine), "About PSSE",
                                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                    break;
                default:
                    Console.Write("TDECommandsControl_Command: Invalid selection");
                    break;
            }
        }

        //Updates the MenuItem based on the selection
        private void UpdateMenuItem(String strTool, UpdateMenuItemEnum menuProp, Boolean bEnable)
        {
            object objTool = null;
            ActiveBar3Library.Tool aTool;

            try
            {
            objTool = m_TDECommands.GetToolMenuEntry(ref strTool);
            }
            catch (Exception ex)
            {
                Console.Write("Exception caught in frmPSetSignalsEditor:UpdateMenuItem {0}", ex.StackTrace);                
            }

            if (objTool != null)
            {
                aTool = (ActiveBar3Library.Tool)objTool;
                switch (menuProp)
                {
                    case UpdateMenuItemEnum.CHECKED:
                    aTool.Checked = bEnable;
                        break;
                    case UpdateMenuItemEnum.ENABLED:
                        aTool.Enabled = bEnable;
                        break;
                    case UpdateMenuItemEnum.VISIBLE:
                        aTool.Visible = bEnable;
                        break;
                    default:
                        break;
                }
            }
        }

        //Updates the MenuItem based on the selection
        private void UpdateMenuFaceName(String strTool, string faceName)
        {
            object objTool = null;
            ActiveBar3Library.Tool aTool;

            try
            {
                objTool = m_TDECommands.GetToolMenuEntry(ref strTool);
            }
            catch (Exception ex)
            {
                Console.Write("Exception caught in frmPSetSignalsEditor:UpdateMenuItem {0}", ex.StackTrace);
            }

            if (objTool != null)
            {
                aTool = (ActiveBar3Library.Tool)objTool;
                aTool.Caption = faceName;
            }
        }

        #endregion TDEToolMenu
        #endregion Code Generation feature

        /// <summary>
        /// Gets or sets the unit test flag
        /// </summary>
        private Boolean UnitTest
        {
            get { return m_bUnitTest; }
            set { m_bUnitTest = value; }
        }

        /// <summary>
        /// Invoke message box and display the the message to the user if the unit test mode is not enabled
        /// </summary>
        /// <param name="strMessage">Message to be displayed to the user</param>
        private void ShowMessageBox(String strMessage, String strTitle)
        {
            if (m_bUnitTest == false)
            {
                if(strTitle == String.Empty)
                    strTitle = this.Name;
                MessageBox.Show(strMessage, strTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        /// <summary>
        /// Copies selected cells to Clipboard and clears the selected cells
        /// </summary>
        private void cutToolStripButton_Click(object sender, EventArgs e)
        {
            if (CopyCellsToClipBoard())
            {
                try
                {
                    DataGridViewSelectedCellCollection collectionSelectedCells = dgvParametersEdit.SelectedCells;

                    // Clear Selected Cells

                    m_undoManager.BeginContext();

                    for (short sIndex = 0; sIndex < collectionSelectedCells.Count; sIndex++)
                    {
                        if (collectionSelectedCells[sIndex].ReadOnly == true)
                            continue; // We skip read only cells.

                        if ((collectionSelectedCells[sIndex].GetType().Name == "DataGridViewComboBoxCellEx") ||
                            (collectionSelectedCells[sIndex].GetType().Name == "DataGridViewTextBoxCell"))
                        {
                            String oldValue = collectionSelectedCells[sIndex].Value != null ? collectionSelectedCells[sIndex].Value.ToString() : null;
                            collectionSelectedCells[sIndex].Value = "";

                            m_undoManager.SaveState(this, new GridUndoInfo(collectionSelectedCells[sIndex].RowIndex,
                                                                           collectionSelectedCells[sIndex].ColumnIndex,
                                                                           oldValue, ""));
                        }
                    }

                    m_undoManager.EndContext();
                    EnableDisableUndoRedoButtons();
            
                    //Enable Paste MenuItem on both Main Menu/ PSSE menu,whenever user fires the Cut Event
                    CheckAndUpdatePasteMenuItem();
                }
                catch (Exception exp)
                {
                    ThrowErrorMessage("Exception caught in cutToolStripButton_Click : ", exp.Message);
                }
            }
        }

        /// <summary>
        /// Copies selected cells to Clipboard
        /// </summary>        
        private void copyToolStripButton_Click(object sender, EventArgs e)
        {
            CopyCellsToClipBoard();

            //Enable Paste MenuItem on both Main Menu/ PSSE menu,whenever user fires the Cut Event
            CheckAndUpdatePasteMenuItem();
        }

        /// <summary>
        /// Pastes Clipboard data at the selected destination
        /// </summary>
        private void pasteToolStripButton_Click(object sender, EventArgs e)
        {
            try
            {
                // it is possible that the user may click on copy / cut when the focus is with another control
                // in which case, we do not copy any data in the grid.
                //
                // But, sometimes, focus is false when a cell is in edit mode, so allow those to be copied.

                if (m_bUnitTest == false && !(dgvParametersEdit.IsCurrentCellInEditMode || dgvParametersEdit.Focused))
                    return;

                if (dgvParametersEdit.SelectedCells.Count == 0)
                {
                    // Select the destination for Paste operation
                    ShowMessageBox(Teradyne.Igxl.PSetSignalsEditor.Properties.Resources.str101, "Parameter Grid - Paste");
                    return;
                }

                String strTest = String.Empty;
                AccessClipboardMethods(ClipBoardAccessEnum.GETTEXT, ref strTest, null);

                if (strTest.Length == 0)
                {
                    // Clipboard is empty
                    ShowMessageBox(Teradyne.Igxl.PSetSignalsEditor.Properties.Resources.str102, "Parameter Grid - Paste");
                    return;
                }

                short sRowCount, sColCount = 0;
                short sIndex, sColIndex;
                String[] strarrColumns;
                String[] strSplitter = new String[] { "\r\n" };
                System.Collections.Generic.List<String> listDataGridCells = new System.Collections.Generic.List<String>();

                // Separating Rows
                String[] strarrRows = strTest.Split(strSplitter, StringSplitOptions.None);
                strSplitter = null;
                sRowCount = (short)strarrRows.Length;

                // Interpret Columns
                for (sIndex = 0; sIndex < sRowCount; sIndex++)
                {
                    // Separating Columns
                    strarrColumns = strarrRows[sIndex].Split('\t');

                    // Update column count for the first row
                    if (sIndex == 0)
                        sColCount = (short)strarrColumns.Length;

                    // Extract Cells
                    for (sColIndex = 0; sColIndex < sColCount; sColIndex++)
                        listDataGridCells.Add(strarrColumns[sColIndex]);
                }

                // Get the row and column indices of the top leftmost cell of the 
                // selected cells collection
                int iCurrentCellRow = 0, iCurrentCellCol = 0;
                int iRow = 0, iCol = 0, iRowTemp = 0, iColTemp = 0;

                GetTopLeftCell(ref iCurrentCellRow, ref iCurrentCellCol);

                // Destination availability check
                if ((sColCount > (dgvParametersEdit.ColumnCount - iCurrentCellCol)) ||
                    (sRowCount > (dgvParametersEdit.RowCount - iCurrentCellRow)))
                {
                    // The Clipboard data can not fit into the selected space
                    ShowMessageBox(Teradyne.Igxl.PSetSignalsEditor.Properties.Resources.str103, "Parameter Grid - Paste");
                    return;
                }

                String oldValue;

                m_undoManager.BeginContext();

                // In case a cell is in edit mode, end it so that we can update the cell
                if (dgvParametersEdit.IsCurrentCellInEditMode)
                    dgvParametersEdit.EndEdit();

                // Pasting
                for (sIndex = 0; ; sIndex++)
                {
                    iColTemp = iCurrentCellCol + iCol;
                    iRowTemp = iCurrentCellRow + iRow;

                    // Should be editable
                    if (dgvParametersEdit[iColTemp, iRowTemp].ReadOnly == false)
                    {
                        switch (dgvParametersEdit[iColTemp, iRowTemp].GetType().Name)
                        {
                            case "DataGridViewTextBoxCell":
                                oldValue = dgvParametersEdit[iColTemp, iRowTemp].Value != null ? dgvParametersEdit[iColTemp, iRowTemp].Value.ToString() : null;
                                dgvParametersEdit[iColTemp, iRowTemp].Value = listDataGridCells[sIndex];
                                m_undoManager.SaveState(this, new GridUndoInfo(iRowTemp, iColTemp, oldValue, dgvParametersEdit[iColTemp, iRowTemp].Value.ToString()));
                                break;
                            case "DataGridViewComboBoxCellEx":
                                DataGridViewComboBoxCell comboCell = (DataGridViewComboBoxCell)dgvParametersEdit[iColTemp, iRowTemp];
                                if (comboCell.Items.Contains(listDataGridCells[sIndex]) == true)
                                {
                                    oldValue = comboCell.Value != null ? comboCell.Value.ToString() : null;
                                    comboCell.Value = listDataGridCells[sIndex];
                                    m_undoManager.SaveState(this, new GridUndoInfo(iRowTemp, iColTemp, oldValue, comboCell.Value.ToString()));
                                }
                                break;
                            case "DataGridViewCheckBoxCell":
                                switch (listDataGridCells[sIndex])
                                {
                                    case "False":
                                        oldValue = dgvParametersEdit[iColTemp, iRowTemp].Value != null ? dgvParametersEdit[iColTemp, iRowTemp].Value.ToString() : null;
                                        dgvParametersEdit[iColTemp, iRowTemp].Value = CheckState.Unchecked;
                                        m_undoManager.SaveState(this, new GridUndoInfo(iRowTemp, iColTemp, oldValue, dgvParametersEdit[iColTemp, iRowTemp].Value.ToString()));
                                        break;
                                    case "True":
                                        oldValue = dgvParametersEdit[iColTemp, iRowTemp].Value != null ? dgvParametersEdit[iColTemp, iRowTemp].Value.ToString() : null;
                                        dgvParametersEdit[iColTemp, iRowTemp].Value = CheckState.Checked;
                                        m_undoManager.SaveState(this, new GridUndoInfo(iRowTemp, iColTemp, oldValue, dgvParametersEdit[iColTemp, iRowTemp].Value.ToString()));
                                        break;
                                }
                                break;
                        }
                    }

                    iCol++;
                    if (iCol == sColCount)
                    {
                        iRow++;
                        if (iRow == sRowCount)
                            break;
                        iCol = 0;
                    }
                }

                m_undoManager.EndContext();
                EnableDisableUndoRedoButtons();
                //Set the 'dirty bit' to true
                m_DefChanged_NotAppliedToDriver = true;
                m_DefChanged_NotAppliedToSheet = true;
                //Enable/Disable the Apply PSet/Load Signal Buttons.
                configureButtons();

                listDataGridCells = null;
            }
            catch (Exception exp)
            {
                ThrowErrorMessage("Exception caught in pasteToolStripButton_Click: ", exp.Message);
            }
        }

        private void AccessClipboardMethods(ClipBoardAccessEnum CBAccEnumValue, ref string strCBText, DataObject objCBData)
        {
            int iCounter = 0;

            while (iCounter < CLIPBOARD_ACCESS_COUNTER)
            {
                try
                {
                    switch (CBAccEnumValue)
                    { 
                        case ClipBoardAccessEnum.GETTEXT:                            
                            strCBText = Clipboard.GetText();                            
                            break;
                        case ClipBoardAccessEnum.SETTEXT:                                                        
                            Clipboard.Clear();
                            Clipboard.SetText(strCBText, TextDataFormat.UnicodeText);                            
                            break;
                        case ClipBoardAccessEnum.SETDATAOBJECT:
                            Clipboard.Clear();
                            Clipboard.SetDataObject(objCBData);
                            break;
                    }

                    break;  // If Clipboard access is fine, break the loop
                }
                catch (System.Runtime.InteropServices.ExternalException)
                {
                    System.Threading.Thread.Sleep(CLIPBOARD_ACCESS_DELAY);
                    iCounter++;
                    continue;
                }        
            }            
        }

        /// <summary>
        /// Copies selected cells to Clipboard
        /// </summary>
        /// <returns>Flag to indicate whether the selected cells are copied to Clipboard or not</returns>
        private Boolean CopyCellsToClipBoard()
        {
            if (AllowUserToCopy())
            {
                try
                {
                    // Add the selection to the Clipboard
                    DataObject SelectedCellsData = dgvParametersEdit.GetClipboardContent();

                    // In case a cell is in edit mode, get the corresponding editing control
                    if (SelectedCellsData == null)
                    {
                        String strCell = "", strCellType;
                        DataGridViewCell cellCurrent = dgvParametersEdit.CurrentCell;

                        if (cellCurrent == null)
                            return false;

                        strCellType = cellCurrent.GetType().Name;                        
                        if (strCellType.Equals("DataGridViewTextBoxCell"))
                        {
                            strCell = ((DataGridViewTextBoxEditingControl)dgvParametersEdit.EditingControl).SelectedText;
                        }                        
                        else if (strCellType.Equals("DataGridViewComboBoxCellEx"))
                        {
                            strCell = ((DataGridViewComboBoxEditingControl)dgvParametersEdit.EditingControl).SelectedText;
                        }

                        dgvParametersEdit.EndEdit();

                        // if the 'edit control' does not have any data, fallback to stored data in the 'cell'
                        if (strCell.Length == 0)
                            strCell = cellCurrent.Value != null ? cellCurrent.Value.ToString() : "";

                        if (strCell.Length != 0) AccessClipboardMethods(ClipBoardAccessEnum.SETTEXT, ref strCell, null);
                        else return false;
                    }
                    else
                    {
                        string strTemp = String.Empty;
                        AccessClipboardMethods(ClipBoardAccessEnum.SETDATAOBJECT, ref strTemp, SelectedCellsData);                        
                    }

                    return true;
                }
                catch (System.Runtime.InteropServices.ExternalException e)
                {
                    // The Clipboard could not be accessed. Please try again
                    ShowMessageBox(Teradyne.Igxl.PSetSignalsEditor.Properties.Resources.str104 + e.StackTrace, "Parameter Grid - Copy/Cut");
                }
            }

            return false;
        }

        /// <summary>
        /// Validates the selection for the Copy operation to continue
        /// </summary>
        private bool AllowUserToCopy()
        {            
            try
            {
                // it is possible that the user may click on copy / cut when the focus is with another control
                // in which case, we do not copy any data in the grid.
                //
                // But, sometimes, focus is false when a cell is in edit mode, so allow those to be copied.

                if (m_bUnitTest == false && !(dgvParametersEdit.IsCurrentCellInEditMode || dgvParametersEdit.Focused))
                    return false;

                int iTop = 0, iLeft = 0, iRight = 0, iBottom = 0, iSelectedCellsCount;

                iSelectedCellsCount = dgvParametersEdit.GetCellCount(DataGridViewElementStates.Selected);

                // Selected some cells
                if (iSelectedCellsCount > 0)
                {
                    GetTopLeftCell(ref iTop, ref iLeft);
                    GetBottomRightCell(ref iBottom, ref iRight);

                    if ((((iRight - iLeft) + 1) * ((iBottom - iTop) + 1)) == iSelectedCellsCount)
                        return true;
                    else    // Selection is invalid
                        ShowMessageBox(Teradyne.Igxl.PSetSignalsEditor.Properties.Resources.str109, "Parameter Grid - Copy/Cut");
                }
                else    // Please select cells
                    ShowMessageBox(Teradyne.Igxl.PSetSignalsEditor.Properties.Resources.str110, "Parameter Grid - Copy/Cut");
            }
            catch (Exception Ex)
            {
                ThrowErrorMessage("Exception caught in AllowUserToCopy: ", Ex.Message);
            }

            return false;
        }

        /// <summary>
        /// Get the row and column indices of the top leftmost cell of the selected cells collection
        /// </summary>
        /// <param name="iTop">Row Index of the top leftmost cell</param>
        /// <param name="iLeft">Column Index of the top leftmost cell</param>
        private void GetTopLeftCell(ref int iTop, ref int iLeft)
        {
            try
            {
                DataGridViewSelectedCellCollection collectionSelectedCells = dgvParametersEdit.SelectedCells;

                // Initialize
                iTop = collectionSelectedCells[0].RowIndex;
                iLeft = collectionSelectedCells[0].ColumnIndex;
                for (short sIndex = 0; sIndex < collectionSelectedCells.Count; sIndex++)
                {
                    if (collectionSelectedCells[sIndex].RowIndex < iTop)
                        iTop = collectionSelectedCells[sIndex].RowIndex;

                    if (collectionSelectedCells[sIndex].ColumnIndex < iLeft)
                        iLeft = collectionSelectedCells[sIndex].ColumnIndex;
                }
            }
            catch (Exception e)
            {
                ThrowErrorMessage("Exception caught in GetTopLeftCell: ", e.Message);
            }
        }

        /// <summary>
        /// Get the row and column indices of the bottom rightmost cell of the selected cells collection
        /// </summary>
        /// <param name="iBottom">Row Index of the bottom rightmost cell</param>
        /// <param name="iRight">Column Index of the bottom rightmost cell</param>
        private void GetBottomRightCell(ref Int32 iBottom, ref Int32 iRight)
        {
            try
            {
                DataGridViewSelectedCellCollection collectionSelectedCells = dgvParametersEdit.SelectedCells;

                // Initialize
                iBottom = collectionSelectedCells[0].RowIndex;
                iRight = collectionSelectedCells[0].ColumnIndex;
                for (short sIndex = 0; sIndex < collectionSelectedCells.Count; sIndex++)
                {
                    if (collectionSelectedCells[sIndex].RowIndex > iBottom)
                        iBottom = collectionSelectedCells[sIndex].RowIndex;

                    if (collectionSelectedCells[sIndex].ColumnIndex > iRight)
                        iRight = collectionSelectedCells[sIndex].ColumnIndex;
                }
            }
            catch (Exception e)
            {
                ThrowErrorMessage("Exception caught in GetBottomRightCell: ", e.Message);
            }
        }

        /// <summary>
        /// This Method handles the click of SelectPSets/SelectSignals button in Compare window.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMixedPSetsOrSignals_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();
            ShrunkErrorPane();

            m_Compare_InstrEngine.UpdateHelper(m_GlobalSettings.ComparePanel.NSPCP_MODE);
            m_Compare_InstrEngine.Rebuild();
            HandlePSetOrSignalsButtonClick(txtMixedPinName.Text, cboMixedInstrument.Text,
                        ref m_GlobalSettings.ComparePanel, ref m_GlobalPSetSignalValues.ComparePanel,
                        ref m_CompNspcpList, ref m_CompUiEngine, ref dgvCompareParamEdit);

            for (int col = 0; col < dgvCompareParamEdit.Columns.Count; col++)
                dgvCompareParamEdit.Columns[col].DefaultCellStyle.BackColor = READ_ONLY_COLOR;

        }

        private void CompareNspcps()
        {
            String mainSource = cboSource.Text;
            String compareSource = cboMixedSource.Text;
            int mainRowsCount = dgvParametersEdit.Rows.Count;
            int compareRowsCount = dgvCompareParamEdit.Rows.Count;
            int mainColCount = dgvParametersEdit.Columns.Count;
            int compareColCount = dgvCompareParamEdit.Columns.Count;

            try
            {
                if ((!pnlRight.Visible == true) ||
                    (!mainSource.Equals(compareSource)) ||
                    (mainRowsCount != compareRowsCount) ||
                    (!cboInstrument.Text.Equals(cboMixedInstrument.Text)) ||(mainColCount ==1 || compareColCount ==1))
                    return;

                for (int nRows = PARAM_NAME_COLUMN; nRows < mainRowsCount; nRows++)
                {
                    DataGridViewCell mainParamNameCell = dgvParametersEdit.Rows[nRows].Cells[PARAM_NAME_COLUMN];
                    DataGridViewCell compareParamNameCell = dgvCompareParamEdit.Rows[nRows].Cells[PARAM_NAME_COLUMN];

                    if (mainParamNameCell.Value == null || compareParamNameCell.Value == null)
                        continue;
                    if (mainParamNameCell.Value.Equals(compareParamNameCell.Value))
                    {
                        DataGridViewCell mainParamValueCell = dgvParametersEdit.Rows[nRows].Cells[PARAM_VALUE_COLUMN];
                        DataGridViewCell compareParamVlaueCell = dgvCompareParamEdit.Rows[nRows].Cells[PARAM_VALUE_COLUMN];

                        if (mainParamValueCell.Value == null || compareParamVlaueCell.Value == null)
                            continue;
                        if (!mainParamValueCell.Value.Equals(compareParamVlaueCell.Value))
                            compareParamVlaueCell.Style.BackColor = COMP_COLOR;
                        else
                            compareParamVlaueCell.Style.BackColor = READ_ONLY_COLOR;
                    }
                }
            }
            catch (NullReferenceException)
            {
                //Added for temporary error handling
                MessageBox.Show("The row which is accessed is null.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (IndexOutOfRangeException)
            {
                //Added for temporary error handling
                MessageBox.Show("The index of the data grid view control is out of range.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void cboMixedInstrument_DropDown(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            m_Compare_InstrEngine.UpdateHelper(m_GlobalSettings.ComparePanel.NSPCP_MODE);
            m_Compare_InstrEngine.Rebuild();

            if (!cboMixedInstrument.Text.Equals(DEFAULT_TEXT_INST))
            {
                m_compareOldInstrumentValue = cboMixedInstrument.Text;
            }

            SetInstrumentsForPins(false);
            Cursor.Current = Cursors.Default;
        }



        #region Undo Redo

        #region IUndoable Members

        public bool ResetState(object undoData, bool isUndoing)
        {
            String toVerify, toApply;
            GridUndoInfo info = undoData as GridUndoInfo;

            if (isUndoing)
            {
                toVerify = info.CurrentData;
                toApply = info.OldData;
            }
            else
            {
                toVerify = info.OldData;
                toApply = info.CurrentData;
            }

            if (dgvParametersEdit.Rows.Count <= info.Row || dgvParametersEdit.Columns.Count <= info.Column)
                return false;

            DataGridViewCell c = dgvParametersEdit.Rows[info.Row].Cells[info.Column];
            if (c == null)
                return false;

            if ((c.Value == null && toVerify == null) || (c.Value != null && c.Value.Equals(toVerify)))
            {
                c.Value = toApply;

                // We need to change the selection to show that this cell has been undone
                // or redone
                dgvParametersEdit.ClearSelection();
                c.Selected = true;
                c.Style.ForeColor = GlobalUtils.CELL_CHANGE_COLOR;

                // Inorm to the UIEngine
                UpdateCellData(info.Column);
            }
            else
                return false;

            return true;
        }

        #endregion

        private void UpdateCellData(int colIndex)
        {
            m_DefChanged_NotAppliedToDriver = true;
            m_DefChanged_NotAppliedToSheet = true;
            //Enable/Disable the Apply PSet/Load Signal Buttons.
            configureButtons();

            //... and configure the nspcp column.
            if (colIndex > 0)
            {
                String nspcpName = dgvParametersEdit.Columns[colIndex].Name;
                //Refresh the objects.
                m_UIEngine.CollectGridInformation(ref m_NSPCPList);
                //Get object
                NspcpItem obj = m_NSPCPList[nspcpName];
                //Configure the grid.
                HandleGridConfiguration(obj, ref m_UIEngine, ref m_Main_InstrEngine);
            }
        }

        private void tsbUndo_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();

             
            ShrunkErrorPane();

            if (dgvParametersEdit.IsCurrentCellInEditMode)
            {
                TextBox box = dgvParametersEdit.EditingControl as TextBox;
                if (box != null && box.CanUndo)
                {
                    box.Undo();
                    return;
                }
            }
            else
                m_undoManager.Undo();

            if ((!m_undoManager.CanUndo) && (m_DefChanged_NotAppliedToDriver || m_DefChanged_NotAppliedToSheet))
                m_DefChanged_NotAppliedToDriver = m_DefChanged_NotAppliedToSheet = false; // We have reached the original state
            else
                m_DefChanged_NotAppliedToDriver = m_DefChanged_NotAppliedToSheet = true; // else the user has made some changes

            configureButtons();
            EnableDisableUndoRedoButtons();
        }

        private void tsbRedo_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();

             
            ShrunkErrorPane();

            m_undoManager.Redo();

            m_DefChanged_NotAppliedToDriver = m_DefChanged_NotAppliedToSheet = true;

            configureButtons();
            EnableDisableUndoRedoButtons();
        }

        private void EnableDisableUndoRedoButtons()
        {
            if (dgvParametersEdit.IsCurrentCellInEditMode)
            {
                TextBox box = dgvParametersEdit.EditingControl as TextBox;
                if (box != null)
                {
                    // In focus edit box can be undone only
                    tsbUndo.Enabled = box.CanUndo;
                    UpdateMenuItem(UNDO, UpdateMenuItemEnum.ENABLED, tsbUndo.Enabled);  // Update menu item
                }
                else
                {
                    // Combos and other controls cannot be undone inplace
                    tsbUndo.Enabled = false;
                    UpdateMenuItem(UNDO, UpdateMenuItemEnum.ENABLED, false);  // Update menu item
                }

                tsbRedo.Enabled = false;
                UpdateMenuItem(REDO, UpdateMenuItemEnum.ENABLED, false);  // Update menu item
            }
            else
            {
                tsbRedo.Enabled = m_undoManager.CanRedo;
                tsbUndo.Enabled = m_undoManager.CanUndo;
                UpdateMenuItem(REDO, UpdateMenuItemEnum.ENABLED, tsbRedo.Enabled);  // Update menu item
                UpdateMenuItem(UNDO, UpdateMenuItemEnum.ENABLED, tsbUndo.Enabled);  // Update menu item
            }
        }
        #endregion

        private void dgvParametersEdit_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            for (int i = 1; i < dgvParametersEdit.Columns.Count; i++)
            {
                dgvParametersEdit.Columns[i].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            }
            CompareNspcps();
            EnableDisableUndoRedoButtons();

            if (dgvParametersEdit[0, e.RowIndex].Value!=null && dgvParametersEdit[0, e.RowIndex].Value.Equals("Signal Definition File"))
            {
                dgvParametersEdit.EndEdit(DataGridViewDataErrorContexts.Commit);
            }
        }

        /// <summary>
        /// This method launches the wavescope in the TDE.
        /// </summary>
        /// <param name="WaveDefName"></param>
        private void ShowWaveScope(String WaveDefName)
        {
            DMGRWAVEDEFINITIONSHEETLIB.DMWaveDefinition DMWaveDef = null;
            DMGRLib.IDMSheets sheets;
            try
            {
                sheets = MyServices.DmgrServiceCOM.ContextMgr.FindSheetsByType(GlobalUtils.DMGR_SHEET_TYPE_PSETSSHEET) as DMGRLib.IDMSheets;
                
                if(sheets == null || sheets.Count == 0) throw new Exception("No WaveDef Sheet Defined");

                foreach (DMGRLib.IDMSheet item in sheets)
                {
                    DMGRWAVEDEFINITIONSHEETLIB.DMWavedefinitionSheet DMWaveDefSheet = item as DMGRWAVEDEFINITIONSHEETLIB.DMWavedefinitionSheet;

                    if (DMWaveDefSheet == null) continue;

                    if (DMWaveDefSheet.WaveDefinitions.Has(WaveDefName))
                    {
                //Get the WaveDef from the WaveDefinition Sheet
                DMWaveDef = DMWaveDefSheet.WaveDefinitions.Find(WaveDefName);
                        break;
                    }
                }

                if (DMWaveDef == null)
                {
                    MessageBox.Show("Wavedefinition of name " + WaveDefName + " does not exist on the active Wave Definitions Sheet.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }

                m_TDE.PerformDefaultVerb(DMWaveDef); //Launch the tool.
                
            }
            catch (COMException)
            {
                MessageBox.Show("The WaveDefinition name specified does not exist on the WaveDef sheet.", this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception exp)
            {
                MessageBox.Show("Error in launching WaveScope and WaveDesigner : " + exp.Message, this.Name, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            //Launch the WaveScope with the required settings           
        }

        //Inorder to show my context menu strip when right clicking
        private void dgvParametersEdit_EditingControlShowing(object sender, DataGridViewEditingControlShowingEventArgs e)
        {
            if (m_UIEngine.IsInitialized && m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.signal && m_waveDefSelected)
            {
                e.Control.ContextMenuStrip = contextMenuStripWaveDef;
            }
            else
            {
                e.Control.ContextMenuStrip = null;
            }

            EnableDisableUndoRedoButtons();
        }

        //Upon clicking on edit.
        private void contextMenuItemEdit_Click(object sender, EventArgs e)
        {
            if (m_currWaveDefName != null)
            {
                ShowWaveScope(m_currWaveDefName);
            }
        }

        private void rbnMixedSheetDefinition_CheckedChanged(object sender, EventArgs e)
        {
            if (rbnMixedSheetDefinition.Checked)
            {
                m_Compare_InstrEngine.UpdateHelper(m_GlobalSettings.ComparePanel.NSPCP_MODE);
                m_Compare_InstrEngine.Rebuild();
                HandleSourceChanged(GlobalUtils.DATA_SOURCE.SHEET, ref m_GlobalSettings.ComparePanel, ref m_CompNspcpList,
                                    ref m_CompUiEngine, ref m_GlobalPSetSignalValues.ComparePanel,
                                    txtMixedPinName.Text, cboMixedInstrument.Text, ref rbnMixedSheetDefinition,
                                    ref rbnMixedCurrentDriverDefinition, ref m_Compare_InstrEngine);
            }
        }

        private void rbnMixedCurrentDriverDefinition_CheckedChanged(object sender, EventArgs e)
        {
            if (rbnMixedCurrentDriverDefinition.Checked)
            {
                m_Compare_InstrEngine.UpdateHelper(m_GlobalSettings.ComparePanel.NSPCP_MODE);
                m_Compare_InstrEngine.Rebuild();
                HandleSourceChanged(GlobalUtils.DATA_SOURCE.DRIVER, ref m_GlobalSettings.ComparePanel, ref m_CompNspcpList,
                                    ref m_CompUiEngine, ref m_GlobalPSetSignalValues.ComparePanel, 
                                    txtMixedPinName.Text, cboMixedInstrument.Text, ref rbnMixedSheetDefinition,
                                    ref rbnMixedCurrentDriverDefinition, ref m_Compare_InstrEngine);
            }
        }

        private void txtPinName_KeyPress(object sender, KeyPressEventArgs e)
        {
            m_Main_InstrEngine.UpdateHelper(m_GlobalSettings.MainPanel.NSPCP_MODE);
            if (txtPinName.Text.Equals(""))
            {
                cboInstrument.Text = "";
                return;
            }
            if (!e.KeyChar.Equals('\r'))
            {
                return;
            }

            SetInstrumentsForPins(true);
        }

        private void txtMixedPinName_KeyPress(object sender, KeyPressEventArgs e)
        {
            m_Compare_InstrEngine.UpdateHelper(m_GlobalSettings.ComparePanel.NSPCP_MODE);
            if (txtMixedPinName.Text.Equals(""))
            {
                cboMixedInstrument.Text = "";
                return;
            }
            if (!e.KeyChar.Equals('\r'))
            {
                return;
            }

            SetInstrumentsForPins(false);
        }

        private void txtMixedPinName_MouseClick(object sender, MouseEventArgs e)
        {
            if (m_bEntertxtMixedPinName == true)
                txtMixedPinName.SelectAll();
            m_bEntertxtMixedPinName = false;
        }

        private void txtMixedPinName_Enter(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();

             
            ShrunkErrorPane();

            m_bEntertxtMixedPinName = true;
        }

        private void txtMixedPinName_Leave(object sender, EventArgs e)
        {
            if (txtMixedPinName.Text.Equals(""))
            {
                txtMixedPinName.Text = DEFAULT_TEXT_PIN;
            }

            if (m_bPinNameChanged)
            {
                SetInstrumentsForPins(false);
                m_bPinNameChanged = false;
            }
        }
        private void dgvParametersEdit_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex == -1)
            {
                if (dgvParametersEdit.IsCurrentCellInEditMode)
                    dgvParametersEdit.EndEdit();

                return;
            }
            
            if (e.ColumnIndex > 0 && (!dgvParametersEdit.IsCurrentCellInEditMode))
            {
                if (this._colNum != e.ColumnIndex)
                {
                    this._colNum = e.ColumnIndex;

                    string colName = dgvParametersEdit.Columns[e.ColumnIndex].Name;
                    NspcpItem tempObj = m_NSPCPList[colName];

                    ReConfigureGrid(tempObj, ref m_Main_InstrEngine);
                }
            }

            // To enable single-click dropdown of combobox
            if ((dgvParametersEdit.CurrentCell == null) || (dgvParametersEdit.CurrentCell.ReadOnly))
                return;

            if (dgvParametersEdit.CurrentCell.EditType.ToString().EndsWith("DataGridViewComboBoxEditingControlEx"))
            {
                dgvParametersEdit.BeginEdit(true);
                DataGridViewComboBoxEditingControl control = dgvParametersEdit.EditingControl as DataGridViewComboBoxEditingControl;
                control.DroppedDown = true;
            }
            
        }

        private void dgvCompareParamEdit_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex == -1)
            {
                if (dgvCompareParamEdit.IsCurrentCellInEditMode)
                    dgvCompareParamEdit.EndEdit();

                return;
            }
            UpdateMenuItem(COPY, UpdateMenuItemEnum.ENABLED, true);
            copyToolStripButton.Enabled = true;

            //// To enable single-click dropdown of combobox
            if ((dgvCompareParamEdit.CurrentCell == null) || (dgvCompareParamEdit.CurrentCell.ReadOnly))
                return;
            if (dgvCompareParamEdit.CurrentCell.EditType.ToString().EndsWith("DataGridViewComboBoxEditingControlEx"))
            {
                dgvCompareParamEdit.BeginEdit(true);
                DataGridViewComboBoxEditingControl control = dgvCompareParamEdit.EditingControl as DataGridViewComboBoxEditingControl;
                control.DroppedDown = true;
            }

        }


        #region Popup the Delete Menu Item on Main Grid

        // <summary>
        /// This method is used to Popup MenuItem over the DataGridView Control
        /// And Perform the Delete operation..
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="eventArgs"></param>
        private void dgvParametersEdit_MouseUp(object sender, MouseEventArgs eventArgs)
        {
            try
            {
                ///Gets  the cursor Hit Information ( it has all the info like columnIndex,
                ///RowIndex, RowX and RowY...
                m_hitInfo = dgvParametersEdit.HitTest(eventArgs.X, eventArgs.Y);

                ///Checks whether the ColumnIndex is Less than or equal to 0.
                ///Because, avoids the null reference exception..
                if (m_hitInfo.ColumnIndex <= 0)
                    return;

                if (m_hitInfo.RowIndex < 0)
                {
                    if (eventArgs.Button == MouseButtons.Right)
                    {
                        // For now we only allow one item to be deleted at a time, so 
                        // first clear any existing selection
                        dgvParametersEdit.ClearSelection();

                        string columnName = dgvParametersEdit.Columns[m_hitInfo.ColumnIndex].Name;
                        int nRowCount = dgvParametersEdit.Rows.Count;   //Get the rowcount.
                        //we need to mark all the rows for the selected column.
                        dgvParametersEdit[0, 0].Selected = false;
                        while (nRowCount > 0)
                        {
                            dgvParametersEdit[m_hitInfo.ColumnIndex, nRowCount - 1].Selected = true;
                            nRowCount--;
                        }
                        
                        contextMenuStripColumn.Show((Control)sender, eventArgs.X, eventArgs.Y);
                    }
                }
                else if(m_hitInfo.RowIndex == 1)
                {
                    if (m_UIEngine.IsInitialized && m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.signal && m_hitInfo.RowIndex != GlobalUtils.ERR_VALUE)
                    {
                        if (dgvParametersEdit[0, m_hitInfo.RowIndex].Value.Equals(GlobalUtils.WAVEDEF_PARAM_NAME))
                        {
                            m_waveDefSelected = true;
                            m_currWaveDefName = (String)dgvParametersEdit[m_hitInfo.ColumnIndex, m_hitInfo.RowIndex].Value;
                            //Check if it is a right click
                            if (eventArgs.Button == MouseButtons.Right)
                            {
                                //Show the context menu strip at that location.
                                contextMenuStripWaveDef.Show((Control)sender, eventArgs.X, eventArgs.Y);
                            }
                        }
                        else
                        {
                            m_waveDefSelected = false;
                        }
                    }

                }
            }
            catch (Exception exp)
            {
                Trace.WriteLine(exp.Message);
            }
        }


        /// <summary>
        /// purpose of this method is used to delete the grid from the 
        /// DataGridView and also delete the item from NspcpItem Collection 
        /// and finally it updates
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DeleteAndUpdateGrid(object sender, EventArgs e)
        {
            string alertMessage = "Are you sure you want to remove the selected Item?";
            
            try
            {
                DialogResult result = MessageBox.Show(alertMessage, GlobalUtils.MESSAGEBOX_CAPTION, MessageBoxButtons.YesNo, MessageBoxIcon.Information);

                //Checks the result of the MessageBox. Whether Yes or No
                if (result.CompareTo(DialogResult.Yes) == 0)
                {
                    if(dgvParametersEdit.Columns.Count == 2)
                    {
                        ClearMainParameterGrid();
                    }
                    else
                    {
                        DataGridViewColumn column = dgvParametersEdit.Columns[m_hitInfo.ColumnIndex];
                        dgvParametersEdit.Columns.Remove(column);
                        dgvParametersEdit.Update();
                        dgvParametersEdit[0, 0].Selected = false;
                        m_NSPCPList.Remove(column.Name);
                       
                        if (m_GlobalSettings.MainPanel.NSPCP_MODE == GlobalUtils.NspcpType.pset)
                        {
                                m_GlobalPSetSignalValues.MainPanel.PSetValues.Remove(column.Name);
                        }
                        else
                        {
                                m_GlobalPSetSignalValues.MainPanel.SignalValues.Remove(column.Name);
                        }
                    }
                                       
                    if (m_NSPCPList.Count > 0)
                    {
                        tsmiActionsSetDriver.Enabled = false;
                        tsmiActionsUpdateSheet.Enabled = false;
                        tsmiActionsApplyPSet.Enabled = true;
                        tsmiActionsLoadSignals.Enabled = false;
                        UpdateMenuItem(SETDRIVER, UpdateMenuItemEnum.ENABLED, false);  // Update menu item
                        UpdateMenuItem(UPDATESHEET, UpdateMenuItemEnum.ENABLED, false);  // Update menu item
                        UpdateMenuItem(APPLYPSET, UpdateMenuItemEnum.ENABLED, true);  // Update menu item
                        UpdateMenuItem(LOADSIGNALSETTINGS, UpdateMenuItemEnum.ENABLED, false);  // Update menu item
                    }
                    configureCopyCodeButton();
                    configureButtons();
                }
                else
                {
                    ///If Result == MessageButtons.NO or !( Result == MessageButrtons.YES) Then 
                    ///    Need to unselect the Selected Column..

                    string columnName = dgvParametersEdit.Columns[m_hitInfo.ColumnIndex].Name;
                    int nRowCount = dgvParametersEdit.Rows.Count;   //Get the rowcount.
                    //we need to mark all the rows for the selected column.
                    while (nRowCount > 0)
                    {
                        dgvParametersEdit[0, 0].Selected = false;
                        //unselect the selected column
                        dgvParametersEdit[m_hitInfo.ColumnIndex, nRowCount - 1].Selected = false;
                        nRowCount--;
                    }
                }
                
            }
            catch (Exception EX)
            {
                Trace.WriteLine(EX.Message);
            }
        }

        
       
        #endregion  Popup the Delete Menu Item on Main Grid

        #region Proxy event handlers

        /// <summary>
        /// Proxy resize event handler
        /// Resize the PSSE as the display proxy resizes (i.e. as user resizes the tool window in TDE)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Proxy_Resize(object sender, EventArgs e)
        {
            // PSSE expand vertically when operated in full screen mode.
            this.Height = Proxy.Height;     //set height            
        }

        #endregion

        private void dgvParametersEdit_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            // ignore row, column headers double click
            if (e.RowIndex < 0)
                return;
            if (e.ColumnIndex == 0)
                return;
            // ignore read only cells
            if (dgvParametersEdit[e.ColumnIndex, e.RowIndex].ReadOnly)
                return;
            dgvParametersEdit.BeginEdit(false);
        }

        private void applyAllToolStripButton_Click(object sender, EventArgs e)
        {
            UpdateMenuItem(APPLYTOALLSITES, UpdateMenuItemEnum.CHECKED, applyAllToolStripButton.Checked);
        }

        /// <summary>
        /// When the user click's Update Tester on the display, set the driver
        /// with the parameter values from the grid.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUpdateTester_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();
            HandleSetDriver();
            CompareNspcps();
        }

        /// <summary>
        /// When the user click's Update Sheet on the display, update the pset/signal
        /// with the parameter values from the grid.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUpdateSheet_Click(object sender, EventArgs e)
        {
            //Clear all existing errors.
            m_UIEngine.ClearAll();
            HandleUpdateSheet();
            MyServices.CoreLanguageServiceCOM.DidExecute = false;
        }

        /// <summary>
        /// Handle the event when the user clicks the Apply PSet
        /// or Load Signal Settings button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnApplyPSet_Click(object sender, EventArgs e)
        {
            if (this._rcInfo != null && (!this._rcInfo.IsEnabled))
            {
                return;
            }
            //Clear all existing errors.
            m_UIEngine.ClearAll();
            HandleApplyLoad();
        }

        private void helpToolStripButton_Click(object sender, EventArgs e)
        {
            m_TDE.ShowHelp(this.Handle.ToInt32(), "TDE.chm", TDECOMMONSRVLib.TDEHelpCommand.tdeHelpContext, 13000);
        }
    
        private void dgvParametersEdit_Enter(object sender, EventArgs e)
        {
            //Disable Cut and Copy, but not the Paste. Only disable paste if clipboard is empty.According to the DataGrid
            UpdateMenuItemStatus(true);
        }

        /// <summary>
        /// Handles the Leave event of the dgvParametersEdit control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The instance containing the event data.</param>
        private void dgvParametersEdit_Leave(object sender, EventArgs e)
        {
            UpdateMenuItemsOnGridLeave(dgvParametersEdit);
        }


        /// <summary>
        /// Updates the menu items on grid leave.
        /// </summary>
        private void UpdateMenuItemsOnGridLeave(NspcpItemsGridView nspcpItemsGridView)
        {
            DataGridViewSelectedCellCollection cellCollection = nspcpItemsGridView.SelectedCells;

            if (cellCollection != null && cellCollection.Count > 0)
            {
                foreach (DataGridViewCell cellItem in cellCollection)
                {
                    if (cellItem.Selected)
                        cellItem.Selected = false;
                }
            }

            //Disable Cut and Copy, but not the Paste. Only disable paste if clipboard is empty.According to the DataGrid
            UpdateMenuItemStatus(false);
        }
        
        /// <summary>
        /// Wndproc
        /// </summary>
        /// <param name="m"></param>
        protected override void WndProc(ref Message m)
        {
            DateTime startTime = DateTime.Now;            
            base.WndProc(ref m);
            if (!m_bTimer)
            {
                endTime = DateTime.Now;
                endTime = endTime.AddMilliseconds(50);
                m_bTimer = true;
            }

            if (IsChecksDuration(startTime, endTime))
                CheckAndUpdatePasteMenuItem();
        }

        /// <summary>
        /// Determines whether the duration is grater than the specified start time or not
        /// </summary>
        /// <param name="startTime">The start time.</param>
        /// <param name="endTime">The end time.</param>
        /// <returns>
        /// 	true, if checks duration is grater than or equal to the specified start time; otherwise, false.
        /// </returns>
        private bool IsChecksDuration(DateTime startTime, DateTime endTime)
        {
            if (startTime.Millisecond < endTime.Millisecond)
                return false;
            else if (startTime.Millisecond >= endTime.Millisecond)
                return true;

            return false;         
        }

        /// <summary>
        /// Handles the Leave event of the dgvCompareParamEdit control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The instance containing the event data.</param>
        private void dgvCompareParamEdit_Leave(object sender, EventArgs e)
        {
            UpdateMenuItemsOnGridLeave(dgvCompareParamEdit);
        }
        private void cboMixedInstrument_MouseClick(object sender, MouseEventArgs e)
        {
            if (!cboMixedInstrument.Text.Equals(DEFAULT_TEXT_INST))
            {
                m_compareOldInstrumentValue = cboMixedInstrument.Text;
            }

        }

        private void txtPinName_TextChanged(object sender, EventArgs e)
        {
            m_bPinNameChanged = true;
        }

        private void txtMixedPinName_TextChanged(object sender, EventArgs e)
        {
            m_bPinNameChanged = true;
        }

        private void masterCheckBox_Click(object sender, EventArgs e)
        {
            //if (_siteValidated == false)
            //{
            //    masterCheckBox.Checked = !masterCheckBox.Checked;
            //    return;
            //}

            _syncMasterSite = masterCheckBox.Checked;
            if (_syncMasterSite)
            {
                if (_siteValidated == false)
                {
                    tscboSite.BackColor = GetDefaultWindowColor();
                    _siteValidated = true;
                }
                tscboSite.SelectedIndex = m_TDE.Site;
            }
        }

        private Color GetDefaultWindowColor()
        {
            return Color.FromKnownColor(System.Drawing.KnownColor.Window);
        }

        private Color GetErrorWindowColor()
        {
            Color temp = System.Drawing.Color.FromArgb(ERROR_WINDOW_COLOR);
            return System.Drawing.Color.FromArgb(255, temp);
        }

        /// <summary>Applies changed site number</summary>
        /// <remarks>
        /// 
        /// 
        /// </remarks>
        /// <param name=""></param>
        /// <returns>void</returns>
        private void ApplySite()
        {
            if (prevSiteSelectionIdx == tscboSite.SelectedIndex)
                return;

            //Clear all existing errors.
            m_UIEngine.ClearAll();

            ShrunkErrorPane();
            //handle change of site
            HandleSiteActivation();
        }

        private void tscboSite_Validating(object sender, CancelEventArgs e)
        {
            int currentSelection = -1;

            //site is valid - complete validation (see tscboSite_Validated)
            if (IsSiteValid(ref currentSelection) == true)
            {
                e.Cancel = false;
                currentSiteSelectionIdx = currentSelection;
            }
            else
            {
                e.Cancel = true;
            }
        }

        private void tscboSite_Validated(object sender, EventArgs e)
        {
            //change of tscboSite.SelectedIndex fires tscboSite_SelectedIndexChanged event (see the handler) 
            tscboSite.SelectedIndex = currentSiteSelectionIdx;
        }

        /// <summary>Checks if string entered into Site ComboBox contains a valid site number</summary>
        /// <remarks>
        /// 
        /// 
        /// </remarks>
        /// <param name="currentSelection">ref to the index in SiteArray</param>
        /// <returns>true if Site is a valid site number, false otherwise</returns>
        private bool IsSiteValid(ref int currentSelection)
        {
            int Index = -1;
            tscboSite.BackColor = GetDefaultWindowColor();
            _siteValidated = false;

            //non digit chars exist
            if (CheckSite(tscboSite.Text, ref Index) == false)
            {
                tscboSite.Focus();
                tscboSite.BackColor = GetErrorWindowColor();
                tscboSite.SelectionLength = 1;
                if (Index == -1)
                {
                    Index = 0;
                    tscboSite.SelectionLength = tscboSite.Text.Length;
                }
                tscboSite.SelectionStart = Index;
                return false;
            }

            //when CheckSite returns true currentSelection is set to the index in site array
            currentSelection = Index;
            _siteValidated = true;

            return true;

        }

        /// <summary>Checks if string Site contains a valid site number</summary>
        /// <remarks>
        /// 
        /// 
        /// </remarks>
        /// <param name="String">String containing user's input</param>
        /// <param name="index">reference to the 1st non-digit character in the string</param>
        /// <returns>true if Site is a valis site number, false otherwise</returns>
        private bool CheckSite (String Site, ref int index)
        {

            Site = Site.Trim();
            if (Site.Length == 0)
                return false;

            index = 0;
            //loop is used because we need the index of non-digit char in the string
            foreach (char c in Site)
            {
                //if non-digit character found return false
                if (char.IsNumber(c)==false)
                    return false;
                index++;
            }

            System.Int16[] sitelist = (System.Int16[])m_TDE.SiteList;
            index = 0;
            //check if Site is in the array of Starting sites
            foreach (System.Int16 site in sitelist)
            {
                if (site == Convert.ToInt16(Site))
                {
                    //Site found in site array: position in the array = index (reference) 
                    return true;
                }
                index++;
            }

            //Site not found in site array
            index = -1;
            return false;
        }

        private void tscboSite_KeyUp(object sender, KeyEventArgs e)
        {
            if (_siteValidated && e.KeyData == Keys.Enter)
            {
                //put focus away of ComboBox
                this.GetNextControl(toolStrip1,true).Focus();
                if ( tscboSite.SelectedIndex != m_TDE.Site )
                {
                    //uncheck Master CheckBox when user types anything into site ComboBox 
                    _syncMasterSite = false;
                    masterCheckBox.Checked = _syncMasterSite;
                }

                tscboSite.SelectedIndex = currentSiteSelectionIdx;
                return;
            }

            int currentSelection = -1;
            if (IsSiteValid(ref currentSelection) == true)
            {
                currentSiteSelectionIdx = currentSelection;
            }
        }

        private void tscboSite_Leave(object sender, EventArgs e)
        {
            //string is still not validated: return focus back to the control
            if (_siteValidated == false)
            {
                tscboSite.Focus();
            }
        }
        /// <summary>
        /// Show Resource conflict toolt tip when user 
        /// hovers the mouse on top of load settings button
        /// </summary>
        /// <param name="sender">sender object</param>
        /// <param name="e">event args</param>
        private void btnApplyPSet_MouseHover(object sender, EventArgs e)
        {
            _isLoadSettingButtonHover = true;

            ShowResourceConflict();
        }
        /// <summary>
        /// Removes the mouse hover state, user needs to manualy close the
        /// Resource conflict tool tip.
        /// </summary>
        /// <param name="sender">sender object</param>
        /// <param name="e">event args</param>
        private void btnApplyPSet_MouseLeave(object sender, EventArgs e)
        {
            _isLoadSettingButtonHover = false;
        }

        #region Site Manager Wrapper

        /// <summary>
        /// This method should perform the following activities
        /// 1. set the current flowdomain which was selected by user in the FlowDomain selector
        /// 2. return the site manager  
        /// </summary>
        /// <returns>returns the site manager</returns>
        private tlExecSiteMgr SiteWrapper
        {
            get
            {
                if (m_siteManager == null)
                {
                    m_siteManager = MyServices.CoreLanguageServiceCOM.Exec.Sites;
                }

                // To set the active flowdomain which was selected by user in the FlowDomain selector.
                m_TDE.SetToolFlowDomain();
                
                return m_siteManager;
            }
        }
        #endregion

            
    
    }
}